/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package edu.cmu.taghelpertools.experiments;


import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

import edu.cmu.taghelpertools.io.Logger;
import edu.cmu.taghelpertools.io.Reader;


public class MPQACollector {

    public static String ANNOTATIONS = "D:/mpqa/man_anns/";
    public static String DATA = "D:/mpqa/docs/";
    public static String SENTENCES_FILE = "gatesentences.mpqa";
    public static String ANNOT_FILE = "gateman.mpqa";


    private ArrayList sentenceList;

    public static void main(String args[]) {
        MPQACollector mpqa = new MPQACollector();
        mpqa.run();
    }

    public MPQACollector() {
        sentenceList = new ArrayList();
    }

    public void run() {

        File rootAnnotDir = new File(MPQACollector.ANNOTATIONS);

        String[] dateDirs = rootAnnotDir.list();

        for (int i = 0; i < dateDirs.length; i++) {
            File dateDir = new File(rootAnnotDir.getAbsolutePath() + "/" + dateDirs[i]);
            String[] annotDirs = dateDir.list();
            for (int j = 0; j < annotDirs.length; j++) {
                File annotDir = new File(dateDir.getAbsolutePath() + "/" + annotDirs[j]);
                File sentFile = new File(annotDir + "/" + MPQACollector.SENTENCES_FILE);
                File annotFile = new File(annotDir + "/" + MPQACollector.ANNOT_FILE);
                this.processSentFile(sentFile, annotFile, dateDir.getName(), annotDir.getName());
            }
        }

        Logger logger = new Logger("subjectivity_data.txt");
        Iterator sentIter = sentenceList.iterator();
        while (sentIter.hasNext()) {
            Sentence sent = (Sentence) sentIter.next();
            if (sent.isSubjective) {
                logger.writeLine("SUBJECTIVE," + '"' + sent.getText().replaceAll("\n", " ").replaceAll("" + '"', "" + '"' + '"') + '"');
            } else {
                logger.writeLine("OBJECTIVE," + '"' + sent.getText().replaceAll("\n", " ").replaceAll("" + '"', "" + '"' + '"') + '"');
            }
            System.out.println(sent);
        }
        logger.close();
    }

    public void label(Sentence sent) {

        boolean isSubjective = false;

        boolean isGateOn = false;
        boolean isOnlyFactiveNo = false;
        boolean isOverallStrenghGood = false;
        boolean containsNotSignificantParticular = false;
        boolean containsGateExpSubj = false;
        boolean stengthNotLow = false;
        Iterator annotIter = sent.getAnnotationsIter();
        while (annotIter.hasNext()) {
            String key = annotIter.next().toString();
            String value = sent.getAnnotationForKey(key);

            if (key.equals("GATE_on")) {
                isGateOn = true;
            }
            if (key.equals("onlyfactive")) {
                if (value.equals("no")) {
                    isOnlyFactiveNo = true;
                }
            }
            if (key.equals("overall-strength")) {
                if ((!value.equals("low")) && (!value.equals("neutral"))) {
                    isOverallStrenghGood = true;
                }
            }
            if (key.equals("not-significant-particular")) {
                containsNotSignificantParticular = true;
            }
            if (key.equals("GATE_expressive-subjectivity")) {
                containsGateExpSubj = true;
            }
            if (key.equals("strength")) {
                if (!value.equals("low")) {
                    stengthNotLow = true;
                }
            }
        }

        isSubjective = (isGateOn && isOnlyFactiveNo && isOverallStrenghGood & (!containsNotSignificantParticular)) || (containsGateExpSubj && stengthNotLow);

        sent.isSubjective = isSubjective;
    }


    public void processSentFile(File sentFile, File annotFile, String date, String id) {
        Reader reader = new Reader(sentFile.getAbsolutePath());
        String line = "";
        while ((line = reader.readLine()) != null) {
            System.out.println(line);
            StringTokenizer st = new StringTokenizer(line);
            st.nextToken();
            String offsetString = st.nextToken();
            String[] offsets = offsetString.split(",");
            int start = Integer.parseInt(offsets[0]);
            int end = Integer.parseInt(offsets[1]);
            Sentence sent = new Sentence(sentFile.getAbsolutePath(), start, end);
            Iterator annotIter = this.processAnnotFile(annotFile, start, end);
            while (annotIter.hasNext()) {
                Annotation annot = (Annotation) annotIter.next();
                sent.addAnnotation(annot.getKey(), annot.getValue());
            }
            this.getTextForSentence(sent, date, id);
            this.label(sent);
            this.sentenceList.add(sent);
        }
    }

    public void getTextForSentence(Sentence sent, String date, String id) {
        File file = new File(MPQACollector.DATA + "/" + date + "/" + id);
        try {
            RandomAccessFile raf = new RandomAccessFile(file, "r");
            byte[] sentBytes = new byte[sent.getEnd() - sent.getStart()];
            try {
                raf.skipBytes(sent.getStart());
                raf.readFully(sentBytes, 0, sentBytes.length);
                String sentText = new String(sentBytes);
                sent.setText(sentText);
            } catch (IOException e) {
                e.printStackTrace();
                System.exit(0);
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            System.exit(0);
        }
    }

    public Iterator processAnnotFile(File annotFile, int refStart, int refEnd) {
        ArrayList list = new ArrayList();
        Reader reader = new Reader(annotFile.getAbsolutePath());
        String line = "";
        while ((line = reader.readLine()) != null) {
            line = line.trim();
            if (line.length() > 0) {
                if (!line.startsWith("#")) {
                    StringTokenizer st = new StringTokenizer(line);
                    st.nextToken();
                    String offsetString = st.nextToken();
                    String[] offsets = offsetString.split(",");
                    int start = Integer.parseInt(offsets[0]);
                    int end = Integer.parseInt(offsets[1]);
                    if ((start >= refStart) && (end <= refEnd)) {
                        while (st.hasMoreTokens()) {
                            String token = st.nextToken();
                            if (token.equals("GATE_on")) {
                                list.add(new Annotation("GATE_on", ""));
                            }
                            if (token.indexOf("=") != -1) {
                                String[] keyvalue = token.split("=");
                                String key = keyvalue[0];
                                String value = keyvalue[1];
                                value = value.replaceAll("" + '"', "");
                                list.add(new Annotation(key, value));
                            }
                        }
                    }
                }
            }
        }
        return list.iterator();
    }


    public class Sentence {

        private String text;
        private String fileAbsolutePath;
        private int start;
        private int end;
        private boolean isSubjective;
        private HashMap annotations;


        public Sentence(String fileAbsolutePath, int start, int end) {
            this.fileAbsolutePath = fileAbsolutePath;
            this.start = start;
            this.end = end;
            isSubjective = false;
            annotations = new HashMap();
            text = "";
        }

        public String toString() {
            String result = "\n" + "--------------------" + "\n";
            result += "FILENAME = " + fileAbsolutePath + "\n";
            result += "(START,END) = (" + start + "," + end + ")\n";
            result += "TEXT =" + text + "\n";
            Iterator annotIter = annotations.keySet().iterator();
            while (annotIter.hasNext()) {
                String key = annotIter.next().toString();
                String value = this.getAnnotationForKey(key);
                result += key + " =" + value + "\n";
            }
            result += "SUBJECTIVE =" + isSubjective + "\n";
            return result;
        }


        public String getFileAbsolutePath() {
            return fileAbsolutePath;
        }

        public int getStart() {
            return start;
        }

        public int getEnd() {
            return end;
        }

        public void setText(String text) {
            this.text = text;
        }

        public String getText() {
            return text;
        }

        public Iterator getAnnotationsIter() {
            return annotations.keySet().iterator();
        }

        public String getAnnotationForKey(String key) {
            return annotations.get(key).toString();
        }

        public void addAnnotation(String key, String value) {
            annotations.put(key, value);
        }
    }

    public class Annotation {
        private String key;
        private String value;

        public Annotation(String key, String value) {
            this.key = key;
            this.value = value;
        }

        public String getKey() {
            return key;
        }

        public String getValue() {
            return value;
        }
    }


}
