/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package edu.cmu.taghelpertools.gui;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import edu.cmu.taghelpertools.middle_layer.*;
import edu.cmu.taghelpertools.middle_layer.extractor.FeatureSpaceMapBuilder;
import edu.cmu.taghelpertools.tokenizer.Tokenizer;
import edu.cmu.taghelpertools.gui.TestModelFrame.Runn;
import edu.cmu.taghelpertools.gui.extractor.*;
import edu.cmu.taghelpertools.io.Reader;
import java.util.*;
import weka.gui.PropertyPanel;
import weka.gui.GenericObjectEditor;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeSupport;

import java.io.File;

public class OptionsFrame extends JFrame implements ActionListener {

	//changing modifiers from public to private, by Hao-Chuan
//    private final static int DEFAULT_NUM = OptionsModel.DEFAULT_NUM;
//    private final static double DEFAULT_PERCENT = OptionsModel.DEFAULT_PERCENT;
//    private final static double DEFAULT_THRESH = OptionsModel.DEFAULT_THRESH;

    private ButtonGroup stopwordBG;
    private JRadioButton removeStopwordYesRB;
    private JRadioButton stemNoRB;
    private JRadioButton removeStopwordNoRB;
    private JRadioButton stemYesRB;
    private ButtonGroup stemBG;
    private ButtonGroup itterBG;

    private JCheckBox posBigramCheckBox;
    private JCheckBox lineLengthCheckBox;
    private JCheckBox containsNonStopwordCheckBox;
    private JCheckBox unigramCheckBox;    

    private JCheckBox bigramsCheckBox;
    private JCheckBox punctCheckBox;
    private JCheckBox removeRareFeaturesCheckBox;
    private JCheckBox doSelfTrainCheckBox;
    private JLabel stopwordLabel;
    private JLabel stemLabel;
    private JLabel classifierLabel;
    private JLabel fileLabel;
    private JLabel thresholdLabel;
//    private JComboBox classifierComboBox;
    private JButton okButton;
    private JTextField frequencyThreshold;

    private OptionsModel optionsModel;
    private JRadioButton fixedNumRB;
    private JRadioButton percentRB;
    private JRadioButton thresholdRB;
    private JRadioButton lastRBCalled;
    private JTextField numTextField;
    private JLabel numLabel;
    private JTextField percentTextField;
    private JLabel percentLabel;
    private JTextField threshTextField;
    private JLabel threshLabel;
    private JTabbedPane tabbedPane;

    //options for fromatting, by Hao-Chuan Wang
    private ButtonGroup formatBG;
    private JRadioButton formatWekaRB;
    private JRadioButton formatMTRB; 
    private JLabel formatLabel; 
    
    private ButtonGroup evalBG;
    private JRadioButton evalYesRB;
    private JRadioButton evalNoRB; 
    private JLabel evalLabel; 
    
    //options for languages-specific stuffs, by Hao-Chuan Wang
    private ButtonGroup langTokenizerBG;
    private JRadioButton langTokenizerEnglishRB;
    private JRadioButton langTokenizerGermanRB;
    private JRadioButton langTokenizerChineseRB;
    private JRadioButton langTokenizerSpanishRB;
    private JLabel langTokenizerLabel;
    
    //button for connecting to advanced feature editor,
    private JButton advFeatureEditButton;
    private ButtonGroup useAdvFeatureBG;
    private JRadioButton useAdvFeatureYesRB; 
    private JRadioButton useAdvFeatureNoRB; 
    private JLabel useAdvFeatureLabel;    
    
    //checkbox for subsetting the dataset
    private JCheckBox doSubsettingInputCheckBox;   
    private JLabel subsettingProportionLabel;
    private JTextField subsettingProportionTextField;    
    
    // for advanced feature extraction
    private File file;
    private FeatureSpaceMapBuilder fsmBuilder = null;
    
    /** Lets the user configure the classifier */
    protected GenericObjectEditor4TagHelper m_ClassifierEditor =
      new GenericObjectEditor4TagHelper();

    /** The panel showing the current classifier selection */
    protected PropertyPanel m_CEPanel = new PropertyPanel(m_ClassifierEditor);
        
    
    /* Register the property editors we need */
    static {
//      java.beans.PropertyEditorManager
//        .registerEditor(weka.classifiers.Classifier.class,
//  		      weka.gui.GenericObjectEditor.class);   
        java.beans.PropertyEditorManager
        .registerEditor(weka.classifiers.Classifier.class,
  		      edu.cmu.taghelpertools.gui.GenericObjectEditor4TagHelper.class);    
        java.beans.PropertyEditorManager
        .registerEditor(weka.attributeSelection.ASEvaluation.class,
        		edu.cmu.taghelpertools.gui.GenericObjectEditor4TagHelper.class);
        java.beans.PropertyEditorManager
        .registerEditor(weka.attributeSelection.ASSearch.class,
        		edu.cmu.taghelpertools.gui.GenericObjectEditor4TagHelper.class);       
    }    
    
    public OptionsFrame(String title, OptionsModel optionsModel)
    {
    	this(title, optionsModel, null);    	
    }
    		

    public OptionsFrame(String title, OptionsModel optionsModel, File file) {

        this.setTitle(title);
        this.optionsModel = optionsModel;
        this.file = file;
        this.optionsModel.setOptionsFrame(this);
        
        if(optionsModel.getFSMBuilder() != null)
        	this.fsmBuilder = optionsModel.getFSMBuilder();

        tabbedPane = new JTabbedPane();
 
        tabbedPane.addTab("Feature Selection", this.buildFeatureSelectionGUI());
        tabbedPane.addTab("Language ("+this.optionsModel.getLanguage()+")", 
				  this.buildLanguageOptionGUI());  
        tabbedPane.addTab("Ouput", this.buildOutputOptionGUI());
        tabbedPane.addTab("Advanced", this.buildIterationOptionsGUI());        

        this.add(tabbedPane);

        this.setSize(getPreferredSize());
        this.setResizable(false);
        this.pack();

        // simply create a new dummy object, which we don't ever use
        this.lastRBCalled = new JRadioButton();
              
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                Object[] options = { "yes", "no" };
                int n = JOptionPane.showOptionDialog(null, "Do you wish to save these settings?", "Warning", JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE, null, options, options[0]);
                if (n == 0) {
                    save();
                }
                dispose();
            }
        });
        
        m_ClassifierEditor.setClassType(weka.classifiers.Classifier.class);
        m_ClassifierEditor.setValue(optionsModel.getClassifier());
        m_ClassifierEditor.addPropertyChangeListener(new PropertyChangeListener() {
          public void propertyChange(PropertyChangeEvent e) {
        	  repaint();
          }
        });        
    }

    public JPanel buildLanguageOptionGUI()
    {
    	JPanel langOptionPanel = new JPanel();
    	langOptionPanel.setLayout(new LangOptionLayout()); 
    	langOptionPanel.setFont(new Font("Arial", Font.PLAIN, 12));
    	
        this.langTokenizerLabel = new JLabel("Language of the Dataset:");
    	
        this.langTokenizerBG = new ButtonGroup();
        this.langTokenizerEnglishRB = new JRadioButton("English", optionsModel.getLanguage().equals(Tokenizer.LANGS[0]));        
        langTokenizerBG.add(langTokenizerEnglishRB);        
        this.langTokenizerGermanRB = new JRadioButton("German", optionsModel.getLanguage().equals(Tokenizer.LANGS[1]));        
        langTokenizerBG.add(langTokenizerGermanRB);
        this.langTokenizerChineseRB = new JRadioButton("Chinese(Trad)", optionsModel.getLanguage().equals(Tokenizer.LANGS[2]));
        langTokenizerBG.add(langTokenizerChineseRB);
        this.langTokenizerSpanishRB = new JRadioButton("Spanish", optionsModel.getLanguage().equals(Tokenizer.LANGS[3]));
        langTokenizerBG.add(langTokenizerSpanishRB);
        
        if(!OptionsModel.enableChineseOption)
        	this.langTokenizerChineseRB.setEnabled(false);
        
        this.langTokenizerEnglishRB.addActionListener(this);
        this.langTokenizerGermanRB.addActionListener(this);
        this.langTokenizerChineseRB.addActionListener(this);
        this.langTokenizerSpanishRB.addActionListener(this);
        langOptionPanel.add(this.langTokenizerLabel);
        langOptionPanel.add(this.langTokenizerEnglishRB);   
        langOptionPanel.add(this.langTokenizerGermanRB);
        langOptionPanel.add(this.langTokenizerChineseRB);
        langOptionPanel.add(this.langTokenizerSpanishRB);
        
    	return langOptionPanel;
    }
    
    public JPanel buildOutputOptionGUI()
    {
    	JPanel outputOptionPanel = new JPanel();
    	outputOptionPanel.setLayout(new OuputOptionLayout()); 
    	outputOptionPanel.setFont(new Font("Arial", Font.PLAIN, 12));
    	
        this.formatLabel = new JLabel("Export Format:");
    	
        this.formatBG = new ButtonGroup();
        this.formatWekaRB = new JRadioButton("Weka(.arff)", optionsModel.isUsingArff());
        formatWekaRB.addActionListener(this);
        formatBG.add(formatWekaRB);
        this.formatMTRB = new JRadioButton("MinorThird(.mt)", !optionsModel.isUsingArff());
        formatMTRB.addActionListener(this);
        formatBG.add(formatMTRB);
        
        
        outputOptionPanel.add(formatLabel);
        outputOptionPanel.add(formatWekaRB);   
        outputOptionPanel.add(formatMTRB);
        
        this.evalLabel = new JLabel("Evaluation Report (may be slow):");
        this.evalBG = new ButtonGroup();
        this.evalYesRB = new JRadioButton("Yes", optionsModel.isEvaluating());
        this.evalBG.add(evalYesRB);
        this.evalNoRB = new JRadioButton("No", !optionsModel.isEvaluating());
        this.evalBG.add(evalNoRB);
        
        if(this.optionsModel.isSelfTrain() || !this.optionsModel.isUsingArff())
        {
          this.evalYesRB.setSelected(false);
          this.evalNoRB.setSelected(true);
          this.evalYesRB.setEnabled(false);
          this.evalNoRB.setEnabled(false);
        }
        
        outputOptionPanel.add(evalLabel);
        outputOptionPanel.add(evalYesRB);
        outputOptionPanel.add(evalNoRB);
        
    	return outputOptionPanel;
    }
    
    public JPanel buildFeatureSelectionGUI() {
        JPanel featureSelectionPanel = new JPanel();
        featureSelectionPanel.setLayout(new FeatureSelectionLayout());
        featureSelectionPanel.setFont(new Font("Arial", Font.PLAIN, 12));

        this.stopwordBG = new ButtonGroup();
        this.stemBG = new ButtonGroup();
        this.removeStopwordNoRB = new JRadioButton("No", !optionsModel.isRemoveStopwords());
        stopwordBG.add(removeStopwordNoRB);
        featureSelectionPanel.add(removeStopwordNoRB);
        
        this.stemYesRB = new JRadioButton("Yes", optionsModel.isStem());
        stemBG.add(stemYesRB);
        featureSelectionPanel.add(stemYesRB);

        this.stopwordLabel = new JLabel("Remove Stopwords?");
        featureSelectionPanel.add(stopwordLabel);

        this.stemLabel = new JLabel("Stem?");
        featureSelectionPanel.add(stemLabel);

        this.removeStopwordYesRB = new JRadioButton("Yes", optionsModel.isRemoveStopwords());
        this.stopwordBG.add(removeStopwordYesRB);
        featureSelectionPanel.add(removeStopwordYesRB);

        this.stemNoRB = new JRadioButton("No", !optionsModel.isStem());
        this.stemBG.add(stemNoRB);
        featureSelectionPanel.add(stemNoRB);

/*GUI setting fo user-defined feature*/
        this.advFeatureEditButton = new JButton("Adv. Feature Editing");
        this.advFeatureEditButton.addActionListener(this);
        this.advFeatureEditButton.setVisible(optionsModel.useAdvFeatures());
        featureSelectionPanel.add(this.advFeatureEditButton);
        
        this.useAdvFeatureLabel = new JLabel("User Defined?");
        this.updateAdvFeatureLabel();
        featureSelectionPanel.add(this.useAdvFeatureLabel);
        
        this.useAdvFeatureBG = new ButtonGroup();
        //this.useAdvFeatureBG.addActionListener(this);
        this.useAdvFeatureYesRB = new JRadioButton("Yes", optionsModel.useAdvFeatures());
        this.useAdvFeatureYesRB.addActionListener(this);
        this.useAdvFeatureBG.add(this.useAdvFeatureYesRB);
        featureSelectionPanel.add(this.useAdvFeatureYesRB);
        
        this.useAdvFeatureNoRB = new JRadioButton("No", !optionsModel.useAdvFeatures());
        this.useAdvFeatureNoRB.addActionListener(this);
        this.useAdvFeatureBG.add(this.useAdvFeatureNoRB);
        featureSelectionPanel.add(this.useAdvFeatureNoRB);        
/*end of GUI settings of user-defined feature*/
                
        this.okButton = new JButton("OK");
        this.okButton.addActionListener(this);
        this.okButton.setEnabled(false);
        featureSelectionPanel.add(okButton);
       
        this.unigramCheckBox = new JCheckBox("unigrams");
        this.unigramCheckBox.setSelected(optionsModel.isUnigrams());
        this.unigramCheckBox.addActionListener(this);
        featureSelectionPanel.add(unigramCheckBox);
        this.bigramsCheckBox = new JCheckBox("bigrams");
        this.bigramsCheckBox.setSelected(optionsModel.isBigrams());
        this.bigramsCheckBox.addActionListener(this);
        featureSelectionPanel.add(bigramsCheckBox);

        this.posBigramCheckBox = new JCheckBox("POS bigrams (slow!)");
        this.posBigramCheckBox.setSelected(optionsModel.isPosBigrams());
        this.posBigramCheckBox.addActionListener(this);
        featureSelectionPanel.add(posBigramCheckBox);

        this.lineLengthCheckBox = new JCheckBox("line length");
        this.lineLengthCheckBox.setSelected(optionsModel.isLineLength());
        this.lineLengthCheckBox.addActionListener(this);
        featureSelectionPanel.add(lineLengthCheckBox);

        this.containsNonStopwordCheckBox = new JCheckBox("contains NON-stopword");
        this.containsNonStopwordCheckBox.setSelected(optionsModel.isContainsNonStopword());
        this.containsNonStopwordCheckBox.addActionListener(this);
        featureSelectionPanel.add(containsNonStopwordCheckBox);

        classifierLabel = new JLabel("Select a Classifier");
//        classifierComboBox = new JComboBox(Classifiers.classifiers);
//        classifierComboBox.setSelectedItem(optionsModel.getClassifier());
//        featureSelectionPanel.add(classifierComboBox);
        featureSelectionPanel.add(m_CEPanel);	
        featureSelectionPanel.add(classifierLabel);

        this.punctCheckBox = new JCheckBox("punctuation");
        this.punctCheckBox.addActionListener(this);
        this.punctCheckBox.setSelected(optionsModel.isPuctuation());
        featureSelectionPanel.add(this.punctCheckBox);

        this.fileLabel = new JLabel(this.getTitle());
        featureSelectionPanel.add(fileLabel);

        this.removeRareFeaturesCheckBox = new JCheckBox("remove rare features");
        this.removeRareFeaturesCheckBox.addActionListener(this);
        this.removeRareFeaturesCheckBox.setSelected(optionsModel.isRemoveRareFeatures());
        featureSelectionPanel.add(this.removeRareFeaturesCheckBox);

        // code to add rare feature removal frequency threshold
        this.thresholdLabel = new JLabel("Frequency Threshold");
        featureSelectionPanel.add(thresholdLabel);
        
        this.frequencyThreshold = new JTextField(String.valueOf(this.optionsModel.getFrequencyThreshold()));
        this.frequencyThreshold.addFocusListener(new FocusListener() {
            public void focusGained(FocusEvent e) {
                if (frequencyThreshold.getText().length() == 0)
                    frequencyThreshold.setText(String.valueOf(OptionsModel.DEFAULT_FREQ_THRESH));
            }

            public void focusLost(FocusEvent e) {
                try {
                    int num = Integer.parseInt(frequencyThreshold.getText());
                    if (!(num > 0)) {
                        JOptionPane.showMessageDialog(null, "Must enter a number > 0.", "INVALID ACTION", JOptionPane.ERROR_MESSAGE);
                        frequencyThreshold.setText(String.valueOf(OptionsModel.DEFAULT_FREQ_THRESH));
                        SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                frequencyThreshold.requestFocus();
                                tabbedPane.setSelectedIndex(0);
                            }
                        });
                    }
                } catch (NumberFormatException e1) {
                    JOptionPane.showMessageDialog(null, "Must enter numeric value.", "INVALID ACTION", JOptionPane.ERROR_MESSAGE);
                    frequencyThreshold.setText(String.valueOf(OptionsModel.DEFAULT_FREQ_THRESH));
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            frequencyThreshold.requestFocus();
                            tabbedPane.setSelectedIndex(0);
                        }
                    });
                }
            }
        });

        featureSelectionPanel.add(frequencyThreshold);

        featureSelectionPanel.setForeground(this.stopwordLabel.getForeground());
        featureSelectionPanel.setBackground(this.stopwordLabel.getBackground());

        boolean unigramsORBigrams = this.unigramCheckBox.isSelected() || this.bigramsCheckBox.isSelected();

        this.stopwordLabel.setVisible(unigramsORBigrams);
        this.stemLabel.setVisible(unigramsORBigrams);
        this.removeStopwordNoRB.setVisible(unigramsORBigrams);
        this.removeStopwordYesRB.setVisible(unigramsORBigrams);
        this.stemNoRB.setVisible(unigramsORBigrams);
        this.stemYesRB.setVisible(unigramsORBigrams);
//        this.removeStopwordNoRB.setVisible(unigramsORBigrams);
//        this.removeStopwordYesRB.setVisible(unigramsORBigrams);
//        this.stemNoRB.setVisible(unigramsORBigrams);
//        this.stemYesRB.setVisible(unigramsORBigrams);
        this.thresholdLabel.setVisible(this.removeRareFeaturesCheckBox.isSelected());
        this.frequencyThreshold.setVisible(this.removeRareFeaturesCheckBox.isSelected());
        
        boolean any = this.unigramCheckBox.isSelected() || this.bigramsCheckBox.isSelected() || this.posBigramCheckBox.isSelected() || this.lineLengthCheckBox.isSelected() || this.punctCheckBox.isSelected() || this.containsNonStopwordCheckBox.isSelected();
        this.okButton.setEnabled(any);

        if(this.optionsModel.getLanguage().toLowerCase().equals("chinese"))
        {
//			this.punctCheckBox.setEnabled(false);
    		this.stemNoRB.setSelected(true);        	
			this.stemYesRB.setEnabled(false);
			this.stemNoRB.setEnabled(false);        
        }

        return featureSelectionPanel;
    }

    public JPanel buildIterationOptionsGUI() {

        JPanel iterOptionsPanel = new JPanel();
        iterOptionsPanel.setLayout(new IterationSettingsLayout());
        iterOptionsPanel.setFont(new Font("Arial", Font.PLAIN, 12));
        itterBG = new ButtonGroup();

        // do self-training check box.
        this.doSelfTrainCheckBox = new JCheckBox("self-train");
        this.doSelfTrainCheckBox.setSelected(this.optionsModel.isSelfTrain());
        this.doSelfTrainCheckBox.addActionListener(this);

        // fixed num
        fixedNumRB = new JRadioButton("add fixed # of predictions");
        fixedNumRB.addActionListener(this);
        fixedNumRB.setSelected((optionsModel.getIterationMode() == MultiDimClassifier.ADD_FIXED_NUM));
        itterBG.add(fixedNumRB);
        numLabel = new JLabel("num:");
        if ((optionsModel.getIterationMode() == MultiDimClassifier.ADD_FIXED_NUM)) {
            numTextField = new JTextField(String.valueOf(optionsModel.getFixedNum()));
        } else {
            numTextField = new JTextField("");//String.valueOf(OptionsFrame.DEFAULT_NUM));
        }
        numTextField.setEnabled((optionsModel.getIterationMode() == MultiDimClassifier.ADD_FIXED_NUM));
        numTextField.addFocusListener(new FocusListener() {
            public void focusGained(FocusEvent e) {
                if (numTextField.getText().length() == 0)
                    numTextField.setText(String.valueOf(OptionsModel.DEFAULT_NUM_EXAMPLES));
            }

            public void focusLost(FocusEvent e) {
                try {
                    int num = Integer.parseInt(numTextField.getText());
                    if (!(num > 0)) {
                        JOptionPane.showMessageDialog(null, "Must enter a number > 0.", "INVALID ACTION", JOptionPane.ERROR_MESSAGE);
                        numTextField.setText(String.valueOf(OptionsModel.DEFAULT_NUM_EXAMPLES));
                        SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                numTextField.requestFocus();
                                tabbedPane.setSelectedIndex(1);
                            }
                        });
                    }
                } catch (NumberFormatException e1) {
                    JOptionPane.showMessageDialog(null, "Must enter numeric value.", "INVALID ACTION", JOptionPane.ERROR_MESSAGE);
                    numTextField.setText(String.valueOf(OptionsModel.DEFAULT_NUM_EXAMPLES));
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            numTextField.requestFocus();
                            tabbedPane.setSelectedIndex(3);
                        }
                    });
                }
            }
        });

        // percent
        percentRB = new JRadioButton("add % of predictions");
        percentRB.addActionListener(this);
        percentRB.setSelected((optionsModel.getIterationMode() == MultiDimClassifier.ADD_PERCENT));
        itterBG.add(percentRB);
        percentLabel = new JLabel("");
        if ((optionsModel.getIterationMode() == MultiDimClassifier.ADD_PERCENT)) {
            percentTextField = new JTextField(String.valueOf(optionsModel.getPercent()));
        } else {
            percentTextField = new JTextField(String.valueOf(""));
        }
        percentTextField.setEnabled((optionsModel.getIterationMode() == MultiDimClassifier.ADD_PERCENT));
        percentTextField.addFocusListener(new FocusListener() {
            public void focusGained(FocusEvent e) {
                if (percentTextField.getText().length() == 0)
                    percentTextField.setText(String.valueOf(OptionsModel.DEFAULT_PERCENT));
            }

            public void focusLost(FocusEvent e) {
                try {
                    double num = Double.parseDouble(percentTextField.getText());
                    if (!((0.0 < num) && (num <= 1.0))) {
                        JOptionPane.showMessageDialog(null, "Must enter decimal: (0 < value <= 1.0)", "INVALID ACTION", JOptionPane.ERROR_MESSAGE);
                        percentTextField.setText(String.valueOf(OptionsModel.DEFAULT_PERCENT));
                        SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                percentTextField.requestFocus();
                                tabbedPane.setSelectedIndex(3);
                            }
                        });
                    }
                } catch (NumberFormatException e1) {
                    JOptionPane.showMessageDialog(null, "Must enter decimal: (0 < value <= 1.0)", "INVALID ACTION", JOptionPane.ERROR_MESSAGE);
                    percentTextField.setText(String.valueOf(OptionsModel.DEFAULT_PERCENT));
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            percentTextField.requestFocus();
                            tabbedPane.setSelectedIndex(3);
                        }
                    });
                }
            }
        });


        // threshold
        thresholdRB = new JRadioButton("add ALL above threshold");
        thresholdRB.addActionListener(this);
        thresholdRB.setSelected((optionsModel.getIterationMode() == MultiDimClassifier.ADD_THRESH));
        itterBG.add(thresholdRB);
        threshLabel = new JLabel("thresh:");

        if ((optionsModel.getIterationMode() == MultiDimClassifier.ADD_THRESH)) {
            threshTextField = new JTextField(String.valueOf(optionsModel.getThresh()));
        } 
        else 
        {
            threshTextField = new JTextField("");
        }
        threshTextField.setEnabled((optionsModel.getIterationMode() == MultiDimClassifier.ADD_THRESH));
        threshTextField.addFocusListener(new FocusListener() {
            public void focusGained(FocusEvent e) {
                if (threshTextField.getText().length() == 0)
                    threshTextField.setText(String.valueOf(OptionsModel.DEFAULT_THRESH));
            }

            public void focusLost(FocusEvent e) {
                try {
                    double num = Double.parseDouble(threshTextField.getText());
                    if (!((0 < num) && (num < 1))) {
                        JOptionPane.showMessageDialog(null, "Must enter decimal: (0 < value < 1.0)", "INVALID ACTION", JOptionPane.ERROR_MESSAGE);
                        threshTextField.setText(String.valueOf(OptionsModel.DEFAULT_THRESH));
                        SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                threshTextField.requestFocus();
                                tabbedPane.setSelectedIndex(3);
                            }
                        });
                    }
                } catch (NumberFormatException e1) {
                    JOptionPane.showMessageDialog(null, "Must enter decimal: (0 < value < 1.0)", "INVALID ACTION", JOptionPane.ERROR_MESSAGE);
                    threshTextField.setText(String.valueOf(OptionsModel.DEFAULT_THRESH));
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            threshTextField.requestFocus();
                            tabbedPane.setSelectedIndex(3);
                        }
                    });
                }
            }
        });
        
        //subsetting input data
        this.doSubsettingInputCheckBox = new JCheckBox("randomly sample input data");
        this.doSubsettingInputCheckBox.setSelected(this.optionsModel.isSubsetInput());
        this.doSubsettingInputCheckBox.addActionListener(this);
        this.subsettingProportionLabel = new JLabel("ratio:");        
        this.subsettingProportionTextField = new JTextField(
        								String.valueOf(this.optionsModel.getSubsetPortion())
        								);
        this.subsettingProportionTextField.setEnabled(this.optionsModel.isSubsetInput());
        this.subsettingProportionTextField.addFocusListener(new FocusListener() {
            public void focusGained(FocusEvent e) {
                if (subsettingProportionTextField.getText().length() == 0)
                	subsettingProportionTextField.setText(String.valueOf(optionsModel.getSubsetPortion()));
            }

            public void focusLost(FocusEvent e) {
                try {
                    double num = Double.parseDouble(subsettingProportionTextField.getText());
                    if (!((0 < num) && (num < 1))) {
                        JOptionPane.showMessageDialog(null, "Must enter decimal: (0 < value < 1.0)", "INVALID ACTION", JOptionPane.ERROR_MESSAGE);
                        subsettingProportionTextField.setText(String.valueOf(optionsModel.getSubsetPortion()));
                        SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                            	subsettingProportionTextField.requestFocus();
                                tabbedPane.setSelectedIndex(3);
                            }
                        });
                    }
                } catch (NumberFormatException e1) {
                    JOptionPane.showMessageDialog(null, "Must enter decimal: (0 < value < 1.0)", "INVALID ACTION", JOptionPane.ERROR_MESSAGE);
                    subsettingProportionTextField.setText(String.valueOf(optionsModel.getSubsetPortion()));
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                        	subsettingProportionTextField.requestFocus();
                            tabbedPane.setSelectedIndex(3);
                        }
                    });
                }
            }
        });
        
        // adding components to options panel
        iterOptionsPanel.add(fixedNumRB);
        iterOptionsPanel.add(percentRB);
        iterOptionsPanel.add(thresholdRB);
        iterOptionsPanel.add(numTextField);
        iterOptionsPanel.add(numLabel);
        iterOptionsPanel.add(percentTextField);
        iterOptionsPanel.add(percentLabel);
        iterOptionsPanel.add(threshTextField);
        iterOptionsPanel.add(threshLabel);
        iterOptionsPanel.add(doSelfTrainCheckBox);
        iterOptionsPanel.add(this.doSubsettingInputCheckBox);
        iterOptionsPanel.add(this.subsettingProportionLabel);
        iterOptionsPanel.add(this.subsettingProportionTextField);
        
        if(!this.optionsModel.isSelfTrain())
        {
	        this.fixedNumRB.setEnabled(false);
	        this.percentRB.setEnabled(false);
	        this.thresholdRB.setEnabled(false);
	        this.numTextField.setEnabled(false);
	        this.threshTextField.setEnabled(false);
	        this.percentTextField.setEnabled(false);
        }
        setSize(getPreferredSize());
        return iterOptionsPanel;
    }

    public void display(boolean bool) 
    {
        if (bool) {
            // open the frame in the middle of the screen
            Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
            int width = this.getWidth();
            int height = this.getHeight();
            int left = (screen.width - width) / 2;
            int top = (screen.height - height) / 2;
            setBounds(left, top, width, height);
            this.setVisible(true);
        } else {
            this.setVisible(false);
        }
    }


    public void actionPerformed(ActionEvent e) {    	
        //by Hao-Chuan
        if(
            e.getSource().equals(this.langTokenizerEnglishRB)||
            e.getSource().equals(this.langTokenizerGermanRB)||
            e.getSource().equals(this.langTokenizerChineseRB)||
            e.getSource().equals(this.langTokenizerSpanishRB)
           )
        {
        	if(this.langTokenizerEnglishRB.isSelected())
        	{
        		this.tabbedPane.setTitleAt(1, "Language ("+Tokenizer.LANGS[Tokenizer.LANG_ENGLISH]+")");
//        		this.punctCheckBox.setEnabled(true);
        		this.stemYesRB.setEnabled(true);
        		this.stemNoRB.setEnabled(true);       
        		this.useAdvFeatureNoRB.setEnabled(true);
        		this.useAdvFeatureYesRB.setEnabled(true);
        	}
        	else if(this.langTokenizerGermanRB.isSelected())
        	{
        		this.tabbedPane.setTitleAt(1, "Language ("+Tokenizer.LANGS[Tokenizer.LANG_GERMAN]+")");
//        		this.punctCheckBox.setEnabled(true);  
        		this.stemYesRB.setEnabled(true);
        		this.stemNoRB.setEnabled(true); 
        		this.useAdvFeatureNoRB.setEnabled(true);
        		this.useAdvFeatureYesRB.setEnabled(true);
        	}
        	else if(this.langTokenizerChineseRB.isSelected())
        	{
        		this.tabbedPane.setTitleAt(1, "Language ("+Tokenizer.LANGS[Tokenizer.LANG_CHINESE]+")");
//        		this.punctCheckBox.setSelected(false);
//        		this.punctCheckBox.setEnabled(true); 
        		this.stemNoRB.setSelected(true);
        		this.stemYesRB.setEnabled(false);
        		this.stemNoRB.setEnabled(false);
        		
        		this.useAdvFeatureNoRB.setSelected(true);
        		this.useAdvFeatureNoRB.setEnabled(false);
        		this.useAdvFeatureYesRB.setEnabled(false);
        		this.advFeatureEditButton.setVisible(false);
        	}
        	else if(this.langTokenizerSpanishRB.isSelected())
        	{
        		this.tabbedPane.setTitleAt(1, "Language ("+Tokenizer.LANGS[Tokenizer.LANG_SPANISH]+")");
//        		this.punctCheckBox.setSelected(false);
//        		this.punctCheckBox.setEnabled(true); 
        		this.stemYesRB.setEnabled(true);
        		this.stemNoRB.setEnabled(true);
        		
        		this.useAdvFeatureNoRB.setSelected(true);
        		this.useAdvFeatureNoRB.setEnabled(false);
        		this.useAdvFeatureYesRB.setEnabled(false);
        		this.advFeatureEditButton.setVisible(false);
        	}
        }
        if(e.getSource().equals(this.doSubsettingInputCheckBox))
        {
        	this.subsettingProportionTextField.setEnabled(this.doSubsettingInputCheckBox.isSelected());
        }
        
        if (e.getSource().equals(doSelfTrainCheckBox)) {

            if (doSelfTrainCheckBox.isSelected()) {
                this.fixedNumRB.setEnabled(true);
                this.fixedNumRB.setSelected(true);
                this.numTextField.setEnabled(true);
                this.numTextField.setText(String.valueOf(this.optionsModel.getFixedNum()));
                this.percentRB.setEnabled(true);
                this.percentRB.setSelected(false);
                this.percentTextField.setEnabled(false);
                this.percentTextField.setText("");
                this.thresholdRB.setEnabled(true);
                this.thresholdRB.setSelected(false);
                this.threshTextField.setEnabled(false);
                this.threshTextField.setText("");
                
                //by Hao-Chuan
                this.evalYesRB.setSelected(false);
                this.evalNoRB.setSelected(true);
                this.evalYesRB.setEnabled(false);
                this.evalNoRB.setEnabled(false);
            } else {
                this.fixedNumRB.setEnabled(false);
                this.fixedNumRB.setSelected(false);
                this.numTextField.setEnabled(false);
                this.numTextField.setText("");
                this.percentRB.setEnabled(false);
                this.percentRB.setSelected(false);
                this.percentTextField.setEnabled(false);
                this.percentTextField.setText("");
                this.thresholdRB.setEnabled(false);
                this.thresholdRB.setSelected(false);
                this.threshTextField.setEnabled(false);
                this.threshTextField.setText("");
                
                //by Hao-Chuan
                this.evalYesRB.setSelected(this.optionsModel.isEvaluating());
                this.evalNoRB.setSelected(!this.optionsModel.isEvaluating());
                this.evalYesRB.setEnabled(true);
                this.evalNoRB.setEnabled(true);                
            }
        }        
        if (     e.getSource().equals(this.fixedNumRB) ||
        		 e.getSource().equals(this.percentRB) ||
        		 e.getSource().equals(this.thresholdRB)
           )  
        {
            this.numTextField.setEnabled(e.getSource().equals(this.fixedNumRB));
            this.percentTextField.setEnabled(e.getSource().equals(this.percentRB));
            this.threshTextField.setEnabled(e.getSource().equals(this.thresholdRB));
            
            if (numTextField.isEnabled()) {
                if (!fixedNumRB.equals(this.lastRBCalled))
                    numTextField.setText(String.valueOf(this.optionsModel.getFixedNum()));
            } 
            else 
            {
                numTextField.setText("");
            }

            if (threshTextField.isEnabled()) {
                if (!thresholdRB.equals(this.lastRBCalled))
                    threshTextField.setText(String.valueOf(this.optionsModel.getThresh()));
            } else {
                threshTextField.setText("");
            }

            if (percentTextField.isEnabled()) {
                if (!percentRB.equals(this.lastRBCalled))
                    percentTextField.setText(String.valueOf(this.optionsModel.getPercent()));
            } else {
                percentTextField.setText("");
            }

            if (e.getSource().getClass().equals(this.lastRBCalled.getClass())) {
                lastRBCalled = (JRadioButton) e.getSource();
            }
        }
        
        if (e.getSource().getClass().equals(this.bigramsCheckBox.getClass())) 
        {
            boolean isAtLeastOneChecked = this.unigramCheckBox.isSelected() || 
            							this.bigramsCheckBox.isSelected() || 
            							this.posBigramCheckBox.isSelected() || 
            							this.lineLengthCheckBox.isSelected() || 
            							this.punctCheckBox.isSelected();
            okButton.setEnabled(isAtLeastOneChecked);
        }                   
        
        if (e.getSource().equals(this.unigramCheckBox) || 
        	e.getSource().equals(this.bigramsCheckBox)) 
        {
            boolean enable = this.unigramCheckBox.isSelected() || 
            			     this.bigramsCheckBox.isSelected();
            this.stopwordLabel.setVisible(enable);
            this.stemLabel.setVisible(enable);
            this.removeStopwordNoRB.setVisible(enable);
            this.removeStopwordYesRB.setVisible(enable);
//            this.removeStopwordYesRB.setSelected(true);
            this.stemNoRB.setVisible(enable);
            this.stemYesRB.setVisible(enable);
//            this.stemYesRB.setSelected(true);
            this.removeStopwordNoRB.setVisible(enable);
            this.removeStopwordYesRB.setVisible(enable);
            this.stemNoRB.setVisible(enable);
            this.stemYesRB.setVisible(enable);
        }
        // code to set visible/invisible the frequency threshold text
        // box and label
        if (e.getSource().equals(this.removeRareFeaturesCheckBox)) {
            this.thresholdLabel.setVisible(this.removeRareFeaturesCheckBox.isSelected());
            this.frequencyThreshold.setVisible(this.removeRareFeaturesCheckBox.isSelected());
        }
        
        //TODO: adv feature
	    if (e.getSource().equals(this.useAdvFeatureYesRB)||e.getSource().equals(this.useAdvFeatureNoRB))
	    {
	    	boolean enable = this.useAdvFeatureYesRB.isSelected();
	    	//this.advFeatureEditButton.setVisible(enable);
	      	this.advFeatureEditButton.setVisible(enable);
	      	this.optionsModel.setUseAdvFeatures(enable);
	      	//System.out.println("use adv feature~~~~~~~");
	    }
        	        
        if(e.getSource().equals(this.advFeatureEditButton))
        {
        	this.optionsModel.setUseAdvFeatures(true);
        	
        	this.langTokenizerEnglishRB.setEnabled(false);
        	this.langTokenizerChineseRB.setEnabled(false);
        	this.langTokenizerGermanRB.setEnabled(false);
        	this.langTokenizerSpanishRB.setEnabled(false);
        	
        	
        	
        	if(this.fsmBuilder == null)
        		this.fsmBuilder = new FeatureSpaceMapBuilder();
//        	List<DataPoint> dataPoints = fsmBuilder.setUp(new Reader(file.getAbsolutePath()), this.optionsModel);
        	//this.optionsModel.setPreprocessedTrainingDataPoints(dataPoints);  
        	this.save();
        	PreExtractor runnn = new PreExtractor(this.fsmBuilder, new Reader(file.getAbsolutePath()), this.optionsModel);
        	runnn.start(); 
        }
        
        if (e.getSource().equals(this.formatMTRB) && this.formatMTRB.isSelected()) {
          this.evalYesRB.setSelected(false);
          this.evalNoRB.setSelected(true);
          this.evalYesRB.setEnabled(false);
          this.evalNoRB.setEnabled(false);
        }
        
        if (e.getSource().equals(this.formatWekaRB) && this.formatWekaRB.isSelected()) {
          this.evalYesRB.setSelected(true);
          this.evalNoRB.setSelected(false);
          this.evalYesRB.setEnabled(true);
          this.evalNoRB.setEnabled(true);
        }
        
        // ok button pressed
        if (e.getSource().equals(this.okButton)) {
            this.save();
            this.dispose();
        }
    }
    
    
    private class PreExtractor extends SwingWorker {
    	FeatureSpaceMapBuilder fsmBuilder;
    	Reader input;
    	OptionsModel optionsModel;

        public PreExtractor(FeatureSpaceMapBuilder fsmBuilder, Reader input, OptionsModel optionsModel) 
        {
        	this.fsmBuilder = fsmBuilder;
        	this.input = input;
        	this.optionsModel = optionsModel;
        }

        public Object construct() {       
        	//TODO: commented temporarily
        	if(this.optionsModel.getPreprocessedTrainingDataPoints() == null)
        	{
	        	ProgressDialog pDialog = new ProgressDialog(null, input.getFilename());
	        	InputFileReader ifr = new InputFileReader();
	        	ArrayList<DataPoint> dataPoints = this.fsmBuilder.setUp(pDialog, 
	        															input, 
	        															optionsModel,
	        															ifr);
	        	optionsModel.setPreprocessed(dataPoints, ifr, this.fsmBuilder);
	        	pDialog.dispose();
        	}
        	new ExtractorGUI(this.fsmBuilder, this.optionsModel);        	
        	return null;
        }
    } 
    
    public void updateAdvFeatureLabel()
    {
    	int count = 0;
    	if(this.fsmBuilder !=null)
    		count = fsmBuilder.getNumSynFeatures();
    	
    	this.useAdvFeatureLabel.setText("User Defined? ("+count+")");
    }
    
    public void save() {
        // feature selection
        optionsModel.setUnigrams(this.unigramCheckBox.isSelected());
        optionsModel.setBigrams(this.bigramsCheckBox.isSelected());
        optionsModel.setPosBigrams(this.posBigramCheckBox.isSelected());
        optionsModel.setLineLength(this.lineLengthCheckBox.isSelected());
        optionsModel.setContainsNonStopword(this.containsNonStopwordCheckBox.isSelected());
        optionsModel.setPunctuation(this.punctCheckBox.isSelected());
        optionsModel.setRemoveRareFeatures(this.removeRareFeaturesCheckBox.isSelected());
        // text processing
        optionsModel.setStem(this.stemYesRB.isSelected());
        optionsModel.setRemoveStopwords(this.removeStopwordYesRB.isSelected());
        optionsModel.setUseAdvFeatures(this.useAdvFeatureYesRB.isSelected());
        
        //data export format setting, by Hao-Chuan 20070110
        optionsModel.setUsingArff(this.formatWekaRB.isSelected());
        
        //by Hao-Chuan
        optionsModel.setEvaluating(this.evalYesRB.isSelected());
        
        //languae related options, by Hao-Chuan 20070118
        int lang = OptionsModel.DEFAULT_LANG;
        if(this.langTokenizerEnglishRB.isSelected())
        {
        	lang = Tokenizer.LANG_ENGLISH;        	
        }
        else if(this.langTokenizerGermanRB.isSelected())
        {
        	lang = Tokenizer.LANG_GERMAN;
        }
        else if(this.langTokenizerChineseRB.isSelected())
        {
        	lang = Tokenizer.LANG_CHINESE;
        }      
        else if(this.langTokenizerSpanishRB.isSelected())
        {
        	lang = Tokenizer.LANG_SPANISH;
        }    
        optionsModel.setLanguage(Tokenizer.LANGS[lang], Tokenizer.STPS[lang]);
        
        // set the user specified frequency threshold in optionsModel
        if (this.removeRareFeaturesCheckBox.isSelected()) {
            int freqThresh = Integer.parseInt(frequencyThreshold.getText());
            optionsModel.setFrequencyThreshold(freqThresh);
        }
        
        // setting classifier by Hao-Chuan Wang
//        optionsModel.setClassifier(Classifiers.classifiers[this.classifierComboBox.getSelectedIndex()]);
//        optionsModel.setClassifierOptions(Classifiers.options4Classifiers[this.classifierComboBox.getSelectedIndex()]);
        optionsModel.setClassifier((weka.classifiers.Classifier)this.m_ClassifierEditor.getValue());
        
        // if doing the self-training step(s), then set iteration settings
        optionsModel.setSelfTraing(doSelfTrainCheckBox.isSelected());
        if (this.doSelfTrainCheckBox.isSelected()) 
        {
        	//fixed by Hao-Chuan
        	this.optionsModel.setSelfTraing(true);
            if (this.fixedNumRB.isSelected())//optionsModel.isSelfTrain()) 
            {
                optionsModel.setIterationMode(MultiDimClassifier.ADD_FIXED_NUM);
                optionsModel.setFixedNum(Integer.parseInt(this.numTextField.getText()));
            } 
            else if (this.thresholdRB.isSelected()) {
                optionsModel.setIterationMode(MultiDimClassifier.ADD_THRESH);
                optionsModel.setThresh(Double.parseDouble(this.threshTextField.getText()));
            } 
            else if (this.percentRB.isSelected()) {
                optionsModel.setIterationMode(MultiDimClassifier.ADD_PERCENT);
                optionsModel.setPercent(Double.parseDouble(this.percentTextField.getText()));
            }
        }
        optionsModel.setSubsetInput(this.doSubsettingInputCheckBox.isSelected());
        optionsModel.setSubsetPortion(Double.parseDouble(this.subsettingProportionTextField.getText()));
    }
}