/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package edu.cmu.taghelpertools.gui;

import edu.cmu.taghelpertools.middle_layer.Driver;
import edu.cmu.taghelpertools.middle_layer.InputFileReader;
import edu.cmu.taghelpertools.middle_layer.OptionsModel;
import gnu.trove.THashMap;
import gnu.trove.THashSet;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.Iterator;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.UIManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;


import jxl.Cell;
import jxl.Sheet;
import jxl.Workbook;

/**
 * 
 */
public class Portal extends JFrame implements ActionListener, MouseListener {

    private JButton trainButton;
    private JButton testButton;

    /**
     * Set the look and feel of the GUI to t	he OP specific look and feel,
     * instead of the Java default look and feel
     */
    private void setLookAndFeel() {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) {
            try {
                UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
            } catch (Exception e1) {
                e1.printStackTrace();
                System.exit(1);
            }
        }
    }
    
    public static void main(String args[]) {
    	Portal p = new Portal();
    } 

    /**
     * Creates the GUI components and creates the GUI, with the help of
     * FileSelectorLayout, a customized layout manager, created with
     * "JavaGUIBuilder".
     * 
     * @param driver -
     *            The Driver (the main executioner of the program) is passed as
     *            parameter because several calls are made from this method to
     *            modify the "state" of the Driver.
     */
    public Portal() {

        // sets the look and feel of the GUI.
        this.setLookAndFeel();

        this.setTitle("TagHelper Portal");

        PortalLayout customLayout = new PortalLayout();

        getContentPane().setFont(new Font("Arial", Font.PLAIN, 14));
        getContentPane().setLayout(customLayout);

       trainButton = new JButton("Train New Models");
        trainButton.addActionListener(this);
        getContentPane().add(trainButton);
        trainButton.setEnabled(true);

        testButton = new JButton("Test A Saved Model");
        testButton.addActionListener(this);
        getContentPane().add(testButton);
        testButton.setEnabled(true);

        this.pack();
        this.setResizable(false);

        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
        int width = this.getWidth();
        int height = this.getHeight();
        int left = (screen.width - width) / 5;
        int top = (screen.height - height) / 5;
        setBounds(left, top, width, height);

        this.setVisible(true);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
    } //End of constructor

    /*
     * Catches all events in the FileSelector. (non-Javadoc)
     * 
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getSource().equals(this.trainButton)) {
            Driver driver = new Driver();
            driver.run();                   
        } 
        else if (e.getSource().equals(this.testButton)) {
        	TestModelFrame t = new TestModelFrame();
        }
    }

  
    public void mouseClicked(MouseEvent e) {

    }

    public void mousePressed(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    public void mouseReleased(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    public void mouseEntered(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    public void mouseExited(MouseEvent e) {
        // TODO Auto-generated method stub

    }
}
