/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package edu.cmu.taghelpertools.gui;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.Window;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.JTextPane;

public class ProgressDialog extends JDialog {

    private JProgressBar progressBar;
    // private JLabel label;
    private JTextPane label;
    private String labelText;

    public ProgressDialog(JFrame parent, String title) {
        super(parent);
        super.setTitle(title);
        this.createGUI();
    }

    public void createGUI() {
        ProgressDialogLayout customLayout = new ProgressDialogLayout();

        getContentPane().setFont(new Font("Arial", Font.PLAIN, 12));
        getContentPane().setLayout(customLayout);

        // label = new JLabel("starting process...");
        label = new JTextPane();
        label.setOpaque(false);
        label.setEditable(false);
        label.setText("starting process...");
        label.setToolTipText("starting process...");
        getContentPane().add(label);

        progressBar = new JProgressBar();
        progressBar.setIndeterminate(false);
        getContentPane().add(progressBar);

        this.setSize(this.getPreferredSize());
        this.pack();
        this.setResizable(false);

        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
        int width = this.getWidth();
        int height = this.getHeight();
        int left = (screen.width - width) / 2;
        int top = (screen.height - height) / 2;
        setBounds(left, top, width, height);

        this.setVisible(true);
        this.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
    }

    public void setMax(int max) {
        progressBar.setMaximum(max);
    }

    public void setValue(int val) {
        progressBar.setValue(val);
    }

    public void setLabel(String label) {
    		// we save the label text here for use by
    		// the updateLabelWithSuffix method
    		this.labelText = label;
        this.label.setText(label);
        this.label.setToolTipText(label);
    }
    
    public void setIsIndeterminate(boolean val){
        this.progressBar.setIndeterminate(val);
    }

    /**
     * This method only adds a suffix to the existing label set
     * on the progress bar.
     * 
     * @param suffix
     */
    public void updateLabelWithSuffix(String suffix) {
      this.label.setText(labelText + " " + suffix);
      this.label.setToolTipText(labelText + " " + suffix);
    }
}
