/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package edu.cmu.taghelpertools.gui;

import edu.cmu.taghelpertools.io.Reader;
import edu.cmu.taghelpertools.middle_layer.ClassifierModel;
import edu.cmu.taghelpertools.middle_layer.DataPoint;
import edu.cmu.taghelpertools.middle_layer.Driver;
import edu.cmu.taghelpertools.middle_layer.InputFileReader;
import edu.cmu.taghelpertools.middle_layer.MultiDimClassifier;
import edu.cmu.taghelpertools.middle_layer.OptionsModel;
import edu.cmu.taghelpertools.middle_layer.SavedModelEvaluator;
import edu.cmu.taghelpertools.middle_layer.SwingWorker;
import edu.cmu.taghelpertools.middle_layer.Driver.RunFile;
import gnu.trove.THashMap;
import gnu.trove.THashSet;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.Iterator;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.UIManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;


import jxl.Cell;
import jxl.Sheet;
import jxl.Workbook;

/**
 * 
 * This class is the GUI for testing a previously trained model by providing new dataset
 * 
 */
public class TestModelFrame extends JFrame implements ActionListener, MouseListener {

    public final static String MODEL_FILE_EXT = "model";
    public final static String DIC_FILE_EXT = "dic";
    public final static String DATA_FILE_EXT = "xls";
    private JButton loadModelButton;
    private JButton loadTestButton;
    private JButton goButton;
    private JTextField loadModelField;
    private JTextField loadTestField;
    private JLabel loadModelLabel;
    private JLabel loadTestLabel;
    
    private File modelFile;
    private File testDataFile;
    private String dimName;
   

    /**
     * Set the look and feel of the GUI to the OP specific look and feel,
     * instead of the Java default look and feel
     */
    private void setLookAndFeel() {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) {
            try {
                UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
            } catch (Exception e1) {
                e1.printStackTrace();
                System.exit(1);
            }
        }
    }
    
    public static void main(String args[]) {
    	TestModelFrame t = new TestModelFrame();
    } 

    /**
     * Creates the GUI components and creates the GUI, with the help of
     * FileSelectorLayout, a customized layout manager, created with
     * "JavaGUIBuilder".
     * 
     * @param driver -
     *            The Driver (the main executioner of the program) is passed as
     *            parameter because several calls are made from this method to
     *            modify the "state" of the Driver.
     */
    public TestModelFrame() {

        // sets the look and feel of the GUI.
        this.setLookAndFeel();

        this.setTitle("Test A Saved Model");

        TestModelLayout customLayout = new TestModelLayout();

        getContentPane().setFont(new Font("Arial", Font.PLAIN, 10));
        getContentPane().setLayout(customLayout);

        loadModelButton = new JButton("Select Model (.model)");
        loadModelButton.addActionListener(this);
        getContentPane().add(loadModelButton);
        loadModelButton.setEnabled(true);
        
        loadModelField = new JTextField("");
        getContentPane().add(loadModelField);
        loadModelField.setEditable(false);        
        loadModelField.setEnabled(true);
        
        loadTestButton = new JButton("Select Testing File (.xls)");
        loadTestButton.addActionListener(this);
        getContentPane().add(loadTestButton);
        loadTestButton.setEnabled(true);
        
        loadTestField = new JTextField("");
        getContentPane().add(loadTestField);
        loadTestField.setEditable(false);        
        loadTestField.setEnabled(true);
        
        goButton = new JButton("GO!");
        goButton.addActionListener(this);
        getContentPane().add(goButton);
        goButton.setEnabled(false);        

        this.pack();
        this.setResizable(false);

        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
        int width = this.getWidth();
        int height = this.getHeight();
        int left = (screen.width - width) / 2;
        int top = (screen.height - height) / 2;
        setBounds(left, top, width, height);

        this.setVisible(true);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
//               System.exit(0);
            	e.getWindow().dispose();
            }
        });
    } //End of constructor

    /*
     * Catches all events in the FileSelector. (non-Javadoc)
     * 
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getSource().equals(this.loadModelButton)) 
        {
        	File m = this.getModelFile();
//        	if(this.modelFile==null)
        		this.modelFile = m;
        	
        	try
        	{
        		if(this.modelFile!=null)
        		{
	        		String path = modelFile.getAbsolutePath();
	        		// get the model file name
	        		this.dimName = modelFile.getName();
	        		// remove the ".model" extension
	        		this.dimName = this.dimName.replaceAll("\\.model$", "");
	        		// remove the last underscore ("_") and the string
	        		// that follows it (the user name)
	        		this.dimName = this.dimName.replaceAll("_.*?$", "");
//	        		System.out.println("Dim Name is: " + this.dimName);
//	        		if(dimName.indexOf("=") > 0)
//	        			dimName = dimName.split("=")[1];
	        		this.loadModelField.setText(path);
        		}
        		else
        		{
        			this.loadModelField.setText("");
        		}
        		enableGoButton();        		
        	}
        	catch(Exception ex)
        	{
//                JOptionPane.showMessageDialog(this,
//                        "Cannot load the specified file", "Error",
//                        JOptionPane.ERROR_MESSAGE);
                return;        		
        	}
        } 
        else if (e.getSource().equals(this.loadTestButton)) 
        {        	        	
        	File f= this.getTestFile();
        	
//        	if(this.testDataFile ==null)
        		this.testDataFile = f;
        	try
        	{
        		if(this.testDataFile!=null)
        		{
	        		String path = this.testDataFile.getAbsolutePath();
	        		this.loadTestField.setText(path);
        		}
        		else
        		{
        			this.loadTestField.setText("");
        		}
        		enableGoButton();        		
        	}
        	catch(Exception ex)
        	{
//                JOptionPane.showMessageDialog(this,
//                        "Cannot load the specified file", "Error",
//                        JOptionPane.ERROR_MESSAGE);
                return;        		
        	}        	
        }
        else if(e.getSource().equals(this.goButton))
        {
//        	ProgressDialog pDialog = new ProgressDialog(null, this.testDataFile.getName());        	
//        	SavedModelEvaluator evaluator = new SavedModelEvaluator(this.modelFile, this.dimName);
//        	evaluator.setUp(pDialog, this.testDataFile);
//        	evaluator.run(pDialog);
        	Runn runn = new Runn(this.modelFile, this.testDataFile, this.dimName, this);
        	runn.start();
        }
    }
    
    private void enableGoButton()
    {
    	if(this.modelFile != null && (this.testDataFile !=null))
    		this.goButton.setEnabled(true);
    }

  
    public void mouseClicked(MouseEvent e) {

    }

    public void mousePressed(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    public void mouseReleased(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    public void mouseEntered(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    public void mouseExited(MouseEvent e) {
        // TODO Auto-generated method stub

    }
    
    public File getModelFile() {

        File inputFile = null;
        ExampleFileFilter filter = new ExampleFileFilter();
        filter.addExtension(TestModelFrame.MODEL_FILE_EXT);
        JFileChooser fc = new JFileChooser();
        fc.setFileFilter(filter);
        fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fc.setDialogTitle("Choose " + TestModelFrame.MODEL_FILE_EXT + " file");
        fc.setCurrentDirectory(new File(System.getProperty("user.dir")+"/CLASSIFIERS"));
         //TODO figure out what's going on here        
        fc.setMultiSelectionEnabled(true);
        
        int returnVal = fc.showOpenDialog(null);
        if (returnVal != JFileChooser.APPROVE_OPTION) {
            return null;
        } else {
            inputFile = fc.getSelectedFile();
            if (!inputFile.isFile()) {
                return null;
            }
        }

            return inputFile;
    }
    
    public File getTestFile() {

        File inputFile = null;
        ExampleFileFilter filter = new ExampleFileFilter();
        filter.addExtension(TestModelFrame.DATA_FILE_EXT);
        filter.addExtension("csv");
        JFileChooser fc = new JFileChooser();
        fc.setFileFilter(filter);
        fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fc.setDialogTitle("Choose " + TestModelFrame.DATA_FILE_EXT+ " file to process");
        fc.setCurrentDirectory(new File(System.getProperty("user.dir")+"/sample_data"));
         //TODO figure out what's going on here        
        fc.setMultiSelectionEnabled(true);
        
        int returnVal = fc.showOpenDialog(null);
        if (returnVal != JFileChooser.APPROVE_OPTION) {
            return null;
        } else {
            inputFile = fc.getSelectedFile();
            if (!inputFile.isFile()) {
                return null;
            }
        }

        // if (validateFile(inputFile)) {
            return inputFile;
        // } else {
        //    return null;
        // }
    }  
    
    public class Runn extends SwingWorker {
    	
        File modelFile;
        File testDataFile;
        String dimName;
        private TestModelFrame gui;

        public Runn(File modelFile, File testDataFile, String dimName, TestModelFrame gui) 
        {
        	this.modelFile = modelFile;
        	this.testDataFile = testDataFile;
        	this.dimName = dimName;
        	this.gui = gui;
        }

        public Object construct() {
        	
        	ProgressDialog pDialog = new ProgressDialog(null, this.testDataFile.getName());        	
        	SavedModelEvaluator evaluator= new SavedModelEvaluator(this.modelFile, this.dimName, gui);
        	int flag = evaluator.setUp(pDialog, this.testDataFile);
        	if(flag == 0)
        		evaluator.run(pDialog);        	

        	return null;
        }
    }    
    
}
