/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package edu.cmu.taghelpertools.gui.extractor;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Insets;
import java.awt.LayoutManager;

import javax.swing.*;
import java.awt.*;

import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.border.LineBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.plaf.basic.BasicSplitPaneUI;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumnModel;
import javax.swing.text.DefaultHighlighter;

import com.aliasi.coref.Matcher;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.BufferedReader;
import java.io.FileReader;
import edu.cmu.taghelpertools.middle_layer.*;
import edu.cmu.taghelpertools.middle_layer.extractor.*;
import gnu.trove.*;

import java.util.*;
import java.util.List;
import java.util.regex.Pattern;

/**
 * This is the main GUI for users to manipualte the feature extraction function.
 * @author haochuan
 *
 */
public class ExtractorFrame extends JFrame implements ActionListener, MouseListener{

    private JPanel mainPanel;    
    private JButton addNewFeatureBtn;
    private JTextArea editingTextField;

    private JList functionList;
    private JList uniList;
    private JList bigramList;
    private JList posList;
    private JList otherList;
    private JList userDefinedList;
    private FunctionListModel functionListModel;
    private UniListModel uniListModel;
    private BigramListModel bigramListModel;
    private POSListModel posListModel;
    private OtherFeatureListModel otherListModel;
    private UserDefinedListModel userDefinedListModel;
    
    private JPanel filterPanel;
    private JLabel stopwordLabel;
    private ButtonGroup stopwordBG;
    private JRadioButton removeStopwordYesRB;
    private JRadioButton removeStopwordNoRB;
    private JLabel stemLabel;
    private ButtonGroup stemBG;
    private JRadioButton stemNoRB;
    private JRadioButton stemYesRB;
    private JLabel thresholdLabel;
	private JTextField frequencyThreshold;
    private JButton filterJButton;
    
    private FeatureSpaceMapBuilder fsmBuilder;
    private FeatureSelector featureSelector;
    private OptionsModel optionsModel;
    
    private SimpleVisualization visualizationGUI;

    private boolean isRemoveStopwords;
    private boolean isStem;
    private int freqThreshold = 1;
       
    
    private static int globalCounter = 1;
    
    public ExtractorFrame(FeatureSpaceMapBuilder fsmBuilder, OptionsModel optionsModel) 
    {
        super("TagHelper Feature Extractor");     
        this.fsmBuilder = fsmBuilder;
        this.featureSelector = fsmBuilder.getFeatureSelector();
        this.optionsModel = optionsModel;
        
        setSize(new Dimension(400,500));
        setLayout(new BorderLayout());      
        this.mainPanel = new JPanel();
        this.mainPanel.setLayout(new ExtractorLayout());
        this.mainPanel.setFont(new Font("Arial", Font.PLAIN, 12));
        
        this.addNewFeatureBtn = new JButton("+");
        this.addNewFeatureBtn.setFont(new Font("Arial", Font.PLAIN, 9));
        this.addNewFeatureBtn.addActionListener(this);
        
        
        
        this.editingTextField = new JTextArea();
        this.filterPanel = new JPanel(new FilterLayout());
        this.filterPanel.setSize(new Dimension(115, 218));
        
        setResizable(false);
        init();        
        initFeatures();
        pack();
        setVisible(true); 

       
//        initVisualization();
    }

    private void init() {    
        JLabel titleLabel = new JLabel("Feature List");
        JLabel uniLabel = new JLabel("Unigram");        
        JLabel bigramLabel = new JLabel("Bigram");        
        JLabel posLabel = new JLabel("POS Bigram");
        JLabel otherLabel = new JLabel("-Ext Feature List-");
        JLabel userLabel = new JLabel("User Defined");
        this.uniListModel = new UniListModel(this);
        this.bigramListModel = new BigramListModel(this);
        this.posListModel = new POSListModel(this);
        this.otherListModel = new OtherFeatureListModel(this);
        this.userDefinedListModel = new UserDefinedListModel(this, this);
        
        this.uniList = this.uniListModel.getJList();
        this.bigramList = this.bigramListModel.getJList();       
        this.posList = this.posListModel.getJList();          
        this.otherList = this.otherListModel.getJList(); 
        this.userDefinedList = this.userDefinedListModel.getJList();
        
        this.mainPanel.add(uniLabel);
        this.mainPanel.add(new JScrollPane(this.uniList));
        this.mainPanel.add(bigramLabel);
        this.mainPanel.add(new JScrollPane(this.bigramList));
        this.mainPanel.add(posLabel);
        this.mainPanel.add(new JScrollPane(this.posList));
        this.mainPanel.add(otherLabel);
        this.mainPanel.add(new JScrollPane(this.otherList));     

        
        this.mainPanel.add(userLabel);
        this.mainPanel.add(new JScrollPane(this.userDefinedList));
        this.mainPanel.add(new JSeparator( JSeparator.HORIZONTAL ));
        this.mainPanel.add(new JSeparator( JSeparator.HORIZONTAL ));
        
        //JLabel functionLabel = new JLabel("<Operator>");        
        this.functionListModel = new FunctionListModel(this);
        this.functionList = functionListModel.getJList();        
        //this.mainPanel.add(functionLabel);
        this.mainPanel.add(new JScrollPane(functionList));
        
        this.editingTextField.setLineWrap(true);
        this.editingTextField.setBorder(new TitledBorder(new EtchedBorder(Color.GRAY, Color.LIGHT_GRAY),
        												"Editor",
        												1,0,new Font("Arial", Font.PLAIN, 12), Color.DARK_GRAY));
        this.editingTextField.setForeground(Color.BLUE);
        this.mainPanel.add(this.editingTextField);
        this.mainPanel.add(this.addNewFeatureBtn);
        
//filter panel
        TitledBorder filterTitle;
        filterTitle = BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "OPTIONS");
        filterPanel.setBorder(filterTitle);
 
        this.stopwordBG = new ButtonGroup();
        this.stemBG = new ButtonGroup();

        this.isRemoveStopwords = optionsModel.isRemoveStopwords();
        this.isStem = optionsModel.isStem();
        
        this.stopwordLabel = new JLabel("Remove Stopwords?");
        this.filterPanel.add(stopwordLabel);
        this.removeStopwordYesRB = new JRadioButton("Yes", this.isRemoveStopwords);
        this.stopwordBG.add(removeStopwordYesRB);
        this.filterPanel.add(removeStopwordYesRB);
        this.removeStopwordNoRB = new JRadioButton("No", !this.isRemoveStopwords);
        stopwordBG.add(removeStopwordNoRB);
        this.filterPanel.add(removeStopwordNoRB);


        this.stemLabel = new JLabel("Stem?");
        this.filterPanel.add(stemLabel);       
        this.stemYesRB = new JRadioButton("Yes", this.isStem);
        this.stemBG.add(stemYesRB);
        this.filterPanel.add(stemYesRB);
        this.stemNoRB = new JRadioButton("No", !this.isStem);
        this.stemBG.add(stemNoRB);
        this.filterPanel.add(stemNoRB);
        
        
        this.thresholdLabel = new JLabel("Frequency Threshold");
        this.filterPanel.add(this.thresholdLabel);
        
        this.filterPanel.add(new JLabel(">="));
        
        if( optionsModel.isRemoveRareFeatures())
        	this.freqThreshold = optionsModel.getFrequencyThreshold();
       
        
        this.frequencyThreshold = new JTextField(String.valueOf(this.freqThreshold));
        this.frequencyThreshold.setColumns(7);
        this.frequencyThreshold.addFocusListener(new FocusListener() {
            public void focusGained(FocusEvent e) {
                if (frequencyThreshold.getText().length() == 0)
                    frequencyThreshold.setText(String.valueOf(freqThreshold));
            }

            public void focusLost(FocusEvent e) {
                try {
                    int num = Integer.parseInt(frequencyThreshold.getText());
                    if (!(num > 0)) {
                        JOptionPane.showMessageDialog(null, "Must enter a number > 0.", "INVALID ACTION", JOptionPane.ERROR_MESSAGE);
                        frequencyThreshold.setText(String.valueOf(freqThreshold));
                        SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                frequencyThreshold.requestFocus();
                                //tabbedPane.setSelectedIndex(0);
                            }
                        });
                    }
                } catch (NumberFormatException e1) {
                    JOptionPane.showMessageDialog(null, "Must enter numeric value.", "INVALID ACTION", JOptionPane.ERROR_MESSAGE);
                    frequencyThreshold.setText(String.valueOf(freqThreshold));
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            frequencyThreshold.requestFocus();
                            //tabbedPane.setSelectedIndex(0);
                        }
                    });
                }
            }
        });

        this.filterPanel.add(frequencyThreshold);
        this.filterJButton = new JButton("Update");
        this.filterJButton.addActionListener(this);
        this.filterPanel.add(this.filterJButton);

        this.mainPanel.add(this.filterPanel);
        getContentPane().add(this.mainPanel);
       
    }
    
    private void initFeatures()
    {
    	THashMap globalFeatureMap = this.fsmBuilder.getFeatureMap();
    	Iterator iter = this.fsmBuilder.getFeatureSetInterator(true);
    	Pattern p ;//= Pattern.compile("(THF_)([a-zA-Z]+_)(.+)");
    	
    	if( this.isStem && this.isRemoveStopwords)
    		p = Pattern.compile("(THF_)([UB]StemRmstopw_)(.+)");
    	else if ( !this.isStem && this.isRemoveStopwords)
    		p = Pattern.compile("(THF_)([UB]NostemRmstopw_)(.+)");
    	else if ( !this.isStem && !this.isRemoveStopwords)
    		p = Pattern.compile("(THF_)([UB]NostemNormstopw_)(.+)");
    	else
    		p = Pattern.compile("(THF_)([UB]StemNormstopw_)(.+)");
    	  	
    	
    	while(iter.hasNext())
    	{
    		
    		String featureName = (String)iter.next();
    		//System.out.println("ExtractionFrame: " + featureName);
    		FeatureGlobalProperty prop = (FeatureGlobalProperty)globalFeatureMap.get(featureName);
    		if(prop.getCount() >= this.freqThreshold)
    		{
    			if(prop.isPOS())
    		
	    		{
	    			this.posListModel.addElement(featureName.substring(8));
        		}
	    		else
	    		{
	    			if(!prop.isNonCountable())
	    			{
	    				java.util.regex.Matcher m = p.matcher(featureName);
	    	    		if (m.find())
	    	    		{//strip off prefix
	    	    			String thirdToken = m.group(3).trim();
	    	    			featureName = thirdToken;
			    			if(prop.isBigram())
			    				this.bigramListModel.addElement(featureName);
			    			else
			    				this.uniListModel.addElement(featureName);
	    	    		}

	    			}
	    		}
    		}
    	}
    	
    	Iterator iter2 = this.featureSelector.getGlobalUserDefinedFeatureSetIterator();
    	while(iter2.hasNext())
    	{
    		String featureName = (String)iter2.next();    		
    		FeatureGlobalProperty prop = (FeatureGlobalProperty)this.featureSelector.getGlobalUserDefinedFeature().get(featureName);
    		this.userDefinedListModel.addElement(featureName, prop.getExpression());
    	} 

    	//this.otherList = this.otherListModel.getJList();
    }    
    
    
    
    
    public void createVisualizationforAllfeatures(List<String> features)
    {
    	createAndShowVisualization("");
    	
    	int dimNum = DataPoint.dimNum;
    	for(int i=0; i< dimNum; i++)
    	{
    		this.visualizationGUI.addColumn(this.optionsModel.getInputFileReader().getNameForDim(i), false, i);
    	}

    	this.visualizationGUI.addColumn("text", true, dimNum); 
    	
    	String[] featureNames = new String[features.size()-1];
    	HashMap<String, Integer> featureCount = new HashMap<String, Integer>();
    	for(int i=1;i <features.size();i++)
    	{
    		//System.out.println(features.get(i));
    		String featureName = features.get(i);
    		Pattern p = Pattern.compile("(.+)(=)(.+)");
			java.util.regex.Matcher m = p.matcher(featureName);
			if(m.find())
				featureName = m.group(1).trim();
    		//System.out.println(featureName);
    		featureNames[i-1] = featureName;
    		this.visualizationGUI.addColumn(featureName, false, dimNum+i);
    		featureCount.put(featureName, 0);
    	}

		ArrayList<DataPoint> dataPoints = this.optionsModel.getPreprocessedTrainingDataPoints();    		
		for(int i=0; i<dataPoints.size(); i++)
		{
			DataPoint tmpPoint = dataPoints.get(i);	
			Object[] rowData = new Object[DataPoint.dimNum+1+featureNames.length];
			
			for(int j=0; j<rowData.length; j++)
			{
				if(j < DataPoint.dimNum) //class
				{
					rowData[j] = tmpPoint.getValueAtDim(j);
				}
				else if(j==DataPoint.dimNum)
					rowData[j] = tmpPoint.getText();
				else
				{
					String fName = featureNames[(j-DataPoint.dimNum)-1];
					if(tmpPoint.isMatchedUserDefined(fName))
					{	
						rowData[j] = "1";
						if(featureCount.containsKey(fName))
							featureCount.put(fName, featureCount.get(fName)+1);
					}
					else
					{
						rowData[j] = "0";
					}
				}
			}
			this.visualizationGUI.addRow(rowData);
		}   

		//System.out.println(featureCount.toString());
		this.visualizationGUI.setFeatureCount(featureCount);
		
    }

//    public void createVisualization(String featureName, String expression)
//    {   
//        createAndShowVisualization(expression);
//    	int indexOfText =  DataPoint.dimNum;
//        for (int i = 0; i < DataPoint.dimNum; i++) 
//        {
//            this.visualizationGUI.addColumn(this.optionsModel.getInputFileReader().getNameForDim(i), false, i);
//        }
//        
//        this.visualizationGUI.addColumn("text", true, indexOfText); 
//        
//        
//		ArrayList<DataPoint> dataPoints = this.optionsModel.getPreprocessedTrainingDataPoints();    		
//		for(int i=0; i<dataPoints.size(); i++)
//		{
//			DataPoint tmpPoint = dataPoints.get(i);	
//			Object[] rowData = new Object[DataPoint.dimNum+1];
//			
//			for(int j=0; j<rowData.length; j++)
//			{
//				if(j < DataPoint.dimNum) //class
//				{
//					rowData[j] = tmpPoint.getValueAtDim(j);
//				}
//				else
//				{
//					if(tmpPoint.isMatchedUserDefined(featureName))
//					//if(tmpPoint.isMatchedUserDefined(expression))
//					{	
//
//						rowData[j] = this.highlight(tmpPoint.getText());
//					}
//					else
//					{
//						rowData[j] = "<html>"+tmpPoint.getText()+"</html>";
//					}
//				}
//			}
//			this.visualizationGUI.addRow(rowData);
//		}   
//		
		
//    	int column =  DataPoint.dimNum; // the text column    	
//		ArrayList<DataPoint> dataPoints = this.optionsModel.getPreprocessedTrainingDataPoints();    		
//		for(int i=0; i<dataPoints.size(); i++)
//		{
//			DataPoint tmpPoint = dataPoints.get(i);	
//			if(tmpPoint.isMatchedUserDefined(featureName))
//			{
//				String content = this.highlight(tmpPoint.getText());
//				this.visualizationGUI.updateCell(content, i, column);
//			}
//			else
//			{
//				this.visualizationGUI.updateCell(tmpPoint.getText(), i, column);				
//			}
//		} 
		
		
		
//		FeatureSpaceMap fsm = tmpPoint.getFeatureSpaceMap();
//				
//		Set<Integer> highlightSet = new TreeSet<Integer>();
//		
//		Iterator synItr= fsm.getSynKeyIterator();
//		while(synItr.hasNext())
//		{
//			String featureName = (String)synItr.next();
//			OperationList opList = fsm.getOplist(featureName);
//			
//			for(int i=0; i<opList.size(); i++)
//			{
//				SynFeatureNode featureNode= (SynFeatureNode)opList.get(i);
//				Iterator posItr = featureNode.getPositionIterator();
//				
//				for(int j=0; posItr.hasNext(); j++)
//				{
//					int pos = ((Integer)posItr.next()).intValue();
//					int length = featureNode.getLength(pos);
//					for(int k=0; k<length; k++)
//						highlightSet.add(new Integer(pos+k));
//				}
//			}			
//		}
//			
//		String aValue = doHighlight(tmpPoint.getText(), highlightSet);
//		this.visualizationGUI.updateCell(aValue, indexOfDataPoint, column);
//    }    
    

    private String highlight(String text)
    {
//        DefaultHighlighter high = new DefaultHighlighter();
//        
//        DefaultHighlighter.DefaultHighlightPainter
//        highlight_painter = new   DefaultHighlighter.DefaultHighlightPainter(new  Color(5, 5, 5, 10));
//        Object highlight_text = null;
//           try
//           {
//               highlight_text = high.addHighlight(0, text.length(), highlight_painter);
//           }
//           catch (Exception e)
//           {
//
//               e.printStackTrace();
//           }
    	
    	
    	String left = "<html><font color='green'>";
    	String right = "</font></html>";
    	
    	return left + text + right;
    }
    
    public void removeUserDefinedFeatureFromModel(String featureName)
    {
		FeatureExtractionHandler.setParameters(this.optionsModel.getPreprocessedTrainingDataPoints(), 
				   								this.featureSelector)	;
		FeatureExtractionHandler.removeFeature(featureName);
    }
    
    public void actionPerformed(ActionEvent e) 
    {   
    	if(e.getSource()== this.filterJButton)
    	{
    		//System.out.println("filterJButton");
    		
    		this.freqThreshold = Integer.parseInt(frequencyThreshold.getText());
    		this.isRemoveStopwords = this.removeStopwordYesRB.isSelected();
    		this.isStem = this.stemYesRB.isSelected();
    		
    		this.uniListModel.clearAll();
    		this.posListModel.clearAll();
    		this.bigramListModel.clearAll();
    		this.otherListModel.clearAll();
        	this.otherListModel.readFile();
        	this.otherListModel.initModel();
    		this.initFeatures();
    		
    	}
    	if(e.getSource()== this.addNewFeatureBtn)
    	{
    		//parse the expression, execute feature extraction
    		String expression = this.editingTextField.getText().trim();
    		if(expression.equals(""))
    			return; 
    		
    		//validate the grammar first
            String featureName = expression.split("=")[0].trim();    			
			int flag = this.userDefinedListModel.checkElement(featureName, expression); 
			
			if(!featureName.startsWith("U:"))
			{
					JOptionPane.showMessageDialog(null, "Cannot parse '"+featureName+"'. Please check.", "SYNTAX ERROR", JOptionPane.ERROR_MESSAGE);
				
			}
			else
			{
				
					
	    		try{    			
	    			if(flag==0)
	    			{    	
	//    				expression = expression+";";
	    				new ThtParsercheck(new edu.cmu.taghelpertools.middle_layer.extractor.ThtYylex(new java.io.StringReader(expression))).parse();    			
	    				this.userDefinedListModel.addElement(featureName, expression);
	    				this.editingTextField.setText("");
	    				this.featureSelector.addGlobalUserDefiendFeature(featureName, expression);
	    			}
	    		}
	    		catch(TokenException ex)
	    		{
	//    			ex.printStackTrace();
	    			JOptionPane.showMessageDialog(null, "Cannot parse '"+expression+"'. Please check.", "Illegal character", JOptionPane.ERROR_MESSAGE);
	    			return;
	    		}
	    		catch(Exception ex)
	    		{
	//    			ex.printStackTrace();
	    			JOptionPane.showMessageDialog(null, "Cannot parse '"+expression+"'. Please check.", "SYNTAX ERROR", JOptionPane.ERROR_MESSAGE);
	    			return;
	    		}
	    		
	    		if(flag==0)
	    		{
		    		//do actual processing
		    		ArrayList<DataPoint> dataPoints = this.optionsModel.getPreprocessedTrainingDataPoints();    		
		    		for(int i=0; i<dataPoints.size(); i++)
		    		{
		    			DataPoint tmpPoint = dataPoints.get(i);
		    			FeatureExtractionHandler2.setParameters(tmpPoint, this.featureSelector);
		    			try{
		    				new ThtParser(new edu.cmu.taghelpertools.middle_layer.extractor.ThtYylex(new java.io.StringReader(expression))).parse();
		    			}
		    			catch (TokenException ex) {
		        			ex.printStackTrace();
		        			//this.removeUserDefinedFeatureFromModel(featureName);
		        			JOptionPane.showMessageDialog(null, "Error on processing data point. index= "+i+". Please check.", "Illegal character", JOptionPane.ERROR_MESSAGE);
		        			return;
		    			}
		    			catch(Exception ex)
		    			{
		        			ex.printStackTrace();
		        			JOptionPane.showMessageDialog(null, "Error on processing data point. index= "+i+". Please check.", "SYNTAX ERROR", JOptionPane.ERROR_MESSAGE);
		        			continue;
		    			}
		    		}
	
	    		}
    		
			}
    	}
    	
//    	Iterator t = this.featureSelector.getFeatureMap().keySet().iterator();
//    	while(t.hasNext())
//    	{
//    		System.out.println( "test: "+(String)t.next());
//    		
//    	}
    	
    	

    }
    
    public void mouseClicked(MouseEvent e) {
//    	if(e.getSource() == this.userDefinedList && e.isPopupTrigger())
//    	{
//    		maybeShowPopup(e);
//    	}    	
    	if (e.getButton()==MouseEvent.BUTTON1 && e.getClickCount() >= 2) 
    	{
				
    		if(e.getSource() == this.userDefinedList && this.userDefinedList.getModel().getSize() > 1)
    		{    
            	JList source = (JList)e.getSource(); 
            	int index= source.getSelectedIndex();            	
							// System.out.println("Calling getExpressionLight with "+index+" (1)");

    			this.editingTextField.setText(this.userDefinedListModel.getExpressionLight(index));
    			return;
    		}
    		  		
    		
        	if(editingTextField.getText().indexOf("=")<0 && e.getSource() == this.functionList)
        	{
        		//System.out.println(editingTextField.getText());
        			this.editingTextField.setText(this.getAdhocFeatureName()+" = ");
        	}
        	
        	JList source = (JList)e.getSource();        	
        	String value = "";
        	
    		if(e.getSource() == this.otherList)
    		{    
            	int index= source.getSelectedIndex();            	
							// System.out.println("Calling getExpressionLight with "+index+" (2)");
    			value = this.otherListModel.getExpressionLight(index);
    		}
    		else
    		{
    			value = (String)source.getSelectedValue();
    		}
    		
        	int caretPos = this.editingTextField.getCaretPosition();
        	this.editingTextField.insert(value, caretPos);
        	
        	this.editingTextField.setCaretPosition(this.planCaretPosition(this.editingTextField.getText(), ""));
        	/*int tmpLeftPaPos = this.editingTextField.getText().indexOf("(")+1;//value.indexOf("(")+oldLength+1 ;
        	if(tmpLeftPaPos> 0)
        	{
        		this.editingTextField.setCaretPosition(tmpLeftPaPos);
        	}
        	else
        	{
        		this.editingTextField.setCaretPosition(this.editingTextField.getText().length()-1);
        	}*/
        }
                
    }
    
//    public void removeUserDefinedFeature(String featureName)
//    {
//		FeatureExtractionHandler.setParameters(this.optionsModel.getPreprocessedTrainingDataPoints(), 
//											   this.ptrFeatureSelector)	;     	
//    }
    
    private int locateComma(String content, int startPosition)
    {
    	int tmpIndex = content.indexOf(",", startPosition);
    	if(tmpIndex >0)
    	{
    		String subContent = content.substring(startPosition+1, tmpIndex);
    		String[] lfs = subContent.split("\\(");
    		String[] rts = subContent.split("\\)");
    		int numPas = lfs.length - 1 + rts.length-1;
    		if(numPas%2 !=0 && lfs.length%2 ==0)
    		{
    			return tmpIndex;
    		}
    		else
    		{
    			return locateComma(content, tmpIndex+1);
    		}
    	}	
    	else
    	{
    		return Integer.MIN_VALUE;
    	}
    }
    
    //TODO check the algorithm again
    public int planCaretPosition(String content, String upperLevelContent)
    {       	
    	int tmpLeftPaPos = content.indexOf("(");//value.indexOf("(")+oldLength+1 ;
    	int tmpRightPaPos = content.lastIndexOf(")");
    	
    	if(tmpLeftPaPos > 0 && tmpRightPaPos >0)
    	{
        	String subContent = content.substring(tmpLeftPaPos+1, tmpRightPaPos);    		
	    	if(subContent.trim().equals("") || subContent.trim().equals(","))
	    	{
	    		return tmpLeftPaPos+1;
	    	}
	    	else
	    	{	
	    		int tmpCommaPos = this.locateComma(content, 0);//content.lastIndexOf(",", tmpLeftPaPos+1);
	    		
	    		if(tmpCommaPos > 0)
	    		{
			    		if(content.substring(tmpCommaPos+1, tmpRightPaPos).equals(""))
			    		{
			    			return content.indexOf(",", tmpLeftPaPos+1)+1;
			    		}
			    		else
			    		{
			    			System.out.println("~~"+subContent);
			    			return tmpLeftPaPos+1+planCaretPosition(subContent, content);
			    		}
	    		}
	    		else
	    		{
	    			System.out.println("=="+subContent);
	    			return tmpLeftPaPos+1+planCaretPosition(subContent, content);	    			
	    		}
	    	}
    	}
    	else
    	{
    		return content.toCharArray().length;
//        	if(upperLevelContent.trim().equals(""))
//        		return upperLevelContent.toCharArray().length;
//        	else
//        	{
//        		System.out.println("Upper: "+upperLevelContent);
//        		return planCaretPosition(upperLevelContent, "");
//        	}
    	}
    }
    
    public String getAdhocFeatureName()
    {
    	String prefix = "U:DEF";
    	return prefix+(ExtractorFrame.globalCounter++);
    }

    public UserDefinedListModel getUserDefinedListModel()
    {
    	return this.userDefinedListModel;
    }
    
    
    public void mousePressed(MouseEvent e) {
        // TODO Auto-generated method stub
    	maybeShowPopup(e);
    }

    public void mouseReleased(MouseEvent e) {
        // TODO Auto-generated method stub
    	//if(e.getSource() == this.userDefinedList )
    		maybeShowPopup(e);
    }

    public void mouseEntered(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    public void mouseExited(MouseEvent e) {
        // TODO Auto-generated method stub

    }    
    
    private void maybeShowPopup(MouseEvent e) {
    	// do popup menu handling correctly for platform independent behavior
    	// the position of the popup menu is slightly offset from the mouse
    	// position since on a Mac, for some reason the first menu item "delete"
    	// gets clicked on immediately if this is not done.
        if (e.getSource() == this.userDefinedList && this.userDefinedList.getModel().getSize() > 1 && e.isPopupTrigger()) {
        	// set the selected index of the feature list
        	this.userDefinedListModel.setIndex(e);
            this.userDefinedListModel.popup.show(e.getComponent(),
                       e.getX()+2, e.getY()+2);
        }
    }

    private void createAndShowVisualization(String featureName) {
        //Create and set up the window.
        //JFrame frame = new JFrame("Visualization: "+ featureName);
    	JFrame frame = new JFrame("Feature Visualization"+ featureName);
    	JPanel mainPanel, menuPanel;
    	TitledBorder menuTitle;
    	ButtonGroup sortBG;
    	JRadioButton sortInc, sortDec; 
    	JButton exportButton;
    	
    	
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        BorderLayout borderLayout = new BorderLayout();
        
        mainPanel = new JPanel(borderLayout);
        menuPanel = new JPanel();
        
        this.visualizationGUI = new SimpleVisualization(this.optionsModel, this);
        this.visualizationGUI .setOpaque(true); //content panes must be opaque
        mainPanel.add(this.visualizationGUI, BorderLayout.CENTER);
         
        menuTitle = BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "FILTER");
        menuPanel.setBorder(menuTitle);
        menuPanel.setPreferredSize(new Dimension(150, 400));
        
        menuPanel.setLayout(new MenuLayout());
        
//Sort features by frequency        
        menuPanel.add(new JLabel("Sort features"));
        
        sortBG = new ButtonGroup();
        sortInc = new JRadioButton("Increasing order", false);
        sortDec = new JRadioButton("Decreasing order", false);
        sortBG.add(sortInc);
        sortBG.add(sortDec);
        sortInc.addActionListener(this.visualizationGUI);
        sortDec.addActionListener(this.visualizationGUI);
        
        
        
        menuPanel.add(sortInc);
        menuPanel.add(sortDec);
        
//Export the data as csv file
        exportButton = new JButton("Export .CSV");
        menuPanel.add(exportButton);
        exportButton.addActionListener(this.visualizationGUI);
        
        mainPanel.add(menuPanel, BorderLayout.WEST);
        frame.setContentPane(mainPanel);
        
//Display the window.
        frame.setResizable(true);
        frame.pack();
        frame.setVisible(true);
    }    
}

class FilterLayout implements LayoutManager {

    public FilterLayout() {
    }

    public void addLayoutComponent(String name, Component comp) {
    }

    public void removeLayoutComponent(Component comp) {
    }

    public Dimension preferredLayoutSize(Container parent) {
        Dimension dim = new Dimension(0, 0);

        Insets insets = parent.getInsets();
        dim.width = 115;
        dim.height = 218;

        return dim;
    }

    public Dimension minimumLayoutSize(Container parent) {
        Dimension dim = new Dimension(0, 0);
        return dim;
    }

    public void layoutContainer(Container parent) {
        Insets insets = parent.getInsets();
        Component c;
        c = parent.getComponent(0);
        c.setBounds(insets.left,		insets.top,	100,		20);  
        c = parent.getComponent(1);
        c.setBounds(insets.left,		insets.top+25,	50,		20);
        c = parent.getComponent(2);
        c.setBounds(insets.left+55,		insets.top+25,	50,		20);
        
        c = parent.getComponent(3);
        c.setBounds(insets.left+30,		insets.top+50,	100,	20);  
        c = parent.getComponent(4);
        c.setBounds(insets.left,		insets.top+75,	50,		20);
        c = parent.getComponent(5);
        c.setBounds(insets.left+55,		insets.top+75,	50,		20);
        
        c = parent.getComponent(6);
        c.setBounds(insets.left,		insets.top+100,	110,	20); 
        c = parent.getComponent(7);
        c.setBounds(insets.left+10,		insets.top+120,	50,		20); 
        c = parent.getComponent(8);
        c.setBounds(insets.left+40,		insets.top+120,	50,		20); 
        
        c = parent.getComponent(9);
        c.setBounds(insets.left+10,		insets.top+160,	80,		20); 
        
    }
}


class MenuLayout implements LayoutManager {

    public MenuLayout() {
    }

    public void addLayoutComponent(String name, Component comp) {
    }

    public void removeLayoutComponent(Component comp) {
    }

    public Dimension preferredLayoutSize(Container parent) {
        Dimension dim = new Dimension(0, 0);

        Insets insets = parent.getInsets();
        dim.width = 150;
        dim.height = 400;

        return dim;
    }

    public Dimension minimumLayoutSize(Container parent) {
        Dimension dim = new Dimension(0, 0);
        return dim;
    }

    public void layoutContainer(Container parent) {
        Insets insets = parent.getInsets();
        Component c;
        c = parent.getComponent(0);
        c.setBounds(insets.left+10,		insets.top,	100,		20);  
        c = parent.getComponent(1);
        c.setBounds(insets.left+10,		insets.top+25,	150,		20);
        c = parent.getComponent(2);
        c.setBounds(insets.left+10,		insets.top+55,	150,		20);
        c = parent.getComponent(3);
        c.setBounds(insets.left+18,		insets.top+350,	100,	20);  
//        c = parent.getComponent(4);
//        c.setBounds(insets.left,		insets.top+75,	50,		20);
//        c = parent.getComponent(5);
//        c.setBounds(insets.left+55,		insets.top+75,	50,		20);
//        
//        c = parent.getComponent(6);
//        c.setBounds(insets.left,		insets.top+100,	110,	20); 
//        c = parent.getComponent(7);
//        c.setBounds(insets.left+10,		insets.top+120,	50,		20); 
//        c = parent.getComponent(8);
//        c.setBounds(insets.left+40,		insets.top+120,	50,		20); 
//        
//        c = parent.getComponent(9);
//        c.setBounds(insets.left+10,		insets.top+160,	80,		20); 
        
    }
}
