package edu.cmu.taghelpertools.gui.extractor;

/*
 * SimpleTableDemo.java requires no other files.
 */

import javax.swing.*;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableCellEditor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent; 
import java.awt.*;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;

import edu.cmu.taghelpertools.gui.ExampleFileFilter;
import edu.cmu.taghelpertools.middle_layer.*;
import edu.cmu.taghelpertools.middle_layer.extractor.*;

import java.util.*;
import java.util.List;
import java.util.regex.Pattern;

public class SimpleVisualization extends JPanel  implements ActionListener{
    private boolean DEBUG = false;
    private DefaultTableModel  myModel = null;
    public JTable myTable = null;
    public JPopupMenu popup = new JPopupMenu();
	private JMenuItem deleteItem = new JMenuItem("delete");
    private OptionsModel optionsModel = null;
    private ExtractorFrame extractorFrame = null;
    
    private String selectedFeatureName=null;
    private int vColIndex;
    private HashMap<String, Integer> featureCount;
    
    
    
    public SimpleVisualization(OptionsModel optionsModel, ExtractorFrame extractorFrame) {
        super(new GridLayout(1,0));
        this.setLookAndFeel();
        this.myTable = new JTable(){
            //Implement table header tool tips. 
            protected JTableHeader createDefaultTableHeader() {
                return new JTableHeader(columnModel) {
                    public String getToolTipText(MouseEvent e) {
                        String tip = null;
                        java.awt.Point p = e.getPoint();
                        int index = columnModel.getColumnIndexAtX(p.x);
                        //int realIndex = columnModel.getColumn(index).getModelIndex();
                        String colName = "";
                        if(index>=0 && index<table.getColumnCount())
                        	colName = table.getColumnName(index).trim();
                        if(featureCount.containsKey(colName))
                        	return featureCount.get(colName).toString();
                        else
                        	return null;
                    }
                };
            }    
        };
        
        this.myTable.setModel(this.initTableModel());
        //this.myTable.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
        this.myTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        this.myTable.setPreferredScrollableViewportSize(new Dimension(400, 400));
        this.myTable.setFont(new Font("Arial", Font.PLAIN, 12));
        //this.myTable.setDefaultRenderer(String.class, new LineWrapCellRenderer());
        this.myTable.setAutoCreateColumnsFromModel(false);
        JTableHeader myTableHeader = this.myTable.getTableHeader();
        myTableHeader.addMouseListener(new ColumnHeaderListener(this));
        
        this.optionsModel = optionsModel;    
        this.extractorFrame = extractorFrame;
		
            
        if (DEBUG) {
            myTable.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    printDebugData(myTable);
                }
            });
        }

        //Create the scroll pane and add the table to it.
        JScrollPane scrollPane = new JScrollPane(this.myTable);

        //Add the scroll pane to this panel.
        add(scrollPane);
        
        //initiate popup menu
    	
		deleteItem.addActionListener(this);
	    this.popup.add(deleteItem);
	
        
        
    }

    private DefaultTableModel initTableModel () 
    { 
        if (this.myModel ==null) 
        { 
            this.myModel= new MyDefaultTableModel() 
            { 
                public boolean isCellEditable ( int row, int colum ) 
                { 
                	return true;
                } 
                
                public Class getColumnClass(int c) {
                    return getValueAt(0, c).getClass();
                }                
            }; 
        }
        return this.myModel; 
    } 
    
    /**
     * Set the look and feel of the GUI to the OP specific look and feel,
     * instead of the Java default look and feel
     */
    private void setLookAndFeel() {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) {
            try {
                UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
            } catch (Exception e1) {
                e1.printStackTrace();
                System.exit(1);
            }
        }
    }
    
    public void addColumn(String columnName, int index)
    {
    	addColumn(columnName, false, index);
    }
    
    
    /**
     * 
     * @param columnName
     */
    public void addColumn(String columnName, boolean isLineWrap, int index)
    {
    	if(this.myModel !=null)
    	{

    		//if(isLineWrap)
    			//this.myTable.getColumnModel().getColumn(index).setCellRenderer(new DefaultTableCellRenderer());
    		//	this.myTable.getColumnModel().getColumn(index).setCellRenderer(new LineWrapCellRenderer());
    		this.myModel.addColumn(columnName);
    		
    		TableColumn c = (isLineWrap) ?
    				new TableColumn(index, 75, new LineWrapCellRenderer(), new ButtonEditor(this.optionsModel, this))
    		      : new TableColumn(index);
    		c.setHeaderValue(columnName);
    		myTable.getColumnModel().addColumn(c);
    		/*
    		if(isLineWrap)
    		{
    			//System.out.println(index + " - " + columnName);
    			//myTable.getColumnModel().getColumn(index).setCellRenderer(new LineWrapCellRenderer());
    			//myTable.getColumnModel().getColumn(index).setCellEditor(new ButtonEditor(this.optionsModel, this));
    			TableColumn c = 
    			c.setHeaderValue(columnName);
    			myTable.getColumnModel().addColumn();
    		}
    		else {
    			myTable.getColumnModel().addColumn();
    		}*/
    		//System.out.println(columnName + " " + index + " has renderor ");
    		//System.out.println(columnName + " has renderor " + myTable.getColumnModel().getColumn(index).getCellRenderer().toString());
    		this.myTable.repaint();
    	}
    }
    
    /**
     * 
     * @param rowDatap 
     */
    public void addRow(Object[] rowData)
    {
    	if(this.myModel !=null)
    	{
    		this.myModel.addRow(rowData);
    		this.myTable.repaint();
    	}    	
    }
    
    public void updateCell(Object aValue, int row, int column)
    {
    	if(this.myTable!=null)
    	{
    		myTable.setValueAt(aValue, row, column);
    		myTable.repaint();
    	}
    }
    
    private void printDebugData(JTable table) {
        int numRows = table.getRowCount();
        int numCols = table.getColumnCount();
        javax.swing.table.TableModel model = table.getModel();

        System.out.println("Value of data: ");
        for (int i=0; i < numRows; i++) {
            System.out.print("    row " + i + ":");
            for (int j=0; j < numCols; j++) {
                System.out.print("  " + model.getValueAt(i, j));
            }
            System.out.println();
        }
        System.out.println("--------------------------");
    }

	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub
		if(e.getSource() == this.deleteItem)
		{
			System.out.println("Delete this feature: " + this.selectedFeatureName);
			
			//remove the feature from table
			removeColumnAndData(myTable, vColIndex);
			
			
			//remove the adv feature from TH feature model
			extractorFrame.removeUserDefinedFeatureFromModel(this.selectedFeatureName);
			UserDefinedListModel udm = extractorFrame.getUserDefinedListModel();
			int index = -1;
			for(int i=0; i<udm.dlm.getSize(); i++)
			{
				String exp = (String)udm.dlm.get(i);
				//System.out.println(exp);
				Pattern p = Pattern.compile("(.+)(=)(.+)");
				java.util.regex.Matcher m = p.matcher(exp);
				if(m.find() && m.group(1).trim().equals(this.selectedFeatureName))
					 index = i;
			}
				
			System.out.println(index);
			if(index>=0)
			{	udm.dlm.removeElementAt(index);
				udm.toolTips.remove(index);
				udm.uniqueFeatures.remove(this.selectedFeatureName);
				udm.jlist.repaint();
			}

		}

		else if (e.getActionCommand().equals("Increasing order"))
		{
			//System.out.println(e.getActionCommand());
		    TableColumnModel colModel = myTable.getColumnModel();
		    Vector sortedVector = getSortedVector(featureCount);
		    //System.out.println(sortedVector);
		    for(int i=0; i<sortedVector.size();i++)
		    {
		    	//yTable.geth
		    	String feaName = sortedVector.get(i).toString();
		    	for(int j=0; j<myTable.getColumnCount();j++)
		    	{
		    		String colName = myTable.getColumnName(j);
		    		if(colName.equals(feaName))
		    		{
		    			colModel.moveColumn(j, myTable.getColumnCount()-1);
		    			break;
		    		}
		    	}
		    
			}
		    //colModel.moveColumn(3, 1);
		    myTable.repaint();
		    
		
		}
		else if (e.getActionCommand().equals("Decreasing order"))
		{
			//System.out.println(e.getActionCommand());
		    TableColumnModel colModel = myTable.getColumnModel();
		    Vector sortedVector = getSortedVector(featureCount);
		    //System.out.println(sortedVector);
		    for(int i=sortedVector.size()-1; i>=0;i--)
		    {
		    	
		    	String feaName = sortedVector.get(i).toString();
		    	for(int j=0; j<myTable.getColumnCount();j++)
		    	{
		    		String colName = myTable.getColumnName(j);
		    		if(colName.equals(feaName))
		    		{
		    			colModel.moveColumn(j, myTable.getColumnCount()-1);
		    			break;
		    		}
		    	}
		    
			}
		    //colModel.moveColumn(3, 1);
		    myTable.repaint();
		}
		else if (e.getActionCommand().equals("Export .CSV"))
		{
			exportTable();
		}
	}

	public Vector getSortedVector(HashMap<String, Integer> hmap)
	{
		Vector sortedFeatureName = new Vector();
		//HashMap map = new LinkedHashMap();
		//List mapKeys = new ArrayList(hmap.keySet());
		List mapValues = new ArrayList(hmap.values());
		//hmap.clear();
		TreeSet sortedSet = new TreeSet(mapValues);
		Object[] sortedArray = sortedSet.toArray();
		int size = sortedArray.length;
//		a) Ascending sort
 
		for (int i=0; i<size; i++)
		{
			int count = Integer.parseInt(sortedArray[i].toString());
			//System.out.println(count);
			Iterator iter = featureCount.keySet().iterator();
	        while (iter.hasNext()) {
	            String feature = iter.next().toString();
	            if (hmap.containsKey(feature)) {
	            	 //System.out.println(feature);
	                 if( count == (hmap.get(feature).intValue()))
	                	 sortedFeatureName.add(feature);
	                
	            } 
	        }
			
 
		}
		return sortedFeatureName;
	}


	public void setSelectedFeatureName(String featureName)
	{
		this.selectedFeatureName = featureName;
	}

	public void setColumnIndex(int vColIndex) {
		// TODO Auto-generated method stub
		this.vColIndex = vColIndex;
		//this.mColIndex = mColIndex;
	}
	
	public void removeColumnAndData(JTable table, int vColIndex) {
        MyDefaultTableModel model = (MyDefaultTableModel)table.getModel();
        TableColumn col = table.getColumnModel().getColumn(vColIndex);
        int columnModelIndex = col.getModelIndex();
        Vector data = model.getDataVector();
        Vector colIds = model.getColumnIdentifiers();
    
        // Remove the column from the table
        table.removeColumn(col);
    
        // Remove the column header from the table model
        colIds.removeElementAt(columnModelIndex);
    
        // Remove the column data
        for (int r=0; r<data.size(); r++) {
            Vector row = (Vector)data.get(r);
            row.removeElementAt(columnModelIndex);
        }
        model.setDataVector(data, colIds);
    
        // Correct the model indices in the TableColumn objects
        // by decrementing those indices that follow the deleted column
        Enumeration<TableColumn> e = table.getColumnModel().getColumns();
        for (; e.hasMoreElements(); ) {
            TableColumn c = (TableColumn)e.nextElement();
            if (c.getModelIndex() >= columnModelIndex) {
                c.setModelIndex(c.getModelIndex()-1);
            }
        }
        model.fireTableStructureChanged();
    }
	
	public void exportTable()
	{
		String[] columnNames=new String[myTable.getColumnCount()];
		for (int i=0;i<columnNames.length;i=i+1)
		{
			 columnNames[i]=myTable.getColumnModel().getColumn(i).getHeaderValue().toString();
		}
		 
		try
		{
			System.out.println("Exporting Table ");
			JFileChooser fc = new JFileChooser();
			fc.showSaveDialog(this);
		    String fileName = fc.getSelectedFile().getAbsolutePath();
		    //System.out.println(fileName);
			String data;
			if (fileName != null)
			{
				try
				{
					BufferedWriter bufferedWriter = new BufferedWriter(new FileWriter(fileName+".csv"));
					PrintWriter fileWriter = new PrintWriter(bufferedWriter);
					//System.out.println("Column Headers ");
					for(int i=0; i<myTable.getColumnCount(); ++i)
					{
						data=columnNames[i]; 
						fileWriter.print(data+",");
					}
			        //System.out.println("Done.... ");
			 		fileWriter.println("");
					System.out.println("Exporting Data: " + fileName+".csv");
					for(int i=0; i<myTable.getRowCount(); ++i)
					{
						for(int j=0; j<myTable.getColumnCount(); ++j)
						{
							data = myTable.getValueAt(i,j).toString();
							fileWriter.print(data+",");
						}
						fileWriter.println("");
					}	
					fileWriter.close();
			        System.out.println("Done.... ");
			     //   System.out.println("File Name" + fileName);
				}catch(Exception e)
				{
					JOptionPane.showMessageDialog(null, "Error "+e);
				}
			}
	
		}
		catch (Exception E)
		{
		      System.out.println("Error While Exporting Table "+E);
		 
		}
	}//export Table
	
	public void setFeatureCount(HashMap fCount)
	{
		this.featureCount = fCount;
	}
}






//class LineWrapCellRenderer extends DefaultTableCellRenderer {
class LineWrapCellRenderer extends JTextArea implements TableCellRenderer {
	
    private Color unselectedForeground;
    private Color unselectedBackground;
    
    public void setForeground(Color c) {
        super.setForeground(c);
        unselectedForeground = c;
    }

    public void setBackground(Color c) {
        super.setBackground(c);
        unselectedBackground = c;
    }                   
	
	public LineWrapCellRenderer() {
      setLineWrap(true);
      setWrapStyleWord(true);
      setFont(new Font("Arial", Font.PLAIN, 12));
    	//setContentType("text/plains");    	    	
   }
    
   public boolean getScrollableTracksViewportWidth() {
	   return true;
   }
    
  public Component getTableCellRendererComponent(JTable table, Object
          value, boolean isSelected, boolean hasFocus, int row, int column) {
	       
	  
	  if (isSelected) {
          super.setForeground(table.getSelectionForeground());
          super.setBackground(table.getSelectionBackground());
       }
       else {
           super.setForeground((unselectedForeground != null) ? unselectedForeground
                                                              : table.getForeground());
           super.setBackground((unselectedBackground != null) ? unselectedBackground
                                                              : table.getBackground());
       }

	  
	  int maxPreferredHeight = 0;
      for (int i = 0; i < table.getColumnCount(); i++) {
          setText("" + table.getValueAt(row, i));
          setSize(table.getColumnModel().getColumn(column).getWidth(), 0);
          maxPreferredHeight = Math.max(maxPreferredHeight, getPreferredSize().height);
      }

      if (table.getRowHeight(row) != maxPreferredHeight)  
          table.setRowHeight(row, maxPreferredHeight);
      
      //table.setRowHeight(row, 100);
      setText(value == null ? "" : value.toString());
      return this;
	  
//      text = text.replaceAll("<LESSTHAN>", "&lt;LESSTHAN&gt;");
//      text = text.replaceAll("<GREATERTHAN>", "&lt;GREATERTHAN&gt;");
//      text = text.replaceAll("<COMMA>", "&lt;COMMA&gt;");
//      text = text.replaceAll("<PERIOD>", "&lt;PERIOD&gt;");
//      text = text.replaceAll("<SEMICOLON>", "&lt;SEMICOLON&gt;");
//      text = text.replaceAll("<COLON>", "&lt;COLON&gt;");
//      text = text.replaceAll("<DASH>", "&lt;DASH&gt;");
//      text = text.replaceAll("<QUESTIONMARK>", "&lt;QUESTIONMARK&gt;");
//      text = text.replaceAll("<EXCLAMATIONMARK>", "&lt;EXCLAMATIONMARK&gt;");
//      text = text.replaceAll("<ASTERISK>", "&lt;ASTERISK&gt;");
//      text = text.replaceAll("<AMPERSAND>", "&lt;AMPERSAND&gt;");
//      text = text.replaceAll("<ATSIGN>", "&lt;ATSIGN&gt;");
//      text = text.replaceAll("<NUMBERSIGN>", "&lt;NUMBERSIGN&gt;");
//      text = text.replaceAll("<PERCENTSIGN>", "&lt;PERCENTSIGN&gt;");
//      text = text.replaceAll("<OPENPAREN>", "&lt;OPENPAREN&gt;");
//      text = text.replaceAll("<CLOSEPAREN>", "&lt;CLOSEPAREN&gt;");
//      text = text.replaceAll("<OPENCURLY>", "&lt;OPENCURLY&gt;");
//      text = text.replaceAll("<CLOSECURLY>", "&lt;CLOSECURLY&gt;");
//      text = text.replaceAll("<DOUBLEQUOTE>", "&lt;DOUBLEQUOTE&gt;");
//      text = text.replaceAll("<DOLLAR>", "&lt;DOLLAR&gt;");
//      text = text.replaceAll("<BACKSLASH>", "&lt;BACKSLASH&gt;");
//      text = text.replaceAll("<FORWARDSLASH>", "&lt;FORWARDSLASH&gt;");
//      text = text.replaceAll("<PLUSSIGN>", "&lt;PLUSSIGN&gt;");
//      text = text.replaceAll("<TILDE>", "&lt;TILDE&gt;");
//      text = text.replaceAll("<CARAT>", "&lt;CARAT&gt;");
//      text = text.replaceAll("<LEFTSQUAREPAREN>", "&lt;LEFTSQUAREPAREN&gt;");
//      text = text.replaceAll("<RIGHTSQUAREPAREN>", "&lt;RIGHTSQUAREPAREN&gt;");
      
//	  setText((String)value);
//
//      //setSize(table.getColumnModel().getColumn(column).getWidth(),
//      //        getPreferredSize().height);
//	  System.out.println(table.getRowHeight(row)+ "   " +getPreferredSize().height);
//      if (table.getRowHeight(row) < getPreferredSize().height) {
//          table.setRowHeight(row, getPreferredSize().height);
//    	  System.out.println("xxxxxxxxxxxxx");
//      }
 //     return this;
  }
} 

class ButtonEditor extends AbstractCellEditor
implements TableCellEditor, ActionListener {
	CustomButton button;
	String label;
	OptionsModel optionsModel;
	static final String SHOW = "show";
	Component mainGUI;

	public ButtonEditor(OptionsModel optionsModel, Component c) {
		this.optionsModel = optionsModel;
		button = new CustomButton();
		button.setActionCommand(SHOW);
		button.addActionListener(this);
		button.setBorderPainted(true);
		
		mainGUI = c;
	}

	public void actionPerformed(ActionEvent e) {
		if (SHOW.equals(e.getActionCommand())) {
		int row = ((CustomButton)e.getSource()).getIndexOfRow();
		
		showTokenizedContent(row);
		fireEditingStopped(); //Make the renderer reappear.
		}
	}

//	Implement the one CellEditor method that AbstractCellEditor doesn't.
	public Object getCellEditorValue() {
		return label;
	}

//	Implement the one method defined by TableCellEditor.
	public Component getTableCellEditorComponent(JTable table,
			Object value,
			boolean isSelected,
			int row,
			int column) 
	{
			label = (String)value;
			button.setText(label);
			button.setIndexOfRow(row);

			return button;
	}
	
	private void showTokenizedContent(int row)
	{
		ArrayList<DataPoint> points = this.optionsModel.getPreprocessedTrainingDataPoints();
		DataPoint tmpPoint = points.get(row);
//		StringBuffer tokenized_uni= new StringBuffer();
//		StringBuffer tokenized_bi = new StringBuffer();
//		StringBuffer tokenized_posbi = new StringBuffer();
//		FeatureSpaceMap fsm  = tmpPoint.getFeatureSpaceMap();
//		Iterator itr = fsm.getOrdKeyIterator();
//		while(itr.hasNext())
//		{
//			String feature = (String)itr.next();
//			if(fsm.isBigram(feature))
//			{
//				if(fsm.isPOS(feature))
//				{
//					tokenized_posbi.append(feature).append(" ");
//				}
//				else
//				{
//					tokenized_bi.append(feature).append(" ");					
//				}
//			}
//			else
//			{
//				tokenized_uni.append(feature).append(" ");				
//			}
//		}
		ScrollDialog.showDialog(mainGUI, mainGUI, tmpPoint.getText()+"\n\n"+
												  tmpPoint.getTaggedResult().trim()+"\n\n",
												  //tokenized_uni.toString()+"\n\n"+
												  //tokenized_bi.toString()+"\n\n"+
												  //tokenized_posbi.toString(),
												  "Info");
		
	}
}

class CustomButton extends JButton
{
	private int indexOfRow;
	
	public void setIndexOfRow(int row)
	{
		indexOfRow = row;
	}
	public int getIndexOfRow()
	{
		return indexOfRow;
	}	
}


class ScrollDialog extends JDialog implements ActionListener{
	private static ScrollDialog dialog;
	private static String value = "";

	public static String showDialog(Component frameComp,
			Component locationComp,
			String content,
			String title) {
		Frame frame = JOptionPane.getFrameForComponent(frameComp);
		dialog = new ScrollDialog(frame,
				locationComp,
				content,
				title);
		dialog.setVisible(true);
		//dialog.repaint();
		return value;
	}

	private ScrollDialog(Frame frame,
			Component locationComp,
			String content,
			String title) {
		super(frame, title, true);

//		Create and initialize the buttons.

		final JButton setButton = new JButton("OK");
		setButton.setActionCommand("OK");
		setButton.addActionListener(this);
		getRootPane().setDefaultButton(setButton);
		JTextArea tx = new JTextArea(content);
		tx.setLineWrap(true);
		JScrollPane scroller = new JScrollPane(tx);
		scroller.setPreferredSize(new Dimension(300, 200));
		scroller.setAlignmentX(LEFT_ALIGNMENT);

//		Create a container so that we can add a title around
//		the scroll pane.  Can't add a title directly to the
//		scroll pane because its background would be white.
//		Lay out the label and scroll pane from top to bottom.
		JPanel listPane = new JPanel();
		listPane.setLayout(new BoxLayout(listPane, BoxLayout.PAGE_AXIS));
		listPane.add(Box.createRigidArea(new Dimension(0,5)));
		listPane.add(scroller);
		listPane.setBorder(BorderFactory.createEmptyBorder(3,3,3,3));

//		Lay out the buttons from left to right.
		JPanel buttonPane = new JPanel();
		buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.LINE_AXIS));
		buttonPane.setBorder(BorderFactory.createEmptyBorder(0, 3, 3, 3));
		buttonPane.add(Box.createHorizontalGlue());
		buttonPane.add(Box.createRigidArea(new Dimension(10, 0)));
		buttonPane.add(setButton);

//		Put everything together, using the content pane's BorderLayout.
		Container contentPane = getContentPane();
		contentPane.add(listPane, BorderLayout.CENTER);
		contentPane.add(buttonPane, BorderLayout.PAGE_END);

//		Initialize values.
		pack();
		setLocationRelativeTo(locationComp);
	}

//	Handle clicks on the Set and Cancel buttons.
	public void actionPerformed(ActionEvent e) {
		if ("OK".equals(e.getActionCommand())) {
			this.dispose();
		}
	}
}


class ColumnHeaderListener extends MouseAdapter {
    
	SimpleVisualization sv;
	public ColumnHeaderListener(SimpleVisualization visualization) {
		// TODO Auto-generated constructor stub
		sv = visualization;
	}
	public void mousePressed(MouseEvent evt) 
	{
    	showPopup(evt);
    }
    public void mouseReleased(MouseEvent evt) 
    {
    	showPopup(evt);
    }
   
    private void showPopup(MouseEvent evt)
    {   
    	if (evt.isPopupTrigger())
    	{
	        //System.out.println("In ColumnHeaderListener~");
		    JTable table = ((JTableHeader)evt.getSource()).getTable();
		    TableColumnModel colModel = table.getColumnModel();
		
		    // The index of the column whose header was clicked
		    int vColIndex = colModel.getColumnIndexAtX(evt.getX());
		    int mColIndex = table.convertColumnIndexToModel(vColIndex);
		

		    // Return if not clicked on any column header
		    if (vColIndex == -1) {
		        return;
		    }
		
		    // Determine if mouse was clicked between column heads
		    Rectangle headerRect = table.getTableHeader().getHeaderRect(vColIndex);
		    if (vColIndex == 0) {
		        headerRect.width -= 3;    // Hard-coded constant
		    } else {
		        headerRect.grow(-3, 0);   // Hard-coded constant
		    }
		    if (!headerRect.contains(evt.getX(), evt.getY())) {
		        // Mouse was clicked between column heads
		        // vColIndex is the column head closest to the click
		
		        // vLeftColIndex is the column head to the left of the click
		        int vLeftColIndex = vColIndex;
		        if (evt.getX() < headerRect.x) {
		            vLeftColIndex--;
		        }
		    }
		    
		    
		    //only feature column can be deleted
		    String colName = table.getColumnName(vColIndex).trim();
		    if(!colName.startsWith("U:"))
		    	return;
		    sv.setSelectedFeatureName(colName);
		    sv.setColumnIndex(vColIndex);
		    sv.popup.show(evt.getComponent(), evt.getX()+2, evt.getY()+2);
    	}
    }
}

// This subclass adds a method to retrieve the columnIdentifiers
// which is needed to implement the removal of
// column data from the table model
class MyDefaultTableModel extends DefaultTableModel {
    public Vector getColumnIdentifiers() {
        return columnIdentifiers;
    }
}



///**
//* Create the GUI and show it.  For thread safety,
//* this method should be invoked from the
//* event-dispatching thread.
//*/
//private static void createAndShowGUI() {
//  //Create and set up the window.
//  JFrame frame = new JFrame("SimpleVisualization");
//  frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
//
//  //Create and set up the content pane.
//  SimpleVisualization newContentPane = new SimpleVisualization(null);
//  newContentPane.setOpaque(true); //content panes must be opaque
//  frame.setContentPane(newContentPane);     
//  //Display the window.
//  frame.pack();
//  frame.setVisible(true);
//}
//
//public static void main(String[] args) {
//  //Schedule a job for the event-dispatching thread:
//  //creating and showing this application's GUI.
//  javax.swing.SwingUtilities.invokeLater(new Runnable() {
//      public void run() {
//          createAndShowGUI();
//      }
//  });
//}