/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package edu.cmu.taghelpertools.gui.extractor;

import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.ListSelectionModel;
import javax.swing.DefaultListSelectionModel;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;

import com.aliasi.coref.Matcher;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.*;
import java.util.regex.Pattern;

/**
 * This GUI-related class is responsible for building and maintaining a JList for User Defined Features.
 * @author haochuan
 *
 */
public class UserDefinedListModel implements ActionListener{

	public DefaultListModel dlm;
	public JList jlist;
	public List<String> toolTips;	
	private String[] defaultSyntheticList= new String[]{""};
	public TreeSet uniqueFeatures = new TreeSet();
	public JPopupMenu popup = new JPopupMenu();
	private JMenuItem deleteItem = new JMenuItem("delete");
	private JMenuItem displayItem = new JMenuItem("display");
	private ExtractorFrame extractorFrame;
	// private int savedIndex = -1;

	private int oldSelIndex = -1;
	private int newSelIndex = -1;

	public UserDefinedListModel(MouseListener m, ExtractorFrame ef)
	{
		this.dlm =  new DefaultListModel();
		this.extractorFrame = ef;
		this.toolTips = new ArrayList<String>();		
		this.jlist = new JList(this.dlm){
				public String getToolTipText(MouseEvent evt)
				{
					int index = locationToIndex(evt.getPoint());
					return toolTips.get(index);
				}
			};

		/* The SingleSelectionModel class calls updateSingleSelection() each 
		 * time the selection changes.  We override that method here to 
		 * update the old/new textfields.
		 */
		ListSelectionModel selectionModel = new SingleSelectionModel() {
				public void updateSingleSelection(int oldIndex, int newIndex) {
					// System.out.println("oldIndex: "+oldIndex+" newIndex: "+newIndex);
					oldSelIndex = oldIndex;
					newSelIndex = newIndex;
				}
		};
		this.jlist.setSelectionModel(selectionModel);

		this.jlist.addMouseListener(m);
		this.jlist.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		this.jlist.setDragEnabled(true);
	
		deleteItem.addActionListener(this);
		displayItem.addActionListener(this);		
		this.popup.add(deleteItem);
		this.popup.add(displayItem);		
		
		initModel();
	}
	
	public int getIndex(MouseEvent evt)
	{
		int index = this.jlist.locationToIndex(evt.getPoint());
		return index;
	}

	/**
	 * Set the selected item in the list of features and save
	 * the index for use in right-click handling
	 * 
	 * @param evt The mouse event to get co-ordinate details
	 */
	public void setIndex(MouseEvent evt)
	{
		int index = this.jlist.locationToIndex(evt.getPoint());
		this.jlist.setSelectedIndex(index);
		// this.savedIndex = index;
	}

//	public DefaultListModel getFunctionListModel()
//	{
//		return this.dlm;
//	}
	
	public JList getJList()
	{
		return this.jlist;
	}
	
	public String getExpressionLight(int index)
	{
		return this.toolTips.get(index);
	}
	
	public int checkElement(String functionString, String expression)
	{
		if(this.uniqueFeatures.contains(functionString))
		{
			JOptionPane.showMessageDialog(null, functionString + " already exists. Please check.", "Feature Name Conflict", JOptionPane.ERROR_MESSAGE);
			return 1;
		}

		// get the feature definition
		String definition = expression.split("=", 2)[1];
		definition = definition.trim();
		definition = definition.replaceAll(" ", "");
		// next statement assumes that the first left parenthesis begins the operands
		// in the feature definition, and that the last character is the ending
		// right parenthesis
		definition = definition.substring(definition.indexOf("(")+1, definition.length()-1);
		String [] operands = definition.split(",");
		for (int i = 0; i < operands.length; i++) {
			if (operands[i].startsWith("U:")) {
				if (!this.uniqueFeatures.contains(operands[i])) {
					JOptionPane.showMessageDialog(null, "Definition of \"" + functionString + "\" refers to \"" + 
							operands[i] + "\" which is not yet defined!", "Undefined Feature Reference", JOptionPane.ERROR_MESSAGE);
					return 1;
				}
			}
		}
		
		return 0;
		
//		if(this.uniqueFeatures.contains(functionString))
//		{
//            Object[] options = { "yes", "no" };
//            int n = JOptionPane.showOptionDialog(null, "Do you wish to replace the exisitng feature?", "Warning", JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE, null, options, options[0]);
//            if (n == 0) {
//    			//int index = this.jlist.getSelectedIndex();
//    			//String featureName = this.jlist.getSelectedValue().toString();
//    			//String featureName = this.jlist.getSelectedValue().toString().split("=")[0].trim();
//    			//TODO actually remove stuffs from the datastructure (featureSpaceMap of each data point and the global memory)
//    			//this.extractorFrame.removeUserDefinedFeatureFromModel(featureName);
//    			//this.dlm.removeElementAt(index);
//    			//this.toolTips.remove(index);
//            	//int index = thi
//            	
//            	
//            	
//    			//this.jlist.repaint();
//            	return 0;
//            }
//            else
//            {
//            	return 1;
//            }
//
//		}
//		else
//		{
//			return 0;
//		}
		
		
		
	}
	
	public int getNumItems()
	{
		return this.toolTips.size();
	}
	
	public void addElement(String functionString, String expression)
	{
		if(this.uniqueFeatures.contains(functionString))
		{
              
                for(int i=0; i<toolTips.size(); i++)
                {
                	if(toolTips.get(i).startsWith(functionString))
                	{
                		toolTips.remove(i);
                		toolTips.add(i, expression);
                	}
                }
		}
		else
		{
			if(functionString.equals(""))
				this.dlm.addElement(functionString);
			else
				this.dlm.addElement(expression);
			this.uniqueFeatures.add(functionString);
			this.toolTips.add(expression);
			this.jlist.repaint();
		}
	}
	
	private void initModel()
	{
		for(int i=0; i<this.defaultSyntheticList.length; i++)
		{
			String word = this.defaultSyntheticList[i];
			this.addElement(word, "");
		}
	}
	public void actionPerformed(ActionEvent e)
	{
		if(e.getSource() == this.deleteItem)
		{
			int index = this.jlist.getSelectedIndex();
 			if (index == -1) {
 					// System.out.println("INFO: Nothing selected in list!\n");

					// setting the selected index to the last one selected
					this.jlist.setSelectedIndex(newSelIndex);
					index = newSelIndex;
 			}
 
 			String featureName = this.jlist.getSelectedValue().toString().split("=")[0].trim();
			
			//TODO actually remove stuffs from the datastructure (featureSpaceMap of each data point and the global memory)
			this.extractorFrame.removeUserDefinedFeatureFromModel(featureName);
			this.dlm.removeElementAt(index);
			this.toolTips.remove(index);
			this.uniqueFeatures.remove(featureName);
			this.jlist.repaint();
		}
    	else if(e.getSource() == this.displayItem)
    	{
			// int index = this.jlist.getSelectedIndex();
			// Pattern p = Pattern.compile("(.+)(=)(.+)");
			
			// System.out.println("Selected index BEFORE is " + this.jlist.getSelectedIndex());
			// System.out.println("Saved index is " + this.savedIndex);
			
			// workaround for Mac OS X control-click behavior. The problem is that
			// if a list item is already selected, the control-clicking it de-selects
			// that item and the "getSelectedValue" call below throws an exception
			//if (this.jlist.getSelectedIndex() == -1) {
			//	this.jlist.setSelectedIndex(this.savedIndex);
			//	index = this.savedIndex;
			//}
			// System.out.println("Selected index AFTER is " + this.jlist.getSelectedIndex());
		
			//String featureName = this.jlist.getSelectedValue().toString();//.split("=")[0].trim();
			//System.out.println("UDL - actionperform display:  " + featureName);
			//java.util.regex.Matcher m = p.matcher(featureName);
			//if(m.find())
			//	featureName = m.group(1).trim();
			//System.out.println("UDL - actionperform display:  " + this.getExpressionLight(index));
			//System.out.println("xxx  "+featureName + "  "+ this.getExpressionLight(index));
			//this.extractorFrame.createVisualization(featureName, this.getExpressionLight(index));
			this.extractorFrame.createVisualizationforAllfeatures(this.toolTips);
    	}
	}

		/**
		 * Create a SINGLE_SELECTION ListSelectionModel that calls a new 
		 * method, updateSingleSelection(), each time the selection
		 * changes.  This can be a little bit more convienent than using the 
		 * ListModels ListSelectionListener, since ListSelectionListeners
		 * are only given the range of indices that the change spans.
		 */
		class SingleSelectionModel extends DefaultListSelectionModel 
		{
				public SingleSelectionModel() {
					setSelectionMode(SINGLE_SELECTION);
				}

				public void setSelectionInterval(int index0, int index1) {
			int oldIndex = getMinSelectionIndex();
			super.setSelectionInterval(index0, index1);
			int newIndex = getMinSelectionIndex();
			if (oldIndex != newIndex) {
					updateSingleSelection(oldIndex, newIndex);
			}
				}

				public void updateSingleSelection(int oldIndex, int newIndex) {
				}
		}


//    public void mouseClicked(MouseEvent e) {
//
//    }
//    
//    public void mousePressed(MouseEvent e) {
//        // TODO Auto-generated method stub
//    	
//    }
//
//    public void mouseReleased(MouseEvent e) {
//        // TODO Auto-generated method stub
//
//    }
//
//    public void mouseEntered(MouseEvent e) {
//        // TODO Auto-generated method stub
//
//    }
//
//    public void mouseExited(MouseEvent e) {
//        // TODO Auto-generated method stub
//
//    }      
}
