/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package edu.cmu.taghelpertools.io;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;

/**
 * A Generic File Reader
 * 
 * @author Jaime Arguello
 */
public class Reader {

    private String filename;
    private BufferedReader in;

    public Reader(String filename) {
        this.filename = filename;
        try {
            in = new BufferedReader(new InputStreamReader(new FileInputStream(filename), "UTF-8"));
        } catch (IOException e) {
            e.printStackTrace();
            System.exit(0);
        }
    }

    public void close() {
        try {
            in.close();
        } catch (IOException e) {
            e.printStackTrace();
            System.exit(0);
        }
    }

    public int countLines() {
        int line = 0;
        while (this.readLine() != null) {
            line++;
        }
        try {
            in = new BufferedReader(new FileReader(new File(filename)));
        } catch (IOException e) {
            e.printStackTrace();
            System.exit(0);
        }
        return line;
    }

    public String getFilename() {
        return filename;
    }

    public String read() {
        String content = "";
        String line = "";
        try {
            while ((line = in.readLine()) != null) {
                content += (line.trim() + "\n");
            }
        } catch (IOException e) {
            e.printStackTrace();
            System.exit(0);
        }
        return content;
    }


    public String readLine() {
        String line = "";
        try {
            if ((line = in.readLine()) != null) {
                return line;
            } else {
                return null;
            }
        } catch (IOException e) {
            e.printStackTrace();
            System.exit(0);
        }
        return null;
    }


    public String readAsHTML() {
        String content = "";
        String line = "";
        try {
            while ((line = in.readLine()) != null) {
                content += (line.trim() + "<br>");
            }
        } catch (IOException e) {
            e.printStackTrace();
            System.exit(0);
        }
        return content;
    }

    public java.io.Reader getReader() {
    	return (java.io.Reader) in;
    }
}