/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package edu.cmu.taghelpertools.middle_layer;

import java.io.File;

import weka.core.FastVector;
import weka.core.Instances;


/**
 * This Model Module holds all the meta-data associated with a Classifier: it's
 * name, it's mod file, dictionary file, and it's list of labels/classes it
 * classifies into.
 * 
 * @author Jaime Arguello
 * 
 */
public class ClassifierModel {


    private static final String INDENT = "         ";
    public static final String CLASSIFIER_START = "<classifier>";
    public static final String CLASSIFIER_END = "</classifier>";
    private static final String CLASSIFIER_MODEL = "@Classifier_Model";
    private static final String CLASSIFIER_NAME = "@Classifier_Name";
    private static final String DICTIONARY = "@Dictionary";
    private static final String CLASS_LABELS = "@Class_Labels";

    private String name;
    private File modFile;
    private File dicFile;
    private FastVector labels;
    private Instances instances;


    public void setInstances(Instances ins)
    {
    	this.instances = ins;
    }
    
    public Instances getInstances()
    {
    	return this.instances;
    }
    
    public void setName(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public File getModFile() {
        return modFile;
    }

    public void setModFile(File modFile) {
        this.modFile = modFile;
    }

    public File getDicFile() {
        return dicFile;
    }

    public void setDicFile(File dicFile) {
        this.dicFile = dicFile;
    }

    public FastVector getLabels() {
        return labels;
    }

    public void setLabels(FastVector labels) {
        this.labels = labels;
    }

    public String toString() {
        String result = "";
        String username = System.getProperty("user.name");
        result += ClassifierModel.CLASSIFIER_START + "\n";
        result += ClassifierModel.INDENT + ClassifierModel.CLASSIFIER_NAME + " " + this.name + "_" + username + "\n";
        result += ClassifierModel.INDENT + ClassifierModel.CLASSIFIER_MODEL + " " + this.modFile.getName() + "\n";
        result += ClassifierModel.INDENT + ClassifierModel.DICTIONARY + " " + this.dicFile.getName() + "\n";
        result += ClassifierModel.INDENT + ClassifierModel.CLASS_LABELS + " ";
        for (int i = 0; i < this.labels.size(); i++) {
            String label = labels.elementAt(i).toString();
            result += label + " ";
        }
        result += "\n";
        result += ClassifierModel.CLASSIFIER_END;
        return result;
    }
}