/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *    
 */

/*
 * Java command line parsercourtesy of John Lloyd
 */
	

package edu.cmu.taghelpertools.middle_layer;

import edu.cmu.taghelpertools.gui.FileSelector;
import edu.cmu.taghelpertools.gui.ProgressDialog;
import edu.cmu.taghelpertools.io.Logger;
import edu.cmu.taghelpertools.io.Reader;
import edu.cmu.taghelpertools.net.TCPUploader;
import edu.cmu.taghelpertools.net.UploadModel;
import edu.cmu.taghelpertools.tokenizer.Tokenizer;
import gnu.trove.THashMap;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.util.*;

import javax.swing.DefaultListModel;
import javax.swing.JOptionPane;
import javax.swing.UIManager;

import argparser.ArgParser;
import argparser.BooleanHolder;
import argparser.IntHolder;
import argparser.DoubleHolder;
import argparser.StringHolder;

import weka.classifiers.Classifier;
import weka.core.FastVector;



/**
 * Driver is the main executioner of the Multi-Dimensional Classifier. It
 * handles all the control flow.
 * 
 * @author Jaime Arguello, Hao-Chuan Wang, Mahesh Joshi
 */
public class Driver {
	//by Hao-Chuan
	private static boolean SUPPLY_TESTING_DATASET = false; // note that there are two kinds of "testing data", 
																 // one is labeled data for research purpose, while another is unlabeled data to be labeled	
	
	// this variable is only used when the -mt option is specified
	private static int outFormat = MultiDimClassifier.FORMAT_MINORTHIRD;
	
	private static boolean addSubPopToSvmLight = false;
	private static boolean noModelBuilding = false;
	private static boolean noProgress = false;
	private static boolean forceCSV = false;
	private static boolean writeFeatures = false;
	private static boolean skipClass = false;
	private static int numAuxTextCols = 0;
	private static long randomSeed = -1;
	
    private static final String NUMBER_CLASSIFIERS = "@Number_of_Classifier";
    private static final String SEGMENT_TAG = "@Segment_Tag";
    private static String CONFIG_FILENAME = "";
    private static String outputPrefix ="";
    private static int errFlag = 0;    

    /**
     * Maps each file that the user selects to a OptionsModel object. The
     * OptionsModel object holds all the Feature Selection options and
     * re-training options selected by the user. Each input .xls file can
     * contain multiple classification problems, each given a different column.
     * The user can set the options differently for each file, but options can't
     * be classifier specific, only file specific.
     */
    private THashMap filePathToOptionModelMap;

    /**
     * Contrains list a list of ClassifierModels
     */
    private ArrayList classifierModelList;
    private DefaultListModel backupFileListModel = null;

    private boolean guiMode;
    //yichia
    private boolean uploadToServer = false;
    
    public static void main(String args[]) {
  		ArgParser argparser =
			new ArgParser(
					"runtht [options] <file1> [<file2> <file3>..]",
					true);    	
		BooleanHolder nongui = new BooleanHolder(false);
		BooleanHolder stop = new BooleanHolder(false);
		BooleanHolder stem = new BooleanHolder(false);
		BooleanHolder removeRare = new BooleanHolder(false);
		IntHolder rareThresh = new IntHolder(0);	
		BooleanHolder sampleInput = new BooleanHolder(false);
		StringHolder sampleRatio = new StringHolder("");	
		Vector featVec = new Vector(7);
		
		StringHolder lang = new StringHolder("eng");
		StringHolder clPar = new StringHolder("weka.classifiers.bayes.NaiveBayes");
		StringHolder prefix = new StringHolder("");
		BooleanHolder noevaluate = new BooleanHolder(false);
		BooleanHolder usingMT = new BooleanHolder(false);
		BooleanHolder outputFreqCounts = new BooleanHolder(false);
		BooleanHolder subPop = new BooleanHolder(false);
		
		argparser.addOption("-ng,--nongui %v #Use command line mode", nongui);
		argparser.addOption(
				"-f,--feature %s {uni,bi,posbi,punc,ll,cns} #Features to use", featVec);
		argparser.addOption(
				"-stop,--stopwords %v #Remove stopwords or functional words from list of features", stop);
		argparser.addOption(
				"-stem,--stemwords %v #Reduce words to morphological root forms", stem);
		argparser.addOption(
				"-noev,--no_evaluate %v #DON'T perform any evaluation; by default TagHelperTools will perform evaluation", noevaluate);		
		argparser.addOption(
				"-mt,--minor3rd %v #Output the dataset in Monorthird format; by default the Weka format is used", usingMT);			
		argparser.addOption(
				"-rr,--removerare %v #Remove rare features (occurring fewer number of times than a threshold)", removeRare);
		argparser.addOption(
				"-rt,--rarethresh %d #The threshold used for removing rare features (specifying this option automatically turns on -rr)",
				rareThresh);
		argparser.addOption(
				"-s,--sample %v #Sample input data", sampleInput);
		argparser.addOption(
				"-sr,--sampleratio %s #A list of sampling ratios to be sampled and compared (specifying this option automatically turns on -s). " +
				"Use comma to separate each ratio parameter, such as: \"0.3,0.5,0.7\" . Note that no space is allowed between parameters.",
				sampleRatio);	
		argparser.addOption(
				"-lang,--language %s {eng, ger, chi, spa} #The language of the dataset",
				lang);
		argparser.addOption(
				"-pf,--prefix %s #Add the specified prefix string into the output files for the ease of identification.",
				prefix);		
		argparser.addOption(
				"-cl,--classifier %s #Configure the Weka classifier and training parameters. " +
				"Use brackets to surround the configuration. e.g. [weka.classifiers.trees.J48 -C 0.25 -M 2]",
				clPar);
		argparser.addOption(
				"-ufc,--usefreqcounts %v # Output frequency counts of features instead of " +
				"the default 0/1 values", outputFreqCounts);
		argparser.addOption("-sub,--subpop %v #Include sub-population string in WEKA ARFF file, as a" +
				"\"string\" type attribute. If the " +
				"input CSV file does not contain a column with heading \"SUBPOP\" immediately after \"text\" " +
				"column, then an empty string is used as the sub-population ID. No classifier model is built " +
				"and no evaluation is performed!", subPop);

		String[] revisedArgs = new String[]{}; 
		
//		for(int i=0;i<args.length;i++)
//		{
//			System.out.println(args[i]);
//		}
		
		List<String> tmpArgs = Driver.handleArgs(args, 0, new StringBuffer(), new ArrayList<String>());
		revisedArgs = tmpArgs.toArray(revisedArgs);
//		for(int i=0;i<tmpArgs.size();i++)
//		{
//			System.out.println(tmpArgs.get(i));
//		}
//		
//		System.exit(0);
		
		String[] unmatched =
			argparser.matchAllArgs(revisedArgs, 0, argparser.EXIT_ON_ERROR);
				
        Driver driver = new Driver();


        if(nongui.value)
        {//command line mode
			driver.setGuiMode(false);          	
			OptionsModel options = new OptionsModel();		
			options= new OptionsModel();
			options.setUnigrams(false);
			options.setBigrams(false);
			options.setPunctuation(false);
			options.setPosBigrams(false);
			options.setLineLength(false);
			options.setContainsNonStopword(false);
			options.setSelfTraing(false);
			options.setEvaluating(true);
			options.setUsingArff(true);
			options.setOutputFreqCounts(false);
			options.setSubPop(false);
			
			for (int i = 0; i < featVec.size(); i++) {
				if (((StringHolder) featVec.get(i)).value.equalsIgnoreCase("uni")) 
				{
					options.setUnigrams(true);
				} 
				else if (((StringHolder) featVec.get(i)).value.equalsIgnoreCase("bi")) 
				{
					options.setBigrams(true);
				} 
				else if (((StringHolder) featVec.get(i)).value
						.equalsIgnoreCase("punc")) 
				{
					options.setPunctuation(true);
				} 
				else if (((StringHolder) featVec.get(i)).value
						.equalsIgnoreCase("posbi")) 
				{
					options.setPosBigrams(true);
				} 
				else if (((StringHolder) featVec.get(i)).value.equalsIgnoreCase("ll")) 
				{
					options.setLineLength(true);
				} 
				else if (((StringHolder) featVec.get(i)).value
						.equalsIgnoreCase("cns")) 
				{
					options.setContainsNonStopword(true);
				}
			}
			
			// if a rare threshold is provided, turn on remove-rare flag
			if (rareThresh.value > 0) {
				removeRare.value = true;
			}
			
			// if remove-rare flag is turned on and rare threshold is not
			// specified, set it to default value of 5
			if (removeRare.value == true && rareThresh.value == 0) {
				rareThresh.value = 5;
			}
			
			options.setRemoveStopwords(stop.value);
			options.setStem(stem.value);
			options.setRemoveRareFeatures(removeRare.value);
			options.setFrequencyThreshold(rareThresh.value);
			options.setEvaluating(!noevaluate.value);
			options.setUsingArff(!usingMT.value);	
			options.setOutputFreqCounts(outputFreqCounts.value);
			options.setSubPop(subPop.value);
			
			// creating a string attribute for sub-population rules
			// out generating a classifier or evaluation
			if (subPop.value == true) {
				options.setEvaluating(false);
				Driver.noModelBuilding = true;
			}
			
			// if sub-sampling is off, but sample ratio is provided
			// turn sub-sampling on
			if (sampleInput.value == false) {
				if(!sampleRatio.value.equals("")) {
					sampleInput.value = true;
				}
				else {
					// sample ratio empty, set it
					// to zero explicitly so that life is
					// without any exceptions down further
					sampleRatio.value = "0";
				}
			} else {
				// sub-sampling is on, but no sampling ratio
				// specified, use default of 0.1
				if(sampleRatio.value.equals("")) {
					sampleRatio.value = "0.1";
				}
			}
			
			options.setSubsetInput(sampleInput.value);
			
			String clParValue = clPar.value.trim().replaceAll("'", "\"");
			String[] values = clParValue.split(" ");			
			List<String> tmpParms = Driver.parseWekaParameters(values, 0, new StringBuffer(), new ArrayList<String>());
			//System.out.println(clParValue);			
			//System.out.println(tmpParms);
			
			String clName = tmpParms.remove(0);
			String[] clOptions = new String[]{};
			clOptions= tmpParms.toArray(clOptions);
			
			//configure the model
			try{
				Classifier cl = Classifier.forName(clName, (String[])clOptions);
				options.setClassifier(cl);
			}
			catch(Exception ex)
			{
				System.err
				.println("ERROR: Cannot build the classifier. Please check the configuration. See usage below.\n");
				System.err.println(argparser.getHelpMessage());
				System.exit(-1);				
			}

			boolean atLeastOneValidFile = false;
			if(unmatched !=null)
			{
				String[] sampleRatios = sampleRatio.value.split(",");
							
				for (int i = 0; i < unmatched.length; i++) 
				{
					File file = new File(unmatched[i]);				
					if (file.exists()) {
			    		String path= MultiDimClassifier.OUTPUT+"/";
			    		if(!prefix.value.equals(""))
			    		{
			    			path = path+prefix.value+"_";
			    		}                		
			    		path = path+"Comparison_";
			    		path = path +file.getName() +".txt";						
						options.setComparisonPath(path);
						
						for(int j=0; j<sampleRatios.length; j++)
						{						
							atLeastOneValidFile = true;

							OptionsModel currOptions = new OptionsModel();

							currOptions.setUnigrams(options.isUnigrams());
							currOptions.setBigrams(options.isBigrams());
							currOptions.setPosBigrams(options.isPosBigrams());
							currOptions.setPunctuation(options.isPuctuation());
							currOptions.setLineLength(options.isLineLength());
							currOptions.setContainsNonStopword(options.isContainsNonStopword());

							currOptions.setRemoveStopwords(options.isRemoveStopwords());
							currOptions.setStem(options.isStem());
							currOptions.setRemoveRareFeatures(options.isRemoveRareFeatures());
							currOptions.setFrequencyThreshold(options.getFrequencyThreshold());

							currOptions.setArffFileName(file.getName() + ".arff");

							currOptions.setEvaluating(options.isEvaluating());
							currOptions.setUsingArff(options.isUsingArff());
							currOptions.setOutputFreqCounts(options.isOutputFreqCounts());
							currOptions.setSubPop(options.isSubPop());

							currOptions.setClassifier(options.getClassifier());
							
							currOptions.setComparisonPath(options.getComparisonPath());

							if(lang.value.equalsIgnoreCase("eng"))
							{
								currOptions.setLanguage(Tokenizer.LANGS[Tokenizer.LANG_ENGLISH], 
										Tokenizer.STPS[Tokenizer.LANG_ENGLISH]);
							}
							else if(lang.value.equalsIgnoreCase("ger"))
							{
								currOptions.setLanguage(Tokenizer.LANGS[Tokenizer.LANG_GERMAN], 
										Tokenizer.STPS[Tokenizer.LANG_GERMAN]);			
							}
							else if(lang.value.equalsIgnoreCase("chi"))
							{
								currOptions.setLanguage(Tokenizer.LANGS[Tokenizer.LANG_CHINESE], 
										Tokenizer.STPS[Tokenizer.LANG_CHINESE]);			
							}
							else if(lang.value.equalsIgnoreCase("spa"))
							{
								currOptions.setLanguage(Tokenizer.LANGS[Tokenizer.LANG_SPANISH], 
										Tokenizer.STPS[Tokenizer.LANG_SPANISH]);			
							}

							double tmpRatio = Double.parseDouble(sampleRatios[j]);
							if(tmpRatio > 0)
							{
								currOptions.setSubsetInput(true);
								currOptions.setSubsetPortion(tmpRatio);
								driver.addFileOptionModelMap(file.getAbsolutePath()+"#r"+tmpRatio, currOptions);
							}
							else
							{
								driver.addFileOptionModelMap(file.getAbsolutePath(), currOptions);
							}	
						}
					} 
					else 
					{
						System.out.println("WARNING: Ignoring non-file non-argument option "
								+ unmatched[i]);
					}
				}
			}

			if (!atLeastOneValidFile) {
				System.err
						.println("ERROR: No valid data files specified for processing!! See usage below.\n");
				System.err.println(argparser.getHelpMessage());
				System.exit(-1);
			}

			driver.processAllFiles(prefix.value);						
        }
        else
        {
			driver.setGuiMode(true);        	
        	driver.run();
        }
    }

    /**
     * Static initializations for Driver
     */
    static {

        // setting windows look and feel (vs. Java look and feel)
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) {
            try {
                UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
            } catch (Exception e1) {
                e1.printStackTrace();
                System.exit(1);
            }
        }

        String useTest = System.getProperty("tht.usetest");
        if (useTest != null && useTest.equals("1"))
        	Driver.SUPPLY_TESTING_DATASET = true;
        String outFormatStr = System.getProperty("tht.outformat");
        if (outFormatStr != null) {
        	if (outFormatStr.equalsIgnoreCase("svmlight"))
        		outFormat = MultiDimClassifier.FORMAT_SVMLIGHT;
        	else if (outFormatStr.equalsIgnoreCase("gibbslda"))
        		outFormat = MultiDimClassifier.FORMAT_GIBBSLDA;
        	else // default is MinorThird format when using -mt option
        		outFormat = MultiDimClassifier.FORMAT_MINORTHIRD;
        }
        String outputSubPopinSVMLight = System.getProperty("tht.svmlight.subpop");
        if (outputSubPopinSVMLight != null && outputSubPopinSVMLight.equals("1"))
        	Driver.addSubPopToSvmLight = true;
        
        String noProgress = System.getProperty("tht.noprogress");
        if (noProgress != null && noProgress.equals("1"))
        	Driver.noProgress = true;
        
        int nAuxTexts = 0;
        String numAuxTexts = System.getProperty("tht.numauxtexts");
        if (numAuxTexts != null && ((nAuxTexts = Integer.parseInt(numAuxTexts)) > 0))
        	Driver.numAuxTextCols = nAuxTexts;
        
        // check we are to skip modeling totally
        // note: this skips evaluation automatically
    		String noModel = System.getProperty("tht.nomodel");
        if (noModel != null && noModel.equals("1"))
        	Driver.noModelBuilding = true;
        
        
        // check we are to skip modeling totally
        // note: this skips evaluation automatically
    		String randomSeed = System.getProperty("tht.subsetter.randomseed");
        if (randomSeed != null) {
        	try {
        		long seed = Long.parseLong(randomSeed);
        		if (seed >= 0) Driver.randomSeed = seed;
        	} catch (Exception e) {
        		// do nothing, default random seed is already set to -1
        	}
        }
        
        // check we are to force reading input file in CSV format
    		String forceCSV = System.getProperty("tht.forcecsv");
        if (forceCSV != null && forceCSV.equals("1"))
        	Driver.forceCSV = true;
        
        // check if we should write features file
        String writeFeatures = System.getProperty("tht.writefeat");
        if (writeFeatures != null && writeFeatures.equals("1"))
        	Driver.writeFeatures = true;
        
        // check if we should skip writing class value in SVM Light format (for LDA)
        String skipClass = System.getProperty("tht.skipclass");
        if (skipClass != null && skipClass.equals("1"))
        	Driver.skipClass = true;
        
        
        /*
         * The following code has been commented out because it's producing the
         * config file, which was used during the Summer School, but not after
         * that.
         */

        // making config directory if one does not exist
        // String currentDirName = System.getProperty("user.dir");
        // File currentDir = new File(currentDirName);
        // File parentDir = currentDir.getParentFile();
        // File tutalkDir = new File(parentDir.getAbsolutePath() + "/" +
        // "TuTalkGUI/");
        // if ((!tutalkDir.exists()) || (!tutalkDir.isDirectory())) {
        // tutalkDir.mkdir();
        // }
        // File config = new File(tutalkDir.getAbsolutePath() + "/" +
        // "config/");
        // if (!(config.exists() && config.isDirectory())) {
        // config.mkdir();
        // }
        // setting config filename
        // String username = System.getProperty("user.name");
        // Driver.CONFIG_FILENAME = "config_" + username + ".txt";
        // Driver.CONFIG_FILENAME = config.getAbsolutePath() + "/config_" +
        // username + ".txt";
    }
    
    
    private static List<String> handleArgs(String[] values, 
								    		int index, 
								    		StringBuffer buffer,
								    		List<String> result)
    {
    	if(index == values.length)
    	{  		
    		return result;
    	}
    	

    	if(values[index].trim().startsWith("["))
    	{
    		if(values[index].endsWith("]"))
    		{
    			String tmp = values[index].replaceAll("\\[", "").replaceAll("]", "");
    			if(buffer.toString().trim().length() == 0)				
    				result.add(tmp);    				
    			else
    				buffer.append(tmp).append(" ");

    			return handleArgs(values, index+1, buffer, result);
    		}
    		else
    		{
    			buffer.append(values[index].replaceAll("\\[", "")).append(" ");
    			return handleArgs(values, index+1, buffer, result);
    		}
    	}
    	else if(values[index].endsWith("]"))
    	{
    		buffer.append(values[index].replaceAll("\\]", ""));
    		if(buffer.toString().trim().length() != 0)				
    			result.add(buffer.toString());
    		buffer = new StringBuffer();
    		return handleArgs(values, index+1, buffer, result);
    	}
    	else
    	{
    		if(buffer.toString().length() == 0)
    			result.add(values[index]);
    		else
    			buffer.append(values[index]).append(" ");
    		
    		return handleArgs(values, index+1, buffer, result);    			
    	}
   }
    
    private static List<String> parseWekaParameters(String[] values, 
    										 int index, 
    										 StringBuffer buffer,
    										 List<String> result)
    {
    	if(index == values.length)
    		return result;
    	
    		if(values[index].startsWith("\""))
    		{
    			if(values[index].endsWith("\""))
    			{
    				String tmp = values[index];//.replaceAll("\"", "");
        			if(buffer.toString().trim().length() == 0)				
        				result.add(tmp);    				
        			else
        				buffer.append(tmp).append(" ");
        			
    				return parseWekaParameters(values, index+1, buffer, result);
    			}
    			else
    			{
    				buffer.append(values[index]).append(" ");//.replaceAll("\"", "")).append(" ");
    				return parseWekaParameters(values, index+1, buffer, result);
    			}
    		}
    		else if(values[index].endsWith("\""))
    		{
				buffer.append(values[index]).append(" ");//.replaceAll("\"", ""));
    			if(buffer.toString().trim().length() != 0)
    			{
    				String tmp = buffer.toString().trim();
    				result.add(tmp.substring(1, tmp.length()-1));
					buffer = new StringBuffer();
    			}
				return parseWekaParameters(values, index+1, buffer, result);
    		}
    		else
    		{
    			if(buffer.toString().trim().length() == 0)
    				result.add(values[index]);
    			else
    				buffer.append(values[index]).append(" ");
				return parseWekaParameters(values, index+1, buffer, result);    			
    		}
    }
    
    public String getOutputPrefix()
    {
    	return Driver.outputPrefix;
    }
    
    public Driver() {
        filePathToOptionModelMap = new THashMap();
        classifierModelList = new ArrayList();
    }
    
    public boolean isGuiMode() {
		return guiMode;
	}

	public void setGuiMode(boolean guiMode) {
		this.guiMode = guiMode;
	}
	
    public void setBackupFileListModel(DefaultListModel backupFileListModel)
    {
    	this.backupFileListModel = backupFileListModel;
    }
    
    public DefaultListModel getBackupFileListModel()
    {
    	return this.backupFileListModel;
    }
    
    /**
     * Opens the fileSelector, which is used to select all the files and set
     * each file-specific feature selection and re-training parameters. The
     * Driver instance is passed to the FileSelector.
     */
    public void run() {
        new FileSelector(this);
    }

    /**
     * This method is called from the FileSelector. Adds a file path and its
     * corresponding optionsModel to the filepathToOptionsModelMap.
     * 
     * @param path -
     *            the string value of the file's *absolute* path.
     * @param optionsModel -
     *            the file's corresponding OptionsModel.
     */
    public void addFileOptionModelMap(String path, OptionsModel optionsModel) {
        this.filePathToOptionModelMap.put(path, optionsModel);
    }

    /**
     * This method is called from the FileSelector. Removes the file from the
     * filePathToOptionsModelMap. This is called when the user removes a file
     * from the inventory of files they wish to process.
     * 
     * @param path -
     *            the string value of the file's *absolute* path
     */
    public void removeFileOptionModelMap(String path) {
        this.filePathToOptionModelMap.remove(path);
    }

    /**
     * This method is called from the FileSelector, when the user clicks on the
     * "GO" button. This processes runs all the training, testing, and
     * re-training step for all files chosen by the user. The trickiest part of
     * understanding this method is that all files are processed in a different
     * thread, but each subsequent thread starts only when the previous file
     * ends. To do this, RunFile is given the NextThread class member that holds
     * the Thread that needs to run after it is finished.
     */
    public void processAllFiles()
    {
    	this.processAllFiles("");
    }
    
    public void processAllFiles(String outputPrefix) {

        // Builds and ArrayList of RunFiles. Each RunFile holds a file and its
        // corresponding OptionsModel
        Iterator fileIter = this.filePathToOptionModelMap.keySet().iterator();
        ArrayList threads = new ArrayList();
        while (fileIter.hasNext()) {
        	String identifier= fileIter.next().toString();
        	
            File file = new File(identifier.split("#")[0]);
            OptionsModel optionsModel = this.getOptionsModelForFile(identifier);
            
            //temporarily use by Hao-Chuan, imporatant!
            String ext = "";
            if(identifier.split("#").length >1)
            {
            	ext = identifier.split("#")[1];
            	ext = "_" + ext;
            }
            // append sampling ratio to the output file prefix
            optionsModel.setOutputPrefix(outputPrefix + ext);            
            RunFile runFile = new RunFile(this, file, !fileIter.hasNext(), optionsModel);
            threads.add(runFile);
        }

        // Runs through the arraylist and sets the NextThread of each RunFile to
        // the one next to it in the ArrayList.
        Iterator threadIter = threads.iterator();
        if (threadIter.hasNext()) {
            RunFile runFile = (RunFile) threadIter.next();
            while (threadIter.hasNext()) {
                RunFile tempRunFile = (RunFile) threadIter.next();
                runFile.setNextThread(tempRunFile);
                runFile = tempRunFile;
            }
            runFile.setNextThread(null);
        }


        // Starts the whole execution by starting the first one finishes, it
        // starts the execution of its next one, and so forth until the last one
        // ends. At that point the process is complete.
        if (threads.size() > 0) {
            ((RunFile) threads.get(0)).start();
        }        
    }

    /**
     * RunFile processes a single file (and all predicitons/classifiers
     * associated with this file). It runs as a single thread. It extends
     * SwingWorker, which is an abstraction of a Thread. In a SwingWorker, all
     * the code in the contruct() method runs when the Thread is started.
     * 
     * @author Jaime Arguello
     * 
     */
    public class RunFile extends SwingWorker {

        private Driver driver;
        private File file;
        private OptionsModel optionsModel;
        private ProgressDialog pDialog;
        private boolean isLastFile;
        private RunFile nextThread;

        public RunFile(Driver driver, 
        		       File file, 
        			   boolean isLastFile, 
        		       OptionsModel optionsModel) 
        {
            this.driver = driver;
            this.file = file;
            this.isLastFile = isLastFile;
            this.optionsModel = optionsModel;        
        }

        public String getOutputPrefix()
        {
        	return this.optionsModel.getOutputPrefix();
        }
        
        public void setNextThread(RunFile nextThread) {
            this.nextThread = nextThread;
        }

        /*
         * Runs through a single file. When the execution of this file ends, it
         * calls the next one, unless the value of *nextThread* is null, in
         * which case the process ends because this file is the final one in the
         * execution list.
         * 
         * @see middle_layer.SwingWorker#construct()
         */
        public Object construct() {
            Driver.outputPrefix = optionsModel.getOutputPrefix();           
            // starts up the progress bar, that gives the user an idea of where
            // the process is.
            if (!Driver.noProgress)
            	pDialog = new ProgressDialog(null, file.getName());
            else
            	pDialog = null;

            // reads in the .xls file
            Reader input = new Reader(file.getAbsolutePath());
            
            // get the associated test file (simply by replacing the ".train." infix
            // with ".test." infix
            Reader test = null;
            String testFilePath;
            
            if(Driver.SUPPLY_TESTING_DATASET) // default is false
            {
	            if (file.getAbsolutePath().indexOf(".train.") > 0) {
	                testFilePath = file.getAbsolutePath().replaceFirst("\\.train\\.", ".test.");
	                if ((new File(testFilePath)).exists()) {
	                    test = new Reader(testFilePath);
	                }
	            }
            }

            // The multiDimClassifier does all the work with training, testing,
            // and re-training of a single file.
            MultiDimClassifier multiDimClassifier = new MultiDimClassifier(
            											  input, 
            											  test, 
            											  this.optionsModel.getIterationMode(), 
            											  this.optionsModel);

            // set up the multiDimClassifier
            if(this.optionsModel.getPreprocessedTrainingDataPoints() == null)
            {
	            try{
	            	multiDimClassifier.setUp(pDialog);
	            }
	            catch(TextNotFoundException e) {
	            	if (pDialog != null) pDialog.dispose();   
	        		Driver.errFlag = 1;
	        		this.nextThread = null;
	        		this.driver.run();
	        		return null;
	            }
	            catch(Exception ex)
	            {
	            	ex.printStackTrace();
  	        		if (pDialog != null) pDialog.dispose();   
  	        		Driver.errFlag = 1;
  	        		this.nextThread = null;
  	        		this.driver.run();
  	        		return null;        		
	            }
            }
            else
            {
            	multiDimClassifier.setPreprocessed(this.optionsModel.getPreprocessedTrainingDataPoints(), null,
            									   this.optionsModel.getInputFileReader(),
            									   this.optionsModel.getFeatureSelector());
            }

            // if using the MinorThird format, just output the dataset
            // and proceed to the next file
            
            // write training and testing dataset to MT
            if(!this.optionsModel.isUsingArff())
            {
            	if (pDialog != null) {
  	            pDialog.setLabel("Writing DATASET files");
  	            pDialog.setIsIndeterminate(false);
            	}
	            for (int i = 0; i < DataPoint.dimNum; i++) {
	                String mtFile = this.optionsModel.getArffFileName();
	                if (pDialog != null) pDialog.setLabel("Writing DATASET files: ");
	                multiDimClassifier.writeDatasetFileForDim(i, mtFile, 
	                		this.optionsModel.getOutputPrefix(), pDialog, Driver.outFormat, 
	                		Driver.addSubPopToSvmLight);
	                if (test != null) {
	                    String testMTFile = mtFile.replaceFirst(".train.", ".test.");
	                    if (pDialog != null) pDialog.setLabel("Writing TEST DATASET files: ");
	                    multiDimClassifier.writeTestDatasetFileForDim(i,testMTFile, 
	                    		this.optionsModel.getOutputPrefix(), pDialog, Driver.outFormat,
	                    		Driver.addSubPopToSvmLight);
	                }
	            }
	            if (pDialog != null) { 
  	            pDialog.setIsIndeterminate(false);
  	            pDialog.dispose();
	            }
            } else {
            	// we are going through the process of training the Weka model
            	// and/or doing evaluation depending on the options
            
            	// the following codes were commented for not doing the training.
            	//already uncommented by Hao-Chuan 20070110
            
	            // runs each individual classification problem in the file at hand.
	            for (int i = 0; i < DataPoint.dimNum; i++) 
	            {	            	
		            	try
		            	{
		            		// noModelBuilding flag is set based on a special system property
		            		// named "tht.nomodel" (only for developers)
		            		if (noModelBuilding == false) {
  		            		ClassifierModel model = multiDimClassifier.run(i, pDialog);
  			                driver.classifierModelList.add(model);	 
  
  			              if(uploadToServer)
  				            {
  			                //TODO:
  			                String prefix = this.optionsModel.getOutputPrefix().equals("")?"":this.optionsModel.getOutputPrefix()+"_";
  				            	uploadFiles(model, prefix);
  				            }
		            		}
		            		
			              if (pDialog != null) {
			              	pDialog.setLabel("Writing ARFF DATASET files");
			              	pDialog.setIsIndeterminate(true);
			              }
		  	            String trainFileName = "";
		  	            // need to invoke writeARFFForDim with resetInstances = true
		  	            // if we have not built a classifier model
		  	            boolean resetInstances = Driver.noModelBuilding;
		  	            // use the input file name, in train/test mode
		  	            if (test != null) {
		  	            	trainFileName = this.optionsModel.getArffFileName();
		  	            } else {
		  	            	// else a fixed string as the file name
		  	            	trainFileName = "train_set.arff";
		  	            }
		  	            
		                multiDimClassifier.writeARFFForDim(i, 
		                		trainFileName,
      							   this.optionsModel.getOutputPrefix(),
      							   resetInstances, pDialog
      							   );
		                if (test != null) {
		                	String testFileName = trainFileName.replaceFirst(".train.", ".test.");
		                	if (pDialog != null) pDialog.setLabel("Writing ARFF DATASET files");
	                    multiDimClassifier.writeTestARFFForDim(i, 
	                    		testFileName, 
	                    		this.optionsModel.getOutputPrefix(),
	                    		pDialog);
	                }

	            	}
	            	catch(Exception ex)
	            	{
	            		ex.printStackTrace();
	            		if (pDialog != null) pDialog.dispose();
	            		Driver.errFlag = 1;
	            		this.nextThread = null;
	            		this.driver.run();
	            		return null;
	            	}
	            }
	            if (pDialog != null) {
  	            pDialog.setLabel("Writing results to the OUTPUT folder");
  	            pDialog.setIsIndeterminate(true);
	            }
	            String outputName = MultiDimClassifier.OUTPUT + "/";
	            if(Driver.outputPrefix !=null && !Driver.outputPrefix.equals(""))
	            {
	                     outputName = outputName + Driver.outputPrefix+"_";
	            }
	                      
	            outputName = outputName +file.getName().replaceAll("." + FileSelector.FILE_EXT, "") + "_OUTPUT.csv";

	            // output format is same as input format (i.e. if .csv
	            // is input, then .csv is output. if .xls is input, then
	            // .xls is output)
	            multiDimClassifier.writeOutput(new File(outputName), 
	            		file.getAbsolutePath().endsWith(".csv"));
	            if (pDialog != null) pDialog.dispose();
            }

	        if (nextThread != null)
            {
            	if (isGuiMode())            	
            		this.driver.getBackupFileListModel().removeElement(this.file.getAbsolutePath());
            	
        		this.driver.filePathToOptionModelMap.remove(this.file.getAbsolutePath());            	
                nextThread.start();
            }
            else 
            {
            	if(isGuiMode())
            	{
	            	System.out.println("\nDone! <please press the GUI button to finish>");
	                JOptionPane.showMessageDialog(null, "done!", "Success", JOptionPane.INFORMATION_MESSAGE);
            	}
            	else
            	{
            		System.out.println("\nDone!");            		
            	}
            }
            return null;
        }
    }

    /**
     * Writes the configuration file for the TuTalk system.
     */
    public void writeConfigFile() {
        Logger config = new Logger(Driver.CONFIG_FILENAME);
        config.writeLine(Driver.NUMBER_CLASSIFIERS + " " + this.classifierModelList.size());
        config.writeLine();
        config.writeLine();
        config.writeLine(Driver.SEGMENT_TAG);
        config.writeLine();
        config.writeLine();
        Iterator classModelIter = this.classifierModelList.iterator();
        while (classModelIter.hasNext()) {
            ClassifierModel classModel = (ClassifierModel) classModelIter.next();
            config.writeLine(classModel.toString());
            config.writeLine();
            config.writeLine();
        }
        config.close();
    }

    /**
     * This is a simple GUI progress bar that has no defined start and end
     * states, but simply indicates the a process is currently running.
     * 
     * @author Jaime Arguello
     * 
     */
    public class IndetPDialog extends SwingWorker {

        private ProgressDialog pDialog;

        public IndetPDialog(ProgressDialog pDialog) {
            this.pDialog = pDialog;
        }

        public void terminate() {
            this.pDialog.dispose();
        }

        public Object construct() {
            pDialog.setLabel("loading...");
            pDialog.setIsIndeterminate(true);
            return null;
        }
    }

    /**
     * Originally this method uploaded the model, dictionary, and config files
     * to the TuTalk server, which used to run in on a remote machine. However,
     * this part is now commented out. Instead, the method simply copies this
     * file to a different directory in the local machine (assumes the TuTalk
     * server runs locally)
     */
    public void uploadFiles() {
        IndetPDialog indetPDialog = new IndetPDialog(new ProgressDialog(null, "Loading files to server"));
        indetPDialog.start();
        Iterator classModelIter = this.classifierModelList.iterator();
        while (classModelIter.hasNext()) {
            ClassifierModel model = (ClassifierModel) classModelIter.next();

            // commented out because now the TuTalk server runs locally

            // uploadModel(model);
            // uploadDic(model);

            // copies model and dictionary file to seperate directory.
            copyFile(model.getModFile());
            copyFile(model.getDicFile());
        }
        // uploadConfig(new File(Driver.CONFIG_FILENAME));
        copyFile(new File(Driver.CONFIG_FILENAME));
        try {
            Thread.currentThread().sleep(5000);
        } catch (InterruptedException e) {
            JOptionPane.showMessageDialog(null, "Could not upload config file", "TERMINAL ERROR", JOptionPane.ERROR_MESSAGE);
            System.exit(0);
        }
        File config = new File(CONFIG_FILENAME);
        String configName = config.getName();

        UploadModel.refreshServer(configName);
        indetPDialog.terminate();
        JOptionPane.showMessageDialog(null, "DONE!", "Information", JOptionPane.INFORMATION_MESSAGE);
    }

    /**
     * Uploads model to TagHelper server.
     * 
     * @param model
     *            to be uploaded to TagHelper server.
     */
    public void uploadModel(ClassifierModel model) {
        try {
            UploadModel.sendFile(model.getModFile());
        } catch (UnsupportedEncodingException e) {
            JOptionPane.showMessageDialog(null, "Could not upload file: " + model.getModFile().getName(), "TERMINAL ERROR", JOptionPane.ERROR_MESSAGE);
            System.exit(0);
        }
    }

    /**
     * Copies the input file to the Taghelper Server directory.
     * 
     * @param src -
     *            the file to be moved to the TagHelperServer directory.
     */
    public void copyFile(File src) {
        File dst = new File("../TagHelperServer/data/" + src.getName());

        InputStream in = null;
        OutputStream out = null;

        try {
            in = new FileInputStream(src);
            out = new FileOutputStream(dst);
        } catch (FileNotFoundException e1) {
            System.exit(0);
            e1.printStackTrace();
        }

        // Transfer bytes from in to out
        byte[] buf = new byte[1024];
        int len;
        try {
            while ((len = in.read(buf)) > 0) {
                out.write(buf, 0, len);
            }
        } catch (IOException e) {
            e.printStackTrace();
            System.exit(0);
        }
        // close files
        try {
            in.close();
            out.close();
        } catch (IOException e) {
            e.printStackTrace();
            System.exit(0);
        }
    }

    /**
     * Uploads - model to the TagHelper Server.
     * 
     * @param model -
     *            Model to be uploaded
     */
    public void uploadDic(ClassifierModel model) {
        try {
            UploadModel.sendFile(model.getDicFile());
        } catch (UnsupportedEncodingException e) {
            JOptionPane.showMessageDialog(null, "Could not upload file: " + model.getDicFile().getName(), "TERMINAL ERROR", JOptionPane.ERROR_MESSAGE);
            System.exit(0);
        }
    }

    /**
     * Uploads config file to the TagHelper Server.
     * 
     * @param configFile -
     *            config file to be uploaded
     */
    public void uploadConfig(File configFile) {
        try {
            UploadModel.sendFile(configFile);
        } catch (UnsupportedEncodingException e) {
            JOptionPane.showMessageDialog(null, "Could not upload file: " + configFile.getName(), "TERMINAL ERROR", JOptionPane.ERROR_MESSAGE);
            System.exit(0);
        }
    }

    /**
     * Getter method
     * 
     * @param filename -
     *            the absolute path of the file for which the OptionsModel is
     *            sought.
     * @return - the OptionsModel corresponding to the input filename.
     */
    public OptionsModel getOptionsModelForFile(String filename) {
        return (OptionsModel) this.filePathToOptionModelMap.get(filename);
    }
    
    public void setUploadMode(boolean upload)
    {
    	uploadToServer = upload;
    }
    
    public void uploadFiles(ClassifierModel model, String prefix)
    {
    	TCPUploader myUploader = null;
    	
   		myUploader = new TCPUploader(model.getName()+"_"+System.getProperty("user.name"), 
    				"CLASSIFIERS" + File.separator + prefix + model.getName() + File.separator);
    	
    	String myConfigText = "";
    	
    	String name = model.getName();
    	String username = System.getProperty("user.name");
    	
    	String fullname = name+"_"+username;
    	
    	myConfigText += "<classifier>" + "\n";
        myConfigText += "\t@Classifier_Name\t" + fullname + "\r";
        myConfigText += "\t@Classifier_Model\t" + fullname + ".model\r";
        myConfigText += "\t@Dictionary\t" + fullname + ".dic\r";
        myConfigText += "\t@Classifier_OM\t" + fullname + ".om\r";
        myConfigText += "\t@Classifier_INS\t" + fullname + ".ins\r";
        myConfigText += "\t@Classifier_FEATURE\t" + fullname + ".feature\r";
        myConfigText += "\t@Classifier_NFN\t" + fullname + ".nfn\r";
        myConfigText += "\t@Classifier_NFVS\t" + fullname + ".nfvs\r";
        myConfigText += "\t@Classifier_UDF\t" + fullname + ".udf\r";

        String label_str = "";
        FastVector labels = model.getLabels(); 
        if(labels != null)
        {
	        for (int i = 0; i < labels.size(); i++) {
	            label_str = label_str + labels.elementAt(i).toString() + " ";
	        }
        }

        myConfigText += "\t@Class_Labels\t" + label_str + "\r";
        myConfigText += "</classifier>" + "\r";

        myUploader.configString = myConfigText;
        
    	myUploader.doUploadandInstall();
    }

		public static boolean isSUPPLY_TESTING_DATASET() {
			return SUPPLY_TESTING_DATASET;
		}

		public static boolean isNoModelBuilding() {
			return noModelBuilding;
		}

		public static boolean isNoProgress() {
			return noProgress;
		}

		public static int getNumAuxTextCols() {
			return numAuxTextCols;
		}

		public static long getRandomSeed() {
			return randomSeed;
		}

		public static int getErrFlag() {
			return errFlag;
		}
		
		public static boolean isForceCSV() {
			return forceCSV;
		}

		public static boolean isSkipClass() {
			return skipClass;
		}

		public static boolean isWriteFeatures() {
			return writeFeatures;
		}
}