/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package edu.cmu.taghelpertools.middle_layer;

import edu.cmu.taghelpertools.middle_layer.extractor.*;
import gnu.trove.THashMap;
import gnu.trove.THashSet;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

/**
 * This class is for SavedModelEvaluator to use.  The is similar to the class DataPoint,
 * but due to special needs of SavedModelEvaluator, we choose not to use DataPoint.  But the two
 * classess both implement the same interface, which is  
 * 
 * @author haochuan
 *
 */
public class EvalDataPoint implements FeatureSpaceMapProvider {

	public static ArrayList nominalFeatureNames;

	public static THashMap nominalFeatureValueSet;

	public static final String MISSING_NOMINAL_VALUE = "MISSING_NOMINAL_VALUE";

	private String dim;

	private String predictedDim;
	
	private boolean truePredictionPresent;
	
	private String subPopId;
	
	private String [] auxTexts;

	private double score;

	private String text;

	private THashMap featureSetMap;

	private FeatureSelector featureSelector;

	private ArrayList nominalFeatureValues;

	//by Hao-Chuan Wang
	private FeatureSpaceMap featureSpaceMap = null;

	private boolean isTestPoint;
	
	private boolean usesAdvancedFeatures;
	
	/**
	 * Instatiates a data point with the dimensions and text specified.
	 * 
	 * @param
	 * @param 
	 */
	public EvalDataPoint(String dim, String text,
			FeatureSelector featureSelector, ArrayList nominalFeatures,
			THashSet featureSet, boolean testPoint, boolean forFeatureExtraction) {
		// feature selector
		this.featureSelector = featureSelector;
		this.isTestPoint = testPoint;
		this.usesAdvancedFeatures = forFeatureExtraction;

		this.text = text;
		this.dim = dim;
		this.truePredictionPresent = !dim.equals(InputFileReader.UNKNOWN_STRING);

		// initiating scores to all zeroes
		this.score = 0.0;

		// extracting the features from text;
		if (testPoint) {
			if (forFeatureExtraction) {
				FeatureSpaceMap tmpfsm = new FeatureSpaceMap();
				// featureSpaceMap used by advanced feature extractor
				this.featureSpaceMap = new FeatureSpaceMap();
				// featureSetMap used when creating WEKA/MinorThird instances
				this.featureSetMap = this.featureSelector.getFeatureSet(text,
						this.featureSpaceMap, tmpfsm);
				// doing advanced feature extraction
				Iterator itr = this.featureSelector
						.getGlobalUserDefinedFeatureSetIterator();
				while (itr.hasNext()) {
					String advAtt = itr.next().toString();
					String expression = this.featureSelector
							.getlobalUserDefinedFeatureExpression(advAtt);

					FeatureExtractionHandler2.setParameters(this, this.featureSelector);
					try {
						new ThtParser(
								new edu.cmu.taghelpertools.middle_layer.extractor.ThtYylex(
										new java.io.StringReader(expression))).parse();
					} catch (Exception ex) {
						ex.printStackTrace();
						JOptionPane.showMessageDialog(null,
								"Error on processing data point. Please check.",
								"SYNTAX ERROR", JOptionPane.ERROR_MESSAGE);
					}
				}
			} else {
				// System.out.println("featureMap2 "+ featureMap.size());
				this.featureSetMap = this.featureSelector.getFeatureSet(text);
			}
		} else {
			// TODO check whether
			// this.featureSelector.addFeatures(this.featureSet);
		}

		// copy the nominal features
		nominalFeatureValues = nominalFeatures;

	}

	public THashMap getFeatures() {
		return featureSetMap;
	}

	public ArrayList getNominalFeatures() {
		return nominalFeatureValues;
	}

	public double getScoreForDim() {
		return score;
	}

	public void setScoreForDim(double score) {
		this.score = score;
	}

	public String getValueAtDim() {
		return dim;
	}

	public void setPredictedValueAtDim(String value) {
			predictedDim = value;
	}

	public String getPredictedValueAtDim() {
		return predictedDim;
	}

	public String getText() {
		//return text;
  	String retText = this.text;
  	
  	if (auxTexts != null)
    	for (int i = 0; i < auxTexts.length; i++)
    		retText = retText + " " + auxTexts[i];
  	
  	return retText;
	}

  public String getQuotedText(String separator) {
    // return text;
  	String retText = "\"" + this.text + "\"";
  	
  	if (auxTexts != null)
    	for (int i = 0; i < auxTexts.length; i++)
    		retText = retText + separator + "\"" + auxTexts[i] + "\"";
  	
  	return retText;
  }

	public void setText(String text) {
		this.text = text;
	}

	public FeatureSpaceMap getFeatureSpaceMap() {
		return this.featureSpaceMap;
	}

	public String toString() {
		String result = "";

		result += (dim.toString() + ",");

		result += this.text;
		return result;
	}

	public boolean isTruePredictionPresent() {
		return truePredictionPresent;
	}
	
	public String getSubPopId() {
		return subPopId;
	}

	public void setSubPopId(String subPopId) {
		this.subPopId = subPopId;
	}

	public void setAuxTexts(String [] auxTexts) {
		this.auxTexts = auxTexts;
		String auxTextPrefix = "auxtext_";
		for (int i = 0; i < auxTexts.length; i++) {
			String customPrefix = auxTextPrefix + (i+2) + "_";
      // extracting the features from text;
			if (this.isTestPoint) {
				if (this.usesAdvancedFeatures) {
  				FeatureSpaceMap tmpfsm = new FeatureSpaceMap();
  				// add auxiliary text features into existing feature space
  				this.featureSetMap.putAll(this.featureSelector.getFeatureSet(auxTexts[i],
  						this.featureSpaceMap, tmpfsm, customPrefix));
  				// doing advanced feature extraction
  				Iterator itr = this.featureSelector
  						.getGlobalUserDefinedFeatureSetIterator();
  				while (itr.hasNext()) {
  					String advAtt = itr.next().toString();
  					String expression = this.featureSelector
  							.getlobalUserDefinedFeatureExpression(advAtt);

  					FeatureExtractionHandler2.setParameters(this, this.featureSelector);
  					try {
  						new ThtParser(
  								new edu.cmu.taghelpertools.middle_layer.extractor.ThtYylex(
  										new java.io.StringReader(expression))).parse();
  					} catch (Exception ex) {
  						ex.printStackTrace();
  						JOptionPane.showMessageDialog(null,
  								"Error on processing data point. Please check.",
  								"SYNTAX ERROR", JOptionPane.ERROR_MESSAGE);
  					}
  				}
  			} else {
  				// add auxiliary text features into existing feature space
          this.featureSetMap.putAll(this.featureSelector.getFeatureSet(auxTexts[i], 
          		customPrefix));
  			}
      } 
      else 
      {
      	// evaluation data point *has* to be a test point
      }
		}
		
	}
	
}
