/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package edu.cmu.taghelpertools.middle_layer.extractor;

import java.util.*;

import edu.cmu.taghelpertools.middle_layer.FeatureSpaceMapProvider;
import edu.cmu.taghelpertools.middle_layer.FeatureSelector;
import edu.cmu.taghelpertools.middle_layer.extractor.FeatureSpaceMap.PositionLengthPair;
import edu.cmu.taghelpertools.middle_layer.DataPoint;

/**
 * This class is the actual class used for handling feature extraction.
 * This class provides static methods for external parsers to employ.
 * 
 * When an (nesting) expression is parsed, e.g. "DEF1=ALL(ANY(foo, boo), goo)",
 * The external parser will recursively indentify the actual function to be called
 * and the arguments to be passed.  
 * 
 * @author haochuan
 *
 */
public class FeatureExtractionHandler2 {
	
	private static final boolean DEBUG =false;
	private static final String NULL = "null";
	
	private static /*DataPoint*/ FeatureSpaceMapProvider dataPoint;
	protected static FeatureSelector featureSelector;
	private static Map<String, OperationList> opListMap;
	private static String opListType = "";
	private static int anonymousCounter;
	
	//for parser
	public static int method_code;
	public static int int_para;

//	public static void removeFeature(List<DataPoint> dataPoints, String featureName)
//	{
//		FeatureExtractionHandler2.featureSelector.removeGlobalUserDefiendFeature(featureName);
//		for(int i=0; i<dataPoints.size(); i++)	//visit every data point
//		{
//			DataPoint tmpPoint = dataPoints.get(i);
//			FeatureSpaceMap fsm = tmpPoint.getFeatureSpaceMap();
//			fsm.removeSyhtheticFeature(featureName);
//		}
//	}
		
	// this method must be called first
	public static void setParameters(/*DataPoint*/FeatureSpaceMapProvider dataPoint, FeatureSelector featureSelector)
	{		
		FeatureExtractionHandler2.dataPoint = dataPoint;
		FeatureExtractionHandler2.opListMap = new TreeMap<String, OperationList>();
		FeatureExtractionHandler2.featureSelector = featureSelector;
		FeatureExtractionHandler2.anonymousCounter = 0; // set the counter to zero;
	}
		
//	private static OperationList getOpList(String featureName)
//	{//TODO:
//		//determine whether this is a basic feature or a synthetic feature
//		if(opListMap.containsKey(featureName)) //synthetic feature
//		{
//			return opListMap.get(featureName);
//		}
//		else //maybe basic feature
//		{
//			//initiate a new operation list with a new node for the basic feature
//			FeatureSpaceMap fsm = dataPoint.getFeatureSpaceMap();
//			PositionLengthPair record = fsm.getOrdFeature(featureName);
//			
//			if(record !=null)
//			{
//				SynFeatureNode tmpNode = new SynFeatureNode(featureName);
//				List<Integer> pl = record.getPositionList();
//				List<Integer> ll = record.getLengthList();
//					
//				for(int i=0; i<pl.size(); i++)
//				{
////					System.out.println(ll.get(i));
//					tmpNode.addInstance(pl.get(i), ll.get(i));
//				}
//				OperationList tmpList = new OperationList();
//				tmpList.add(tmpNode);
//				return tmpList;
//			}
//			else
//			{
//				return null;
//			}
//		}
//	}
	
	private static OperationList getOpList(String featureName)
	{//TODO:
		FeatureSpaceMap fsm = dataPoint.getFeatureSpaceMap();
		PositionLengthPair record = null;
		//determine whether this is a basic feature or a synthetic feature
		if(opListMap.containsKey(featureName)) //synthetic feature
		{
			return opListMap.get(featureName);
		}
		else if (fsm.getOrdFeature(featureName) !=null)//maybe basic feature
		{
			//initiate a new operation list with a new node for the basic feature
			
			record = fsm.getOrdFeature(featureName);

				SynFeatureNode tmpNode = new SynFeatureNode(featureName);
				List<Integer> pl = record.getPositionList();
				List<Integer> ll = record.getLengthList();
					
				for(int i=0; i<pl.size(); i++)
				{
//					System.out.println(ll.get(i));
					tmpNode.addInstance(pl.get(i), ll.get(i));
				}
				OperationList tmpList = new OperationList();
				tmpList.add(tmpNode);
				return tmpList;
		}
		else if (fsm.getSynFeature(featureName)!=null)
		{
			
			//initiate a new operation list with a new node for the syn feature
			//fsm = dataPoint.getFeatureSpaceMap();
			record = fsm.getSynFeature(featureName);
			
				SynFeatureNode tmpNode = new SynFeatureNode(featureName);
				List<Integer> pl = record.getPositionList();
				List<Integer> ll = record.getLengthList();
					
				for(int i=0; i<pl.size(); i++)
				{
//					System.out.println(ll.get(i));
					tmpNode.addInstance(pl.get(i), ll.get(i));
				}
				OperationList tmpList = new OperationList();
				tmpList.add(tmpNode);
				return tmpList;
		}
		else
		{
			return null;
		}
		
	}
	public static void ASSIGN(String ID, String feature)
	{
		
		OperationList result = getOpList(feature);
//		if(result != null && !result.hasDefaultNodeOnly())
//		{
		if(result != null)
		{		
			//TODO for future visualization need, set a pointer to the OperationList in DataPoint
		    FeatureSpaceMap fsm = dataPoint.getFeatureSpaceMap();
		    fsm.removeSyhtheticFeatureIfNotMatched(ID, FeatureExtractionHandler2.featureSelector.getlobalUserDefinedFeatureExpression(ID));		    
			fsm.addSyntheticFeature(ID, 
									FeatureExtractionHandler2.featureSelector.getlobalUserDefinedFeatureExpression(ID), 
									fsm.NOT_COUNTABLE, 
									fsm.NOT_COUNTABLE,
									result);
			if(dataPoint.getClass() == DataPoint.class)
			{
				((DataPoint)dataPoint).setMatchedUserDefined(ID);
				//System.out.println("FEH2 - Assign found" + ID + "	" + feature);
			}
		}		
	}
	
//	public static String ANY(String feature1, String feature2)
//	{			
//		OperationList list1 = getOpList(feature1);
//		OperationList list2 = getOpList(feature2);
//		OperationList result = null;
//		
//        if(list1==null && list2==null)
//		{
//			result = null;
//		}
//        else if(list1==null && list2 !=null)
//        {
//        	result =list2;
//        }
//        else if(list1!=null && list2 ==null)
//        {
//        	result =list1;
//        }        
//		else
//		{
//			//merge
//			result = (OperationList)list1.clone();
//			for(int j=0; j<list2.size(); j++)
//			{
//				SynFeatureNode tmpNode2= (SynFeatureNode)list2.get(j);
//				if(!result.hasNode(tmpNode2.getSynFeatureName()))
//				{
//					result.add(tmpNode2);
//				}
//			}			
////			result = (OperationList)list1.clone();
////			for(int i=0; i<list1.size(); i++)
////			{
////				SynFeatureNode tmpNode1= (SynFeatureNode)list1.get(i);
////				
////				for(int j=0; j<list2.size(); j++)
////				{
////					SynFeatureNode tmpNode2= (SynFeatureNode)list2.get(j);
////					if(!tmpNode1.getSynFeatureName().equals(tmpNode2.getSynFeatureName()))
////					{
////						result.add(tmpNode2);
////					}
////				}				
////			}
//		}
//			
//			if(result!=null)
//			{
//				String anonymousID = "anonymous"+(anonymousCounter++);
//				result.setListType(OperationList.OR_LIST);
//				opListMap.put(anonymousID, result);
//				return anonymousID;
//			}
//			else
//			{
//				return FeatureExtractionHandler2.NULL;
//			}
//	}
	public static String ANY(String feature1, String feature2)//throws TokenException
	{			
//		if(FeatureExtractionHandler2.isIllegalUserFeature(feature1) || FeatureExtractionHandler2.isIllegalUserFeature(feature2))
//		{
//			System.err.println("Illegal User Feature");
//			throw new TokenException();
//		}
//		
		OperationList list1 = getOpList(feature1);
		OperationList list2 = getOpList(feature2);
		OperationList result = null;
		
        if(list1==null && list2==null)
		{
			result = null;
		}
        else if(list1==null && list2 !=null)
        {
        	result =list2;
        }
        else if(list1!=null && list2 ==null)
        {
        	result =list1;
        }        
		else
		{
			//merge
			result = (OperationList)list1.clone();
			for(int j=0; j<list2.size(); j++)
			{
				SynFeatureNode tmpNode2= (SynFeatureNode)list2.get(j);
				if(!result.hasNode(tmpNode2.getSynFeatureName()))
				{
					result.add(tmpNode2);
				}
			}			

		}
			
			if(result!=null)
			{
				String anonymousID = "anonymous"+(anonymousCounter++);
				result.setListType(OperationList.OR_LIST);
				opListMap.put(anonymousID, result);
				return anonymousID;
			}
			else
			{
				return FeatureExtractionHandler2.NULL;
			}
	}
	public static String OPTIONAL(String feature)
	{
		OperationList optList = getOpList(feature);
		
		if(optList == null)
		{
			optList = new OperationList();
			SynFeatureNode defaultNode= new SynFeatureNode();
			defaultNode.setIsOptional(true);
			optList.add(defaultNode); //add a default node; so that  it wont be null			
		}
		else
		{			
			//TODO do we have to add a default node here ??
			SynFeatureNode defaultNode= new SynFeatureNode();			
			optList.add(defaultNode);
			for(int i=0; i<optList.size(); i++)
			{
				SynFeatureNode tmpNode = (SynFeatureNode)optList.get(i);
				tmpNode.setIsOptional(true);
			}
		}		
		
		String anonymousID = "anonymous"+(anonymousCounter++);
		optList.setListType(OperationList.OR_LIST);
		opListMap.put(anonymousID, optList);
		return anonymousID;					
	}

//	public static String ALL(String feature1, String feature2)
//	{
//		OperationList list1 = getOpList(feature1);
//		OperationList list2 = getOpList(feature2);
//		OperationList result = null;
//		
//        if(list1==null || list2==null)
//		{
//			result = null;
//		}       
//		else
//		{
//			//merge
//			result = (OperationList)list1.clone();
//			for(int j=0; j<list2.size(); j++)
//			{
//				SynFeatureNode tmpNode2= (SynFeatureNode)list2.get(j);
//				if(!result.hasNode(tmpNode2.getSynFeatureName()))
//				{
//					result.add(tmpNode2);
//				}
//			}					
//		}
//		
//	
//        if(result!=null)
//        {
//        	String anonymousID = "anonymous"+(anonymousCounter++);
//        	result.setListType(OperationList.AND_LIST);
//        	opListMap.put(anonymousID, result);
//        	return anonymousID;
//        }
//        else
//        {
//        	return FeatureExtractionHandler2.NULL;
//        }
//	}
	public static String ALL (String feature1, String feature2)//	throws TokenException 
	{
		
//		if(FeatureExtractionHandler2.isIllegalUserFeature(feature1) || FeatureExtractionHandler2.isIllegalUserFeature(feature2))
//			throw new TokenException();
		
		OperationList list1 = getOpList(feature1);
		OperationList list2 = getOpList(feature2);
		OperationList result = null;
		
        if(list1==null || list2==null)
		{
			result = null;
		}       
		else
		{
			//merge
			result = (OperationList)list1.clone();
			for(int j=0; j<list2.size(); j++)
			{
				SynFeatureNode tmpNode2= (SynFeatureNode)list2.get(j);
				if(!result.hasNode(tmpNode2.getSynFeatureName()))
				{
					result.add(tmpNode2);
				}
			}					
		}
		
	
        if(result!=null)
        {
        	String anonymousID = "anonymous"+(anonymousCounter++);
        	result.setListType(OperationList.AND_LIST);
        	opListMap.put(anonymousID, result);
        	return anonymousID;
        }
        else
        {
        	return FeatureExtractionHandler2.NULL;
        }
	}
	
	public static boolean isIllegalUserFeature(String feature)
	{
//		Iterator iter = FeatureExtractionHandler2.featureSelector.getGlobalUserDefinedFeatureSetIterator();
//		while(iter.hasNext())
//		{
//			iter.next()
//		}	
		

		if(feature.trim().startsWith("U:"))
		{
			System.out.println("IllegalUserFeature  " + feature);
			Object tmp = FeatureExtractionHandler2.featureSelector.getlobalUserDefinedFeatureExpression(feature);
		
			if (tmp=="")
				return true;
			else
				return false;
		}
		
		
		return false;
	}
	
	
	private static void expandQueue(OperationList andList,int index,															
								    ArrayList<SimpleNode> smList,
								    List<PriorityQueue<SimpleNode>> queueCollection)
	{
		if(index == andList.size())
		{
			PriorityQueue<SimpleNode> q = new PriorityQueue<SimpleNode>();
			q.addAll(smList);
			queueCollection.add(q);
			return;
		}
		else
		{	
			SynFeatureNode tmpSynNode = (SynFeatureNode)andList.get(index);			
			Iterator<Integer> positionItr = tmpSynNode.getPositionIterator();		
			for(int j=0; positionItr.hasNext(); j++)
			{
				int tmpHead = positionItr.next().intValue();
				int tmpTail = tmpSynNode.getTailPosition(j);	
				ArrayList<SimpleNode> copiedSMList = (ArrayList<SimpleNode>)smList.clone();
				copiedSMList.add(new SimpleNode(tmpHead, tmpTail, tmpSynNode));
				expandQueue(andList, index+1, copiedSMList, queueCollection); 
			}	
		}
	}
	
	private static List<PriorityQueue<SimpleNode>> getANDFullPattern(OperationList andList)
	{
		if(!andList.isANDListType())
			return null;
		
		List<PriorityQueue<SimpleNode>> queueCollection = new ArrayList<PriorityQueue<SimpleNode>>();
		expandQueue(andList, 0, new ArrayList<SimpleNode>(), queueCollection);
				
		return queueCollection;
	}
	
	public static String CONCATENATE(String feature1, String feature2) // has a special semantics
	{
		OperationList list1 = getOpList(feature1);
		OperationList list2 = getOpList(feature2);
		if(!list1.isANDListType() && !list2.isANDListType())
		{
			return CONCATENATE_OR(list1, list2);
		}
		
		OperationList result = null;
		
        if(list1==null || list2==null)
		{
			result = null;
		}
		else
		{
			result = new OperationList();
			
			if(list1.isANDListType() && !list2.isANDListType())
			{
				List<PriorityQueue<SimpleNode>> list1Full = getANDFullPattern(list1);			 

				for(int i=0; i<list2.size(); i++)
				{
					SynFeatureNode n_node = (SynFeatureNode)list2.get(i);					
					SynFeatureNode synNode = new SynFeatureNode(feature1+"-"+n_node.getSynFeatureName());
					
					for(int j=0; j<list1Full.size(); j++)
					{
						PriorityQueue<SimpleNode> pq = list1Full.get(j);
						SimpleNode[] array = (SimpleNode[])pq.toArray();
						int head = array[0].head;	
						int tail = array[array.length-1].tail;
						int length = tail - head +1;						
						
						if(n_node.isDefault())
						{					
							synNode.addInstance(head, length);
						}
						else
						{
							int expectPos_n = tail+1;
							if(n_node.occurAt(expectPos_n))
							{
								int n_length = n_node.getLength(i);
								synNode.addInstance(head, length+n_length);
							}							
						}
					}
					
					if(synNode.numOfInstances() !=0)
						result.add(synNode);
				}				 				 
			}
			else if(!list1.isANDListType() && list2.isANDListType())
			{
				 List<PriorityQueue<SimpleNode>> list2Full = getANDFullPattern(list2);	
				 
					for(int i=0; i<list1.size(); i++)
					{
						SynFeatureNode m_node = (SynFeatureNode)list1.get(i);					
						SynFeatureNode synNode = new SynFeatureNode(m_node.getSynFeatureName()+"-"+feature2);
						
						for(int j=0; j<list2Full.size(); j++)
						{
							PriorityQueue<SimpleNode> pq = list2Full.get(j);
							SimpleNode[] array = (SimpleNode[])pq.toArray();
							int head = array[0].head;	
							int tail = array[array.length-1].tail;
							int length = tail - head +1;						
							
							if(m_node.isDefault())
							{					
								synNode.addInstance(head, length);
							}
							else
							{
								int m_length = m_node.getLength(i);
								int expectPos_m = head - m_length;
								if(m_node.occurAt(expectPos_m))
								{
									synNode.addInstance(expectPos_m, length+m_length);
								}							
							}
						}
						
						if(synNode.numOfInstances() !=0)
							result.add(synNode);						
					}					 
			}
			else
			{
				 List<PriorityQueue<SimpleNode>> list1Full = getANDFullPattern(list1);	
				 List<PriorityQueue<SimpleNode>> list2Full = getANDFullPattern(list2);
				 
					for(int i=0; i<list1.size(); i++)
					{
						PriorityQueue<SimpleNode> pq1= list1Full.get(i);
						SimpleNode[] array1 = (SimpleNode[])pq1.toArray();
						int head1 = array1[0].head;	
						int tail1 = array1[array1.length-1].tail;
						int length1 = tail1 - head1 +1;	
						
						SynFeatureNode synNode = new SynFeatureNode(feature1+"-"+feature2);
						
						for(int j=0; j<list2Full.size(); j++)
						{
							PriorityQueue<SimpleNode> pq2 = list2Full.get(j);
							SimpleNode[] array2 = (SimpleNode[])pq2.toArray();
							int head2 = array2[0].head;	
							int tail2 = array2[array2.length-1].tail;
							int length2 = tail2 - head2 +1;						

							if(head2 == tail1+1)
							{
								synNode.addInstance(head1, length1+length2);
							}							

						}
						
						if(synNode.numOfInstances() !=0)
							result.add(synNode);						
					}				 
			}
											
		}

        if(result!=null)
        {
        	if(result.size() !=0)
        	{
        		String anonymousID = "anonymous"+(anonymousCounter++);
        		result.setListType(OperationList.OR_LIST);
        		opListMap.put(anonymousID, result);
        		return anonymousID;
        	}
        	else
        	{
        		return FeatureExtractionHandler2.NULL;
        	}

        }
        else
        {
        	return FeatureExtractionHandler2.NULL;
        }
	}
	
	// THE VERSION FOR OR
	public static String CONCATENATE_OR(OperationList list1, OperationList list2)
	 
	{
		OperationList result = null;		
        if(list1.isANDListType()|| list2.isANDListType())
        {
        	System.err.println("CONCATENATE_OR cannot has a nesting AND inside. Return null.");
        	result = null;
        }
		else
		{
			result = new OperationList();
			//merge			
			for(int i=0; i<list1.size(); i++)
			{
				SynFeatureNode m_node = (SynFeatureNode)list1.get(i);
				for(int j=0; j<list2.size(); j++)
				{
					SynFeatureNode n_node = (SynFeatureNode)list2.get(j);
					
					//TODO
					SynFeatureNode combined = combineWithOrder(m_node, n_node);
					if(combined != null)
						result.add(combined);
				}
			
			}						
		}
        
			if(result!=null)
			{
				if(result.size() !=0)
				{
					String anonymousID = "anonymous"+(anonymousCounter++);
					result.setListType(OperationList.OR_LIST);
					opListMap.put(anonymousID, result);
					return anonymousID;
				}
				else
				{
					return FeatureExtractionHandler2.NULL;
				}
					
			}
			else
			{
				return FeatureExtractionHandler2.NULL;
			}
	}


	private static SynFeatureNode combineWithOrder(SynFeatureNode m, SynFeatureNode n)
	{
		if(m.isDefault())
		{
			if(n.isDefault())
			{
				return m;
			}
			else
			{
				return n;
			}
		}
		else
		{
			if(n.isDefault())
			{
				return m;
			}
			
			String synName = m.getSynFeatureName()+"-"+n.getSynFeatureName();
//			int synLength = m.getLength() + n.getLength();
			SynFeatureNode synNode = new SynFeatureNode(synName);
//			synNode.setLength(synLength);
			
			Iterator<Integer> itr_m = m.getPositionIterator();
//			int length_m = m.getLength();
//			int length_n = n.getLength();
			for(int i=0; itr_m.hasNext(); i++)
			{
				int pos_m = itr_m.next().intValue();
				int length_m = m.getLength(i);
				int length_n = n.getLength(i);
				
				int expectPos_n = pos_m + length_m;
				
				if(n.occurAt(expectPos_n))
				{
					synNode.addInstance(pos_m, length_m+length_n);
				}
			}
			
			if(synNode.numOfInstances() ==0)
				return null;
			else
				return synNode;
		}
	}
	
	
	
	
	public static void main(String[] args)
	{
//		String input = "1%2%abc@4";
//		String[] values = input.split("\\@");
//		for(int i =0; i<values.length; i++)
//			System.out.println(values[i]);
//		
//		for(int i=0; i<10; i++)
//		{
//			for(int j=0; j<10; j++)
//			{
//				if(j==6)
//					continue;
//				
//				System.out.println("i="+i+", j="+j);
//
//			}
//		}
		
		PriorityQueue q= new PriorityQueue();
		
		q.add(new SimpleNode(5, 6,null));
		q.add(new SimpleNode(6, 7, null));
		q.add(new SimpleNode(7,9, null));	
		
		System.out.println(q);
	}
}

class SimpleNode implements Comparable{
	int head;
	int tail;
	SynFeatureNode synFeatureNode;
	
	public SimpleNode(int head, int tail, SynFeatureNode synFeatureNode)
	{
		this.head = head;
		this.tail = tail;
		this.synFeatureNode = synFeatureNode;
	}
	
    public int compareTo(Object o){
    	if(this.tail > ((SimpleNode)o).head)
    		return 1;
    	else
    		return -1;
    	
    }
    
    public String toString()
    {
    	if(this.synFeatureNode!=null)
    		return synFeatureNode.getSynFeatureName()+"_"+head+"[h]_"+tail+"[t]";
    	else
    		return head+"[h]_"+tail+"[t]";
    }
}