/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package edu.cmu.taghelpertools.middle_layer.extractor;

import javax.swing.JOptionPane;
import java.util.*;
import edu.cmu.taghelpertools.io.*;
import edu.cmu.taghelpertools.middle_layer.*;
import edu.cmu.taghelpertools.gui.ProgressDialog;
import gnu.trove.*;

/**
 * The name of the class is probably confusing. Dont cofuse this with 
 * edu.cmu.taghelpertools.middle_layer.util.SimpleFeatureSpaceBuilder. The function
 * is different. The class may be considered to be renamed.
 * It's primary function is to read in the input data and initiate feature spaces
 * for later feature extraction. Note that the class will invokve InputFileReader
 * to actually read in the data, and a boolean argument enforcing the construction
 * of FeatureSpaceMap object associated with each data point is passed.
 * 
 * @author haochuan
 *
 */
public class FeatureSpaceMapBuilder {
	private FeatureSelector featureSelector;
	
	public Iterator getFeatureSetInterator(boolean sorting)
	{
		return this.featureSelector.getFeatureSetIterator(sorting);
	}
	
	public THashMap getFeatureMap()
	{
		return this.featureSelector.getFeatureMap();
	}
	
	public FeatureSelector getFeatureSelector()
	{
		return this.featureSelector;
	}
	   
	public int getNumSynFeatures()
	{
		return this.featureSelector.getNumGlobalUserDefiendFeature();
	}
	
	public ArrayList<DataPoint> setUp(ProgressDialog pDialog, 
									  Reader input, 
									  OptionsModel optionsModel,
									  InputFileReader inputFileReader)
	{
	        System.out.println("reading file " + input.getFilename() + "...");

	       	this.featureSelector = new FeatureSelector(optionsModel);
	       	ArrayList<DataPoint> dataPoints = new ArrayList<DataPoint>();
	        Iterator iter = null;
	        try
	        {
	        	// make sure that FeatureSapceMap will be built for each data point
	        	iter= inputFileReader.readFile(input, pDialog, featureSelector, true);
	        }
	        catch(Exception ex)
	        {
	        	//TODO exception handling
	        	ex.printStackTrace();
	        	System.exit(-1);
	        }	        
	        
	        while (iter.hasNext()) {
	            DataPoint dataPoint = (DataPoint) iter.next();
//	            textList.add(dataPoint.getText());
	            dataPoints.add(dataPoint);
	        }

//	        if (optionsModel.isRemoveRareFeatures()) {
//	            featureSelector.removeRareFeatures();
//	        }

	        optionsModel.setNominalFeatureNames(DataPoint.nominalFeatureNames);
	        optionsModel.setNominalFeatureValueSet(DataPoint.nominalFeatureValueSet);
	        System.out.println("feature space size: " + featureSelector.getFeatureSetCount());

	        // print out the dimensions/classes found in input file
	        inputFileReader.writeDims();

	        // print out number of datapoints found in the input file
	        System.out.println("found " + dataPoints.size() + " datapoints");
	        return dataPoints;
	    }	
}
