/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package edu.cmu.taghelpertools.middle_layer.extractor;
import java.util.*;

/**
 *  This class is employed by FeatureExtractionHandler2 as a supplemental data structure
 *  for handling feature extraction.  When handling expression like "ANY(foo,bar)". Two nodes 
 *  will be added into this list, one is "foo" and one is "bar". Each node is a SynFeatureNode,
 *  which further records information about position and length.
 *  
 * @author haochuan
 *
 */
public class OperationList extends ArrayList{
	public  static final String AND_LIST ="and_list";
	public  static final String OR_LIST = "or_list";	
	
	private String listType;
	
	public OperationList()
	{
		super();
		this.listType = OperationList.OR_LIST;
	}
	
	public void setListType(String listType)
	{
		this.listType = listType;
	}
	
	public String getListType()
	{
		return this.listType;
	}
	
	public boolean hasNode(String featureName)
	{
		Iterator itr = this.iterator();
		while(itr.hasNext())
		{
			SynFeatureNode tmpNode= (SynFeatureNode)itr.next();
			if(featureName.equals(tmpNode.getSynFeatureName()))
				return true;
			else
				continue;	
		}
		
		return false;
	}
	
	public boolean hasDefaultNodeOnly()
	{
		if(this.size() == 1)
		{
			SynFeatureNode tmpNode = (SynFeatureNode)this.get(0);
			if(tmpNode.isDefault())
				return true;
			else
				return false;
		}
		else
			return false;
	}
	
	
	public boolean isANDListType()
	{
		if(this.listType.equals(OperationList.AND_LIST))
			return true;
		else
			return false;
			
	}
	
	public String toString()
	{
		StringBuffer sb = new StringBuffer();
		sb.append("[");
		for(int i=0; i<this.size(); i++)
		{
			sb.append(((SynFeatureNode)this.get(i)).getSynFeatureName());
			if(i!=this.size()-1)
				sb.append(",");
		}
		sb.append("]");
		return sb.toString();
	}
}
