/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package edu.cmu.taghelpertools.middle_layer.extractor;
import java.util.*;

/**
 * An abstraction representing a node in the OperationList
 * This class is required for feature extraction handled by FeatureExtractionHandler2
 * 
 * @author haochuan
 *
 */
public class SynFeatureNode {
	public static final String DEFAULT_NODE = "default";	
	private String synFeatureName;
	private List<Integer> positionList;
//	int length = -1;
	private List<Integer> lengthList;
	boolean isOptional = false;
	
	public SynFeatureNode(String synFeatureName)
	{
		this.positionList = new ArrayList<Integer>();
		this.lengthList = new ArrayList<Integer>();
		this.synFeatureName = synFeatureName;
	}
	
	public SynFeatureNode()
	{
		this.synFeatureName = SynFeatureNode.DEFAULT_NODE;
	}	
	
	public void addInstance(int position, int length)
	{
		this.positionList.add(new Integer(position));
		this.lengthList.add(new Integer(length));
//		if(this.length == -1)
//			this.length = length;
//		else
//		{
//			if(this.length != length)
//			{
//				System.err.println("Error on initiating SynFeatureNode");
//				System.exit(0);
//			}
//		}	
	}
	
//	public void setLength(int length)
//	{
//		this.length = length;
//	}
	
	public String getSynFeatureName()
	{
		return this.synFeatureName;
	}
	
	public Iterator<Integer> getPositionIterator()
	{
		return this.positionList.iterator();
	}
	
	public Iterator<Integer> getLengthIterator()
	{
		return this.lengthList.iterator();
	}
	
	public Iterator<Integer> getTailIterator()
	{
		ArrayList l = new ArrayList();
		for(int i=0; i<this.positionList.size(); i++)
		{
			int head = this.positionList.get(i).intValue();
			int length = this.lengthList.get(i).intValue();
			l.add(new Integer(head+length -1));			
		}
		return l.iterator();
	}
	
	public int getLength(int index)
	{
		return this.lengthList.get(index).intValue();
	}
	
	public int getTailPosition(int index)
	{
		int head = this.positionList.get(index).intValue();
		int length = this.lengthList.get(index).intValue();
		return head+length -1;
	}
	
//	public int getLength()
//	{
//		return this.length;
//	}
	
	public boolean isDefault()
	{
		return this.synFeatureName.equals(SynFeatureNode.DEFAULT_NODE);
	}
	
	public void setIsOptional(boolean isOptional)
	{
		this.isOptional = isOptional;
	}
	
	public boolean isOptional()
	{
		return this.isOptional;
	}
	
	public boolean occurAt(int position)
	{
		for(int i=0; i<this.positionList.size(); i++)
		{
			if(this.positionList.get(i).intValue() == position)
				return true;
		}
		return false;
	}
	
	public int numOfInstances()
	{
		return this.positionList.size();
	}
	
}
