/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package edu.cmu.taghelpertools.middle_layer.util;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * Tester is an example of using SimpleFeatureSpaceBuilder for extracting 
 * features from given texts. This is intended to serve as an example of how to 
 * use the simple programming interface.  The source code Tester.java can be found
 * in the main folder of TagHelperTools2. 
 * 
 * 
 * @author Hao-Chuan Wang
 * 
 */
public class Tester {
	public static void main(String[] args)
	{
		ArrayList texts = new ArrayList();
		try{
			String currentLine = "";
			BufferedReader rd = new BufferedReader(new FileReader("sample_text.txt"));             
			for(int i=0; (currentLine = rd.readLine())!= null; i++)
			{
				texts.add(currentLine.trim().toLowerCase());
			}
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
           
		SimpleFeatureSpaceBuilder sf = new SimpleFeatureSpaceBuilder("-lang ger -f uni -f bi -f posbi -f ll -stop -f cns");
		ArrayList allAtts = sf.build(texts);		
		System.out.println("\n\nAttributes:\n"+allAtts);
		System.out.println("Numeric Attributes:\n"+sf.getNumericFeatures()+"\n\n");		
		
		String text = "die schon in der Familie liegen soll.";
		//get binary features for the given text
		Iterator itr = sf.getBinaryFeatureIterator(text);
		sf.printoutOptions();
		System.out.println("\nBinary attributes for the given text: ");	
		while(itr.hasNext())
		{
			System.out.println(itr.next().toString());
		}
		
		//get names of numeric features
		System.out.println("\nNumeric attributes for the given text: ");			
		ArrayList nfNames = sf.getNumericFeatures();
		ArrayList<Double> numericFeatures = sf.getNumericFeatureValues(text, nfNames);
		
		for(int i=0; i<numericFeatures.size(); i++)
		{
			System.out.println(nfNames.get(i)+"= "+numericFeatures.get(i));
		}
		
		System.out.println("\nWrite the dictionary file. ");
		sf.writeDicFile("testDic.dic");
		
		System.out.println("\nReload the dictionary file. ");		
		SimpleFeatureSpaceBuilder sf2 = new SimpleFeatureSpaceBuilder();
		ArrayList allAtts2 = sf2.build("testDic.dic");
		sf2.printoutOptions();
		System.out.println("\n\nAttributes:\n"+allAtts2);
		System.out.println("Done! ");	
		
		System.out.println("Invoking sf.getBinaryFeatureIterator ...");
		Iterator it = sf.getBinaryFeatureIterator("Die Familie faehrt in Urlaub");
		while(it.hasNext()){
			System.out.println(it.next());
		}
		System.out.println("Invoking sf2.getBinaryFeatureIterator ...");
		Iterator it2 = sf2.getBinaryFeatureIterator("Die Familie faehrt in Urlaub");
		while(it2.hasNext()){
			System.out.println(it2.next());
		}
	}
}
