package edu.cmu.taghelpertools.net;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.net.Socket;
import java.util.Properties;

import javax.swing.JOptionPane;

import edu.cmu.taghelpertools.gui.ProgressDialog;
import edu.cmu.taghelpertools.middle_layer.SwingWorker;

public class TCPUploader
{
	public String classifierName;
	
	public String userName;
	
	public String classifierPath;

	static String verifyString = "2602AXYZABCPQR==PQRABCXYZA2062";
	
	String SERVER_HOST_NAME = "localhost";
	int port = 7557;
	int uploadport = 7558;
	
	int maxBufferSize = 1 * 1024 * 1024;
	
	public String configString = "";
	
	public TCPUploader(String fullname, String path)
	{
		classifierName = fullname;
		classifierPath = path;
		
		readServerProperties();
	}
	
	public void readServerProperties() {
		
		Properties properties = new Properties();
    
    try
    {
	properties.load(new FileInputStream("ths.properties"));
}
    catch (Exception e)
{
	e.printStackTrace();
}

    SERVER_HOST_NAME = properties.getProperty("taghelperserver.serverhost");
    port = Integer.parseInt(properties.getProperty("taghelperserver.serverport"));
    uploadport = Integer.parseInt(properties.getProperty("taghelperserver.uploadserverport"));
    
	}
	
	public void doUploadandInstall()
	{
		//Upload all the files first
		doUpload(classifierPath + classifierName + ".dic", "Dictionary_Type");
		doUpload(classifierPath + classifierName + ".feature", "Feature_Type");
		doUpload(classifierPath + classifierName + ".ins", "INS_Type");
		doUpload(classifierPath + classifierName + ".nfn", "NFN_Type");
		doUpload(classifierPath + classifierName + ".nfvs", "NFVS_Type");
		doUpload(classifierPath + classifierName + ".model", "Model_Type");
		doUpload(classifierPath + classifierName + ".om", "Options_Type");
		doUpload(classifierPath + classifierName + ".udf", "UDF_Type");
		
		//Then do the Install sequence
        String message = "message-start\r";
        message += "new-classifier-start\r";
        message += (classifierName + "\r");
        message += "config-file-start\r";
        message += configString;
        message += "config-file-end\r";
        message += "new-classifiers-end\r";
        message += "message-end\r";

        System.out.println(message);

        try
        {
            Socket skt = new Socket(SERVER_HOST_NAME, port);
            
            PrintWriter out = new PrintWriter(skt.getOutputStream(), true);
            out.print(message);
            out.flush();
            
            BufferedReader in = new BufferedReader(new InputStreamReader(skt.getInputStream()));

            System.out.print("Received string: ");
            String input = "";
            while ((input = in.readLine()) != null)
            {
                System.out.println(input + "\n");
            }
            out.close();
            in.close();
        }
        catch (Exception e)
        {
        	e.printStackTrace();
        }
	}
	
	void doUpload(String filename, String filetype)
	{
        File file;
        file = new File(filename);
        
        System.out.println("Uploading: " + filename);

        try
        {
            Socket skt = new Socket(SERVER_HOST_NAME, uploadport);
            
            DataOutputStream out = new DataOutputStream(skt.getOutputStream());

            int len = 0;
            len = this.verifyString.length();
            out.writeInt(len);
            out.flush();
            
            for(int i=0; i < len; i++)
            	out.writeChar(this.verifyString.charAt(i));
            out.flush();
            System.out.println("Verification Sent");
            
            len = classifierName.length();
            out.writeInt(len);
            out.flush();
            
            for(int i=0; i < len; i++)
            	out.writeChar(classifierName.charAt(i));
            out.flush();
            System.out.println("Name Sent");
            
            len = filetype.length();
            out.writeInt(len);
            out.flush();
            
            for(int i=0; i < len; i++)
            	out.writeChar(filetype.charAt(i));
            out.flush();
            System.out.println("Type Sent");
                        
            out.writeInt((int) file.length());
            out.flush();
            System.out.println("Size Sent");

            FileInputStream fileInputStream = new FileInputStream(file);
            int bytesAvailable = fileInputStream.available();

            int bufferSize = Math.min(bytesAvailable, maxBufferSize);
            byte[] buffer = new byte[bufferSize];

            int bytesRead = fileInputStream.read(buffer, 0, bufferSize);
            int totalBytesWritten = 0;
            while (bytesRead > 0)
            {
            	out.write(buffer, 0, bytesRead);
            	out.flush();
            	totalBytesWritten += bytesRead;
            	System.out.println("Wrote: " + Integer.toString(totalBytesWritten));
                bytesAvailable = fileInputStream.available();
                bufferSize = Math.min(bytesAvailable, maxBufferSize);
                bytesRead = fileInputStream.read(buffer, 0, bufferSize);
            }
            System.out.println("Data Sent");
            out.flush();
            
            BufferedReader in = new BufferedReader(new InputStreamReader(skt.getInputStream()));
            String input = in.readLine();
            System.out.println("Received string: " + input);

            out.close();
            in.close();
        }
        catch (Exception e)
        {
        	e.printStackTrace();
        }
    }
}

