/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package edu.cmu.taghelpertools.net;


import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.Socket;
import java.net.URL;


import javax.swing.JOptionPane;

import edu.cmu.taghelpertools.io.Reader;

/**
 * Uploads the input file to the TagHelper Server. 
 * @author Rohit Kumar
 */

public class UploadModel {

    public static String SERVER_HOST_NAME = "";
    public static final String SERVER_HOST_FILE_NAME = "serverhost.txt";

    static {
        Reader reader = new Reader(UploadModel.SERVER_HOST_FILE_NAME);
        String input = "";
        while ((input = reader.readLine()) != null) {
            UploadModel.SERVER_HOST_NAME += input.trim();
        }
        reader.close();
        System.out.println("set SERVER_HOST_NAME = " + UploadModel.SERVER_HOST_NAME);
    }

    public static void sendFile(File file) throws UnsupportedEncodingException {
        String boundary = "-+-+-*ThisBoundaryStringCanBeAnythingWhichDoesNotAppearInTheMessage*-+-+-";
        String twoHyphens = "--";
        String lineEnd = "\r\n";
        int maxBufferSize = 1 * 1024 * 1024;

        // This is how to Talk to a HTTP SERVER
        try {
            URL url;
            url = new URL("http://why2.lrdc.pitt.edu/cgi-bin/tutalk/uploadmodels.cgi");
            try {
                HttpURLConnection httpconn;
                httpconn = (HttpURLConnection) url.openConnection();

                httpconn.setRequestMethod("POST");
                httpconn.setRequestProperty("Connection", "Keep-Alive");
                httpconn.setDoOutput(true);
                httpconn.setUseCaches(false);
                httpconn.setRequestProperty("Accept-Charset", "iso-8859-1,*,utf-8");
                httpconn.setRequestProperty("Accept-Language", "en");
                httpconn.setRequestProperty("Content-type", "multipart/form-data; boundary=" + boundary);
                httpconn.setInstanceFollowRedirects(false);
                httpconn.setDoInput(true);


                DataOutputStream outStream = new DataOutputStream(httpconn.getOutputStream());


                outStream.writeBytes(twoHyphens + boundary + lineEnd);
                outStream.writeBytes("Content-Disposition: form-data; type=\"file\"; name=\"filename\";" + " filename=\"" + file.getName() + "\"" + lineEnd);
                outStream.writeBytes(lineEnd);

                FileInputStream fileInputStream = new FileInputStream(file);
                int bytesAvailable = fileInputStream.available();

                int bufferSize = Math.min(bytesAvailable, maxBufferSize);
                byte[] buffer = new byte[bufferSize];

                int bytesRead = fileInputStream.read(buffer, 0, bufferSize);
                while (bytesRead > 0) {
                    outStream.write(buffer, 0, bufferSize);

                    bytesAvailable = fileInputStream.available();
                    bufferSize = Math.min(bytesAvailable, maxBufferSize);
                    bytesRead = fileInputStream.read(buffer, 0, bufferSize);
                }

                outStream.writeBytes(lineEnd);
                outStream.writeBytes(twoHyphens + boundary + twoHyphens + lineEnd);

                fileInputStream.close();
                outStream.flush();
                outStream.close();

                System.out.println(httpconn.getResponseMessage());
                System.out.println("Reading from http now");

                InputStream inn = httpconn.getInputStream();
                BufferedReader in = new BufferedReader(new InputStreamReader(inn));
                String inputLine;
                while ((inputLine = in.readLine()) != null) {
                    System.out.println(inputLine + "\n");
                }
                inn.close();
                in.close();
                httpconn.disconnect();
            } catch (IOException e1) {
                System.out.println("Some stupid IO Exception it says!");
            }
        } catch (MalformedURLException e3) {
            System.out.println("Some stupid malformed URL Exception it says!");
        }
    }

    public static void refreshServer(String configfilename) {
        String message = "message-start\r";
        message += "new-classifiers-start\r";
        message += configfilename + "\r";
        message += "new-classifiers-end\r";
        message += "message-end\r";

        System.out.println(message);

        try {
            Socket skt = new Socket(UploadModel.SERVER_HOST_NAME, 7557);
            PrintWriter out = new PrintWriter(skt.getOutputStream(), true);
            out.print(message);
            out.flush();
            System.out.println("sent message!");
            BufferedReader in = new BufferedReader(new InputStreamReader(skt.getInputStream()));
            System.out.print("Received string: ");
            String input = "";
            while ((input = in.readLine()) != null) {
                System.out.println(input + "\n");
            }
            out.close();
            in.close();
        } catch (Exception e) {
            JOptionPane.showMessageDialog(null, "Could not refresh server due to error: " + e.getMessage(), "TERMINAL ERROR", JOptionPane.ERROR_MESSAGE);
            System.exit(0);
        }
    }

    public static void main(String args[]) {
    }
}
