/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package edu.cmu.taghelpertools.net;


import java.io.File;
import java.io.UnsupportedEncodingException;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.UIManager;


import edu.cmu.taghelpertools.gui.ExampleFileFilter;
import edu.cmu.taghelpertools.gui.ProgressDialog;
import edu.cmu.taghelpertools.middle_layer.Driver;
import edu.cmu.taghelpertools.middle_layer.SwingWorker;



public class UploadModelToServer {

    static {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) {
            try {
                UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
            } catch (Exception e1) {
                e1.printStackTrace();
                System.exit(1);
            }
        }
    }

    public static void main(String args[]) {
        UploadModelToServer uploadModel = new UploadModelToServer();
        uploadModel.run();
    }


    public void run() {
        File inputFile = this.getFile();
        if (inputFile != null) {
            this.uploadModel(inputFile);
        } else {
            JOptionPane.showMessageDialog(null, "Could not load file.", "TERMINAL ERROR", JOptionPane.ERROR_MESSAGE);
            System.exit(0);
        }
    }

    public void uploadModel(File file) {
        UploadFileThread uploadThread = new UploadFileThread(file);
        uploadThread.start();
    }

    public class UploadFileThread extends SwingWorker {
        private File file;
        private ProgressDialog pDialog;

        public UploadFileThread(File file) {
            this.file = file;
            this.pDialog = new ProgressDialog(null, "loading " + file.getName() + " to server");
        }

        public Object construct() {
            try {
                pDialog.setLabel("loading...");
                pDialog.setIsIndeterminate(true);
                UploadModel.sendFile(file);
                try {
                    Thread.currentThread().sleep(5000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                // String currentDirName = System.getProperty("user.dir");
                // File currentDir = new File(currentDirName);
                // File parentDir = currentDir.getParentFile();
                // File tutalkDir = new File(parentDir.getAbsolutePath() + "/" +
                // "TuTalkGUI/");
                // if ((!tutalkDir.exists()) || (!tutalkDir.isDirectory())) {
                // tutalkDir.mkdir();
                // }

                // File config = new File(tutalkDir.getAbsolutePath() + "/" +
                // "config/");
                // if ((config.exists() && config.isDirectory())) {
                String username = System.getProperty("user.name");
                String configName = "config_" + username + ".txt";
                UploadModel.refreshServer(configName);

                // } else {
                // JOptionPane.showMessageDialog(null, "Could not upload file: "
                // + file.getName(), "TERMINAL ERROR",
                // JOptionPane.ERROR_MESSAGE);
                // System.exit(0);
                // }
            } catch (UnsupportedEncodingException e) {
                JOptionPane.showMessageDialog(null, "Could not upload file: " + file.getName(), "TERMINAL ERROR", JOptionPane.ERROR_MESSAGE);
                System.exit(0);
            }
            JOptionPane.showMessageDialog(null, "DONE!", "Information", JOptionPane.INFORMATION_MESSAGE);
            pDialog.dispose();
            return null;
        }
    }

    public File getFile() {
        File inputFile = null;
        ExampleFileFilter filter = new ExampleFileFilter();
        filter.addExtension("model");
        JFileChooser fc = new JFileChooser();
        fc.setFileFilter(filter);
        fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fc.setDialogTitle("Choose .mod file to upload to server");
        fc.setCurrentDirectory(new File(System.getProperty("user.dir")));
        int returnVal = fc.showOpenDialog(null);
        if (returnVal != JFileChooser.APPROVE_OPTION) {
            System.exit(0);
        } else {
            inputFile = fc.getSelectedFile();
            if (!inputFile.isFile()) {
                return null;
            }
        }
        return inputFile;
    }


}