/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package edu.cmu.taghelpertools.tokenizer;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.HashMap;
import edu.cmu.taghelpertools.middle_layer.OptionsModel;

/**
 * This singleton class reads in the contents of "default.stp" an unstemmed list
 * of stopwords and maintains this list in a HashMap. Other modules call this
 * isStopword(String s) method to check if a term is found in the list of known
 * stopwords (default.stp).
 * 
 * NOTE: changes have been made by Hao-Chuan Wang, default.stp--> <language>.stp
 * @author Jaime Arguello
 * 
 */
public class StopwordManager {

    private HashMap stopwordMap;
    private static StopwordManager instance = null; //new StopwordManager();
    private static String currentPath = "none";

    private StopwordManager(String stopwordFilename) {
        stopwordMap = new HashMap();
        BufferedReader in = null;
        String tempString;
        try {
            in = new BufferedReader(new FileReader(stopwordFilename));
        } catch (FileNotFoundException fnfe) {
            fnfe.printStackTrace();
            System.exit(0);
        }
        try {
            while (true) {
                tempString = in.readLine().trim();//.toLowerCase();
                stopwordMap.put(tempString, null);
//                System.out.println(tempString+", "+tempString.hashCode());                
            }
        } 
        catch (Exception e) 
        {
        }          
    }

    public static StopwordManager getInstance(String filePath) {
    	if(StopwordManager.instance == null || !currentPath.equals(filePath))
    	{
    		StopwordManager.instance = new StopwordManager(filePath);
    		currentPath = filePath;
    		return StopwordManager.instance;
    	}
    	else
    	{
    		return StopwordManager.instance;
    	}
    }
    
    public static StopwordManager getInstance(OptionsModel optionsModel)
    {
    	return getInstance(optionsModel.getStopwordListPath());
    }

    public HashMap getStopWordMap() {
        return stopwordMap;
    }

    /**
     * Checks if the input word is a stopword.
     * 
     * @param input
     *            String to be tested against list of known stopwords
     * @return true if input is a stopword, false otherwise
     */
    public boolean isStopword(String input) {
        if (stopwordMap.containsKey(input))
            return true;
        else
            return false;
    }

    public static void main(String args[]) {
        StopwordManager myStopwordManager = StopwordManager.getInstance("etc/german.stp");
        System.out.println(myStopwordManager.isStopword("P"));
        System.out.println(myStopwordManager.isStopword("alle"));
    }
}
