/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package edu.cmu.taghelpertools.tokenizer;

/**
 * Each Token has a surface string and an index string. The surface string is
 * the exact same string that is originally read in. The index string depends on
 * the options set by the user. If the user is stemming, the index string is the
 * stem, while the surface form is the original string. If the user is removing
 * stopwords, and this word is a stopword, then the index string is the filler "<EMPTY>"
 * and the surface form is the original string.
 * 
 * @author Jaime Arguello
 * 
 */
public class Token {

	/**
	 * Flag used as the index string of stopwords (when the user opts to remove
	 * stopwords)
	 */
	public final static String EMPTY = "<EMPTY>";

	private String surfaceToken;
	private String indexToken;
	private String pos;

	public Token(String indexToken, String surfaceToken) {
		this.surfaceToken = surfaceToken;
		this.indexToken = indexToken;
	}

	public String getSurfaceToken() {
		return surfaceToken;
	}

	public String getIndexToken() {
		return indexToken;
	}

	public String toString() {
		return "index: " + indexToken + ", " + "surface: " + surfaceToken;
	}

	public String getPos() {
		return pos;
	}

	public void setPos(String pos) {
		this.pos = pos;
	}
}