/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package edu.cmu.taghelpertools.tokenizer;

import edu.cmu.taghelpertools.middle_layer.OptionsModel;

public class Tokenizer {
	public static final int LANG_ENGLISH = 0;
	public static final int LANG_GERMAN = 1;
	public static final int LANG_CHINESE = 2;
	public static final int LANG_SPANISH = 3;
	
	public static final String[] LANGS = {"English", "German", "Chinese", "Spanish"};
	public static final String[] STPS = {"etc/english.stp", "etc/german.stp", "etc/chinese.stp", "etc/spanish.stp"};
    private static AbstractTokenizer instance = null;
    private static String currentLang = "none";
    
    public static AbstractTokenizer getInstance(OptionsModel optionsModel) 
    {
    	return getInstance(optionsModel.getLanguage());
    }

    public static AbstractTokenizer getInstance(String lang) {
    	if(Tokenizer.instance == null || !lang.equals(currentLang))
    	{
            if(lang.equals(LANGS[Tokenizer.LANG_ENGLISH]))
    		{
    			Tokenizer.instance = new EnglishTokenizer();
    		}
    		else if(lang.equals(LANGS[Tokenizer.LANG_GERMAN]))
    		{
    			Tokenizer.instance = new GermanTokenizer();
    		}
    		else if(lang.equals(LANGS[Tokenizer.LANG_CHINESE]))
    		{   
    			Tokenizer.instance = new ChineseTokenizer();
    		}
    		else if(lang.equals(LANGS[Tokenizer.LANG_SPANISH]))
    		{   
    			Tokenizer.instance = new SpanishTokenizer();
    		}
    		else
    		{
    			System.err.println("Failed to initiate language-specific tokenizer. " +
		                           "No tokenizer implementation available for "+lang);
    			System.exit(1);		
    		}
            currentLang = lang;
    		return Tokenizer.instance;
    	}
    	else
    	{
    		return Tokenizer.instance;
    	}
    }
}
